@@ -67,26 +67,27 @@ cfg_put_str (int fh, char *var, char *value)
    write (fh, buf, strlen (buf));
 }
 
-static void
-cfg_put_int (int fh, int value, char *var)
+void
+cfg_put_int (int fh, unsigned long value, char *var)
 {
    char buf[400];
 
    if (value == -1)
       value = 1;
 
-   snprintf (buf, sizeof buf, "%s= %d\n", var, value);
+   snprintf (buf, sizeof buf, "%s= %lu\n", var, value);
    write (fh, buf, strlen (buf));
 }
 
-static int
+unsigned long
 cfg_get_int (char *cfg, char *var)
 {
    char str[128];
 
-   cfg_get_str (cfg, var, str);
+   if (!cfg_get_str (cfg, var, str))
+      return 0;
 
-   return atoi (str);
+   return atol (str);
 }
 
 char *
@@ -158,6 +159,9 @@ load_config (void)
          cfg_get_str (cfg, "dccdir ", prefs.dccdir);
          cfg_get_str (cfg, "doubleclickuser ", prefs.doubleclickuser);
          cfg_get_str (cfg, "bluestring ", prefs.bluestring);
+         cfg_get_str (cfg, "dnsprogram ", prefs.dnsprogram);
+         cfg_get_str (cfg, "hostname ", prefs.hostname);
+         cfg_get_str (cfg, "ipaddress ", prefs.ipaddress);
 
          /* booleans */
          prefs.autosave = cfg_get_int (cfg, "autosave ");
@@ -179,6 +183,7 @@ load_config (void)
          prefs.autodccsend = cfg_get_int (cfg, "autodccsend ");
          prefs.noautoopendccsendwindow = cfg_get_int (cfg, "noautoopendccsendwindow ");
          prefs.noautoopendccrecvwindow = cfg_get_int (cfg, "noautoopendccrecvwindow ");
+         prefs.noautoopendccchatwindow = cfg_get_int (cfg, "noautoopendccchatwindow ");
          prefs.transparent = cfg_get_int (cfg, "transparent ");
          prefs.tint = cfg_get_int (cfg, "tint ");
          prefs.zvt = cfg_get_int (cfg, "zvt ");
@@ -190,10 +195,6 @@ load_config (void)
          prefs.fastdccsend = cfg_get_int (cfg, "fastdccsend ");
          prefs.skipserverlist = cfg_get_int (cfg, "skipserverliset ");
          prefs.filterbeep = cfg_get_int (cfg, "filterbeep ");
-         prefs.bg_color = cfg_get_int (cfg, "bg_color ");
-         prefs.fg_color = cfg_get_int (cfg, "fg_color ");
-         prefs.dialog_bg_color = cfg_get_int (cfg, "bg_dialog_color ");
-         prefs.dialog_fg_color = cfg_get_int (cfg, "fg_dialog_color ");
          prefs.dccwithnick = cfg_get_int (cfg, "dccwithnick ");
          prefs.hilitenotify = cfg_get_int (cfg, "hilight_notify ");
          prefs.tabsatbottom = cfg_get_int (cfg, "tabsatbottom ");
@@ -202,21 +203,23 @@ load_config (void)
          prefs.logging = cfg_get_int (cfg, "logging ");
          prefs.newtabstofront = cfg_get_int (cfg, "newtabs_to_front ");
          prefs.hidever = cfg_get_int (cfg, "hide_version ");
-         prefs.get_ip_from_server = cfg_get_int (cfg, "get_ip_from_server ");
          prefs.panelize_hide = cfg_get_int (cfg, "panelize_hide ");
          prefs.panel_vbox = cfg_get_int (cfg, "panel_vbox ");
          prefs.raw_modes = cfg_get_int (cfg, "raw_modes ");
          prefs.no_server_logs = cfg_get_int (cfg, "no_server_logs ");
          prefs.show_away_once = cfg_get_int (cfg, "show_away_once ");
-         prefs.emacs_key_bindings = cfg_get_int (cfg, "emacs_key_bindings ");
+         prefs.show_away_message = cfg_get_int (cfg, "show_away_message ");
+         prefs.nouserhost = cfg_get_int (cfg, "nouserhost ");
+         prefs.autosave_url = cfg_get_int (cfg, "autosaveurl ");
+         prefs.detecthost = cfg_get_int (cfg, "detecthost ");
+         prefs.detectip = cfg_get_int (cfg, "detectip ");
+         prefs.ip_from_server = cfg_get_int (cfg, "ip_from_server ");
+	   prefs.use_server_tab = cfg_get_int (cfg, "use_server_tab ");
+	   prefs.autoresume = cfg_get_int (cfg, "auto_resume ");
 
          /* numbers */
          prefs.bufsize = cfg_get_int (cfg, "buffer_size ");
          prefs.notify_timeout = cfg_get_int (cfg, "notify_timeout ");
-         prefs.bg_color = cfg_get_int (cfg, "bg_color ");
-         prefs.fg_color = cfg_get_int (cfg, "fg_color ");
-         prefs.dialog_bg_color = cfg_get_int (cfg, "bg_dialog_color ");
-         prefs.dialog_fg_color = cfg_get_int (cfg, "fg_dialog_color ");
          prefs.nu_color = cfg_get_int (cfg, "nu_color ");
          prefs.dcctimeout = cfg_get_int (cfg, "dcc_timeout ");
          prefs.dccstalltimeout = cfg_get_int (cfg, "dcc_stall_timeout ");
@@ -227,6 +230,9 @@ load_config (void)
          prefs.zvt_lines = cfg_get_int (cfg, "zvt_lines ");
          prefs.dccpermissions = cfg_get_int (cfg, "dcc_permissions ");
          prefs.bt_color = cfg_get_int (cfg, "bt_color ");
+	   prefs.recon_delay = cfg_get_int (cfg, "reconnect_delay ");
+	   prefs.bantype = cfg_get_int (cfg, "ban_type ");
+
          free (cfg);
       }
       close (fh);
@@ -243,18 +249,24 @@ load_config (void)
                                         "  create a User Account and use that to login.\n\n"
                                         "* The default DCC receive directory is your\n"
                                         "  home dir, you should change this at some stage.");
-	   gtk_signal_connect(GTK_OBJECT(dialog), "destroy", gtkutil_null_this_var, &dialog);
-         while (dialog) gtk_main_iteration ();
+         gtk_signal_connect (GTK_OBJECT (dialog), "destroy", gtkutil_null_this_var, &dialog);
+         while (dialog)
+            gtk_main_iteration ();
       }
 #endif
       if (!am_root)
       {
          dialog = gtkutil_simpledialog ("The default DCC receive directory is your\n"
-                                        "home dir, you should change this at some stage.");	
-	   gtk_signal_connect(GTK_OBJECT(dialog), "destroy", gtkutil_null_this_var, &dialog);
-         while (dialog) gtk_main_iteration ();
+                                        "home dir, you should change this at some stage.");
+         gtk_signal_connect (GTK_OBJECT (dialog), "destroy", gtkutil_null_this_var, &dialog);
+         while (dialog)
+            gtk_main_iteration ();
       }
       /* put in default values */
+#ifdef USE_GNOME
+      prefs.zvt = 1;
+      prefs.dialog_zvt = 1;
+#endif
       prefs.fastdccsend = 1;
       prefs.autosave = 1;
       prefs.autodialog = 1;
@@ -263,8 +275,6 @@ load_config (void)
       prefs.tabchannels = 1;
       prefs.privmsgtab = 1;
       prefs.nickcompletion = 1;
-      prefs.bg_color = 1;
-      prefs.dialog_bg_color = 1;
       prefs.nu_color = 4;
       prefs.bt_color = 8;
       prefs.dccpermissions = 0600;
@@ -272,6 +282,10 @@ load_config (void)
       prefs.bufsize = 32768;
       prefs.mainwindow_width = 599;
       prefs.mainwindow_height = 422;
+      prefs.dcctimeout = 240;
+      prefs.dccstalltimeout = 240;
+      prefs.autosave_url = 1;
+	prefs.notify_timeout = 15000;
       strcpy (prefs.nick1, username);
       strcpy (prefs.nick2, username);
       strcat (prefs.nick2, "_");
@@ -282,7 +296,7 @@ load_config (void)
       strcpy (prefs.dccdir, g_get_home_dir ());
       strcpy (prefs.doubleclickuser, "/QUOTE WHOIS %s");
       strcpy (prefs.awayreason, "I'm busy");
-      strcpy (prefs.quitreason, "X-Chat!");
+      strcpy (prefs.quitreason, "[x]chat");
       strcpy (prefs.font_normal, "-misc-fixed-medium-r-semicondensed-*-*-100-*-*-c-*-iso8859-1");
       strcpy (prefs.font_bold, "-misc-fixed-bold-r-semicondensed-*-*-100-*-*-c-*-iso8859-1");
       strcpy (prefs.dialog_font_normal, prefs.font_normal);
@@ -294,6 +308,8 @@ load_config (void)
       prefs.mainwindow_height = 106;
    if (prefs.mainwindow_width < 116)
       prefs.mainwindow_width = 116;
+   if (!prefs.dnsprogram[0])
+      strcpy (prefs.dnsprogram, "host");
 }
 
 int
@@ -326,6 +342,9 @@ save_config (void)
       cfg_put_str (fh, "dccdir ", prefs.dccdir);
       cfg_put_str (fh, "doubleclickuser ", prefs.doubleclickuser);
       cfg_put_str (fh, "bluestring ", prefs.bluestring);
+      cfg_put_str (fh, "dnsprogram ", prefs.dnsprogram);
+      cfg_put_str (fh, "hostname ", prefs.hostname);
+      cfg_put_str (fh, "ipaddress ", prefs.ipaddress);
 
       /* booleans */
       cfg_put_int (fh, prefs.autosave, "autosave ");
@@ -347,6 +366,7 @@ save_config (void)
       cfg_put_int (fh, prefs.autodccsend, "autodccsend ");
       cfg_put_int (fh, prefs.noautoopendccsendwindow, "noautoopendccsendwindow ");
       cfg_put_int (fh, prefs.noautoopendccrecvwindow, "noautoopendccrecvwindow ");
+      cfg_put_int (fh, prefs.noautoopendccchatwindow, "noautoopendccchatwindow ");
       cfg_put_int (fh, prefs.transparent, "transparent ");
       cfg_put_int (fh, prefs.tint, "tint ");
       cfg_put_int (fh, prefs.zvt, "zvt ");
@@ -366,21 +386,23 @@ save_config (void)
       cfg_put_int (fh, prefs.logging, "logging ");
       cfg_put_int (fh, prefs.newtabstofront, "newtabs_to_front ");
       cfg_put_int (fh, prefs.hidever, "hide_version ");
-      cfg_put_int (fh, prefs.get_ip_from_server, "get_ip_from_server ");
       cfg_put_int (fh, prefs.panelize_hide, "panelize_hide ");
       cfg_put_int (fh, prefs.panel_vbox, "panel_vbox ");
       cfg_put_int (fh, prefs.raw_modes, "raw_modes ");
       cfg_put_int (fh, prefs.no_server_logs, "no_server_logs ");
       cfg_put_int (fh, prefs.show_away_once, "show_away_once ");
-      cfg_put_int (fh, prefs.emacs_key_bindings, "emacs_key_bindings ");
+      cfg_put_int (fh, prefs.show_away_message, "show_away_message ");
+      cfg_put_int (fh, prefs.nouserhost, "nouserhost ");
+      cfg_put_int (fh, prefs.autosave_url, "autosaveurl ");
+      cfg_put_int (fh, prefs.detecthost, "detecthost ");
+      cfg_put_int (fh, prefs.detectip, "detectip ");
+      cfg_put_int (fh, prefs.ip_from_server, "ip_from_server ");
+	cfg_put_int (fh, prefs.use_server_tab, "use_server_tab ");
+	cfg_put_int (fh, prefs.autoresume, "auto_resume ");
 
       /* numbers */
       cfg_put_int (fh, prefs.bufsize, "buffer_size ");
       cfg_put_int (fh, prefs.notify_timeout, "notify_timeout ");
-      cfg_put_int (fh, prefs.bg_color, "bg_color ");
-      cfg_put_int (fh, prefs.fg_color, "fg_color ");
-      cfg_put_int (fh, prefs.dialog_bg_color, "bg_dialog_color ");
-      cfg_put_int (fh, prefs.dialog_fg_color, "fg_dialog_color ");
       cfg_put_int (fh, prefs.nu_color, "nu_color ");
       cfg_put_int (fh, prefs.dcctimeout, "dcc_timeout ");
       cfg_put_int (fh, prefs.dccstalltimeout, "dcc_stall_timeout ");
@@ -391,6 +413,8 @@ save_config (void)
       cfg_put_int (fh, prefs.zvt_lines, "zvt_lines ");
       cfg_put_int (fh, prefs.dccpermissions, "dcc_permissions ");
       cfg_put_int (fh, prefs.bt_color, "bt_color ");
+	cfg_put_int (fh, prefs.recon_delay, "reconnect_delay ");
+	cfg_put_int (fh, prefs.bantype, "ban_type ");
 
       close (fh);
       return 1;
