@@ -16,7 +16,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include "style.h"
 #include "xchat.h"
 #include "gtkutil.h"
 #include <time.h>
@@ -30,9 +29,9 @@
 #include <zvt/zvtterm.h>
 #endif
 
-extern gushort rxvt_red[];
-extern gushort rxvt_blu[];
-extern gushort rxvt_grn[];
+extern gushort zvt_red[];
+extern gushort zvt_blu[];
+extern gushort zvt_grn[];
 extern GSList *sess_list;
 extern struct xchatprefs prefs;
 extern GdkFont *dialog_font_normal;
@@ -52,9 +51,11 @@ extern void createmenus (void *app, struct session *sess);
 #else
 extern GtkWidget *createmenus (struct session *sess);
 #endif
+extern void X_session (GtkWidget * button, struct session *sess);
+extern int maingui_pagetofront (int page);
 extern void zvt_clear (GtkWidget * wid);
 extern void gui_set_title (struct session *sess);
-extern void menu_popup (struct session *sess, GdkEventButton * event, char *nick);
+extern void menu_popup (struct session *sess, GdkEventButton * event, char *nick, char *);
 extern void show_and_unfocus (GtkWidget * wid);
 extern void term_change_pos (GtkWidget * widget);
 extern void add_tip (GtkWidget * wid, char *text);
@@ -65,7 +66,6 @@ extern int tcp_send_len (struct server *serv, char *buf, int len);
 extern int tcp_send (struct server *serv, char *buf);
 extern int handle_command (char *cmd, struct session *sess, int history, int);
 extern unsigned char *strip_color (unsigned char *text);
-extern void end_logging (int fd);
 extern int open_log_file (char *servname, char *channame);
 struct session *new_dialog (struct session *sess);
 extern void zvt_set_style (GtkWidget * zvt, char *bg_pic, int trans, int tint,
@@ -76,6 +76,10 @@ extern void handle_inputgad (GtkWidget * igad, struct session *sess);
 extern gint kill_session_callback (GtkWidget * win, struct session *sess);
 extern void gui_new_tab (GtkWidget * widget, GtkNotebookPage * nbpage, guint page);
 extern void focus_in (GtkWindow * win, GtkWidget * wid, struct session *sess);
+extern gint maingui_textgad_clicked_zvt (GtkWidget * wid, GdkEventButton * even, struct session *sess);
+extern void maingui_textgad_clicked_gtk (GtkWidget * wid, GdkEventButton * even, struct session *sess);
+int key_handle_key_press (GtkWidget, GdkEventKey, gpointer);
+
 
 #ifdef USE_GNOME
 void
@@ -134,9 +138,10 @@ dialog_change_nick (struct server *serv, char *nick, char *newnick)
 void
 dialog_whois (GtkWidget * wid, struct session *sess)
 {
-   char tbuf[256];
-   snprintf (tbuf, sizeof (tbuf), "WHOIS %s\r\n", sess->channel);
+   char *tbuf = malloc(strlen(sess->channel) + 9);
+   sprintf (tbuf, "WHOIS %s\r\n", sess->channel);
    tcp_send (sess->server, tbuf);
+   free (tbuf);
 }
 
 void
@@ -156,18 +161,10 @@ dialog_chat (GtkWidget * wid, struct session *sess)
 void
 dialog_ping (GtkWidget * wid, struct session *sess)
 {
-   if (sess->server->connected)
-   {
-      char tbuf[256];
-      unsigned long tim;
-      struct timeval timev;
-
-      gettimeofday (&timev, 0);
-      tim = (timev.tv_sec - 50000) * 1000000 + timev.tv_usec;
-
-      snprintf (tbuf, sizeof (tbuf), "PRIVMSG %s :\001PING %lu\001\r\n", sess->channel, tim);
-      tcp_send (sess->server, tbuf);
-   }
+   char *cmd = malloc (strlen(sess->channel) + 7);
+   sprintf (cmd, "/ping %s", sess->channel);
+   handle_command (cmd, sess, FALSE, FALSE);
+   free (cmd);
 }
 
 void
@@ -214,15 +211,6 @@ dialog_notify_kill (struct server *serv)
    }
 }
 
-void
-dialog_click (GtkWidget * widget, GdkEventButton * even, struct session *sess)
-{
-   if (!even)
-      return;
-   if (even->button == 3)
-      menu_popup (sess, even, sess->channel);
-}
-
 #ifdef USE_GNOME
 static GtkTargetEntry dnd_targets[] =
 {
@@ -290,20 +278,20 @@ open_dialog_window (struct session *sess)
    gtk_widget_show (hbox);
 
 #ifdef USE_GNOME
-   wid = gtkutil_stock_button (sess->window, GNOME_STOCK_BUTTON_CANCEL,
-                               0, gtkutil_destroy, sess->window, 0);
+   wid = gtkutil_button (sess->window, GNOME_STOCK_BUTTON_CANCEL,
+                         0, X_session, sess, 0);
    gtk_box_pack_start (GTK_BOX (hbox), wid, 0, 0, 0);
    gtk_widget_show (wid);
 #else
    wid = gtk_button_new_with_label ("X");
    gtk_box_pack_start (GTK_BOX (hbox), wid, 0, 0, 0);
    gtk_signal_connect (GTK_OBJECT (wid), "clicked",
-                       GTK_SIGNAL_FUNC (gtkutil_destroy), sess->window);
+                       GTK_SIGNAL_FUNC (X_session), sess);
    gtk_widget_show (wid);
 #endif
    add_tip (wid, "Close Dialog");
 #ifdef USE_GNOME
-   wid = gtkutil_stock_button (sess->window, GNOME_STOCK_BUTTON_UP,
+   wid = gtkutil_button (sess->window, GNOME_STOCK_BUTTON_UP,
            0, relink_window, sess, 0);
    gtk_box_pack_start (GTK_BOX (hbox), wid, 0, 0, 0);
 #else
@@ -316,7 +304,7 @@ open_dialog_window (struct session *sess)
    gtk_widget_show (wid);
 
 #ifdef USE_PANEL
-   wid = gtkutil_stock_button (sess->window, GNOME_STOCK_BUTTON_DOWN,
+   wid = gtkutil_button (sess->window, GNOME_STOCK_BUTTON_DOWN,
         0, maingui_panelize, sess, 0);
    gtk_box_pack_start (GTK_BOX (hbox), wid, 0, 0, 0);
    add_tip (wid, "Panelize");
@@ -367,10 +355,6 @@ open_dialog_window (struct session *sess)
    {
       sess->zvt = TRUE;
       sess->textgad = zvt_term_new ();
-      gtk_signal_connect (GTK_OBJECT (sess->textgad), "key_press_event",
-                          GTK_SIGNAL_FUNC (textgad_get_focus_cb), sess);
-      gtk_signal_connect (GTK_OBJECT (sess->textgad), "button_press_event",
-                          GTK_SIGNAL_FUNC (dialog_click), sess);
       if (sess->is_tab)
          gtk_signal_connect_object (GTK_OBJECT (main_window), "configure_event",
                                     GTK_SIGNAL_FUNC (term_change_pos),
@@ -382,14 +366,19 @@ open_dialog_window (struct session *sess)
       gtk_widget_set_usize (sess->textgad,
          prefs.mainwindow_width - 115,
        prefs.mainwindow_height - 105);
+      gtk_signal_connect (GTK_OBJECT (sess->textgad), "key_press_event",
+                          GTK_SIGNAL_FUNC (textgad_get_focus_cb), sess);
+      gtk_signal_connect (GTK_OBJECT (sess->textgad), "button_press_event",
+                          GTK_SIGNAL_FUNC (maingui_textgad_clicked_zvt), sess);
       zvt_term_set_scrollback ((ZvtTerm *) sess->textgad, prefs.zvt_lines);
+	zvt_term_set_blink ((ZvtTerm *) sess->textgad, FALSE);
       zvt_set_style (sess->textgad, prefs.background_dialog, prefs.dialog_transparent,
                      prefs.dialog_tint, dialog_font_normal, dialog_font_bold);
       gtk_container_add (GTK_CONTAINER (hbox), sess->textgad);
-      gtk_widget_show (sess->textgad);
+      show_and_unfocus (sess->textgad);
 
       sess->vscrollbar = gtk_vscrollbar_new (((ZvtTerm *) sess->textgad)->adjustment);
-      gtk_box_pack_start (GTK_BOX (hbox), sess->vscrollbar, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (hbox), sess->vscrollbar, FALSE, FALSE, 1);
       show_and_unfocus (sess->vscrollbar);
    } else
    {
@@ -398,7 +387,7 @@ open_dialog_window (struct session *sess)
       gtk_signal_connect (GTK_OBJECT (sess->textgad), "key_press_event",
                           GTK_SIGNAL_FUNC (textgad_get_focus_cb), sess);
       gtk_signal_connect (GTK_OBJECT (sess->textgad), "button_press_event",
-                          GTK_SIGNAL_FUNC (dialog_click), sess);
+                          GTK_SIGNAL_FUNC (maingui_textgad_clicked_gtk), sess);
       gtk_widget_set_style (sess->textgad, dialogwin_style);
       gtk_text_set_word_wrap (GTK_TEXT (sess->textgad), TRUE);
       gtk_container_add (GTK_CONTAINER (hbox), sess->textgad);
@@ -410,8 +399,8 @@ open_dialog_window (struct session *sess)
 #endif
       gtk_widget_show (sess->textgad);
 
-      wid = gtk_vscrollbar_new (GTK_TEXT (sess->textgad)->vadj);
-      gtk_box_pack_start (GTK_BOX (hbox), wid, FALSE, FALSE, 0);
+	wid = sess->vscrollbar = gtk_vscrollbar_new (GTK_TEXT (sess->textgad)->vadj);
+      gtk_box_pack_start (GTK_BOX (hbox), wid, FALSE, FALSE, 1);
       gtk_widget_show (wid);
 #ifdef USE_GNOME
    }
@@ -420,11 +409,11 @@ open_dialog_window (struct session *sess)
    gtk_widget_realize (sess->textgad);
 
    sess->inputgad = gtk_entry_new_with_max_length (2048);
-   gtk_box_pack_end (GTK_BOX (vbox), sess->inputgad, FALSE, FALSE, 0);
+   gtk_box_pack_start (GTK_BOX (vbox), sess->inputgad, FALSE, FALSE, 2);
    gtk_signal_connect (GTK_OBJECT (sess->inputgad), "activate",
                        GTK_SIGNAL_FUNC (handle_inputgad), sess);
    gtk_signal_connect (GTK_OBJECT (sess->inputgad), "key_press_event",
-                       GTK_SIGNAL_FUNC (history_keypress), &sess->history);
+                       GTK_SIGNAL_FUNC (key_handle_key_press), sess);
    gtk_widget_show (sess->inputgad);
 
    if (!page)
@@ -433,14 +422,15 @@ open_dialog_window (struct session *sess)
    {
       if (prefs.newtabstofront)
       {
-         while (gtk_events_pending ())
-            gtk_main_iteration ();
          page = gtk_notebook_page_num (GTK_NOTEBOOK (main_book), sess->window);
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), page);
+
+	   gtk_idle_add ((GtkFunction)maingui_pagetofront, (gpointer)page);
+
          gtk_widget_grab_focus (sess->inputgad);
          current_tab = sess;
          menu_sess = sess;
-         sess->server->front_session = sess;
+	   if (!prefs.use_server_tab)
+      	sess->server->front_session = sess;
          sess->nick_said = FALSE;
          sess->new_data = FALSE;
          gtk_widget_set_style (sess->changad, normaltab_style);
@@ -451,7 +441,7 @@ open_dialog_window (struct session *sess)
 #ifdef USE_GNOME
    if (prefs.dialog_zvt)
    {
-      zvt_term_set_color_scheme ((ZvtTerm *) sess->textgad, rxvt_red, rxvt_grn, rxvt_blu);
+      zvt_term_set_color_scheme ((ZvtTerm *) sess->textgad, zvt_red, zvt_grn, zvt_blu);
       gtk_widget_hide (sess->textgad);
       gtk_widget_show (sess->textgad);
    }
