@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#define SKIPCONFIG
 #include "xchat.h"
 #undef USE_GNOME
 #include "draw.h"
@@ -199,7 +200,7 @@ dcc_open_draw_window (struct DCC *dcc)
    dcc->dccdraw = malloc (sizeof (struct dccdraw));
    memset (dcc->dccdraw, 0, sizeof (struct dccdraw));
 
-   sprintf (tbuf, "DCC-Draw: %s", dcc->nick);
+   snprintf (tbuf, sizeof tbuf, "DCC-Draw: %s", dcc->nick);
    dcc->dccdraw->window = gtkutil_window_new (tbuf, "dcc-draw", dcc->pos + 4, dcc->size + 4,
                    kill_dccdraw, dcc);
 
