@@ -84,12 +84,12 @@ list_addentry (GSList ** list, char *cmd, char *name)
       return;
    if (!cmd)
    {
-      strcpy (pop->name, name);
+      strncpy (pop->name, name, 81);
       pop->cmd[0] = 0;
    } else
    {
-      strcpy (pop->name, name);
-      strcpy (pop->cmd, cmd);
+      strncpy (pop->name, name, 81);
+      strncpy (pop->cmd, cmd, 255);
    }
    *list = g_slist_append (*list, pop);
 }
@@ -97,7 +97,7 @@ list_addentry (GSList ** list, char *cmd, char *name)
 void
 list_loadconf (char *file, GSList ** list, char *defaultconf)
 {
-   char cmd[132];
+   char cmd[256];
    char name[82];
    char *buf, *ibuf;
    int fh, pnt = 0;
@@ -105,10 +105,10 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
 
    buf = malloc (1000);
    snprintf (buf, 1000, "%s/%s", get_xdir (), file);
-   fh = open (buf, O_RDONLY);
+   fh = open (buf, O_RDONLY | OFLAGS);
    if (fh == -1)
    {
-      fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT, 0600);
+	fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
       if (fh != -1)
       {
          write (fh, defaultconf, strlen (defaultconf));
@@ -136,10 +136,10 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
       if (*buf != '#')
       {
          if (!strncasecmp (buf, "NAME ", 5))
-            strcpy (name, buf + 5);
+            strncpy (name, buf + 5, 81);
          if (!strncasecmp (buf, "CMD ", 4))
          {
-            strcpy (cmd, buf + 4);
+            strncpy (cmd, buf + 4, 255);
             if (*name)
             {
                list_addentry (list, cmd, name);
@@ -155,16 +155,16 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
 void
 editlist_gui_load (GtkWidget * listgad)
 {
-   gchar *new[1][2];
+   gchar *nnew[2];
    GSList *list = editlist_list;
    struct popup *pop;
 
    while (list)
    {
       pop = (struct popup *) list->data;
-      new[0][0] = pop->name;
-      new[0][1] = pop->cmd;
-      gtk_clist_append (GTK_CLIST (listgad), new[0]);
+      nnew[0] = pop->name;
+      nnew[1] = pop->cmd;
+      gtk_clist_append (GTK_CLIST (listgad), nnew);
       list = list->next;
    }
 }
@@ -229,10 +229,8 @@ void
 editlist_gui_addnew (GtkWidget * igad)
 {
    int i;
-   gchar *new[1][2];
-   new[0][0] = "*NEW*";
-   new[0][1] = "EDIT ME";
-   i = gtk_clist_append (GTK_CLIST (editlist_gui_list), new[0]);
+   gchar *nnew[2] = {"*NEW*", "EDIT ME"};
+   i = gtk_clist_append (GTK_CLIST (editlist_gui_list), nnew);
    gtk_clist_select_row (GTK_CLIST (editlist_gui_list), i, 0);
    gtk_clist_moveto (GTK_CLIST (editlist_gui_list), i, 0, 0.5, 0);
 }
@@ -257,8 +255,8 @@ editlist_gui_save (GtkWidget * igad)
    char buf[256];
    char *a, *b;
 
-   sprintf (buf, "%s/%s", get_xdir (), editlist_file);
-   fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT, 0600);
+   snprintf (buf, sizeof buf, "%s/%s", get_xdir (), editlist_file);
+   fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
    if (fh != -1)
    {
       while (1)
@@ -266,10 +264,7 @@ editlist_gui_save (GtkWidget * igad)
          if (!gtk_clist_get_text (GTK_CLIST (editlist_gui_list), i, 0, &a))
             break;
          gtk_clist_get_text (GTK_CLIST (editlist_gui_list), i, 1, &b);
-         if (!strcasecmp (a, "SEP"))
-            strcpy (buf, "SEP\n");
-         else
-            sprintf (buf, "NAME %s\nCMD %s\n\n", a, b);
+         sprintf (buf, "NAME %s\nCMD %s\n\n", a, b);
          write (fh, buf, strlen (buf));
          i++;
       }
@@ -368,17 +363,17 @@ editlist_gui_moveup (GtkWidget * igad)
    }
 }
 
-void
+int
 editlist_gui_close (void)
 {
    editlist_gui_window = 0;
+   return 0;
 }
 
 void
 editlist_gui_open (GSList * list, char *title, char *wmclass, char *file, char *help)
 {
-   gchar *titles[] =
-   {"Name", "Command"};
+   gchar *titles[] = {"Name", "Command"};
    GtkWidget *vbox, *hbox, *button;
 
    if (editlist_gui_window)
@@ -389,7 +384,7 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file, char *
    editlist_help = help;
 
    editlist_gui_window = gtkutil_window_new (title, wmclass, 450, 250,
-               editlist_gui_close, 0);
+            					   editlist_gui_close, 0);
    gtk_window_set_policy (GTK_WINDOW (editlist_gui_window), TRUE, TRUE, FALSE);
 
    vbox = gtk_vbox_new (0, 2);
@@ -407,65 +402,50 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file, char *
    gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
    gtk_widget_show (hbox);
 
-#ifndef USE_GNOME
-   gtkutil_button_new ("Move Up", 100, 0, editlist_gui_moveup, 0, hbox);
-   gtkutil_button_new ("Move Dn", 100, 0, editlist_gui_movedown, 0, hbox);
-#else
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_PIXMAP_UP, "Move Up",
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_UP, "Move Up",
         editlist_gui_moveup, 0, hbox);
    gtk_widget_set_usize (button, 100, 0);
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_PIXMAP_DOWN, "Move Dn",
+
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_DOWN, "Move Dn",
       editlist_gui_movedown, 0, hbox);
    gtk_widget_set_usize (button, 100, 0);
-#endif
 
    button = gtk_vseparator_new ();
    gtk_container_add (GTK_CONTAINER (hbox), button);
    gtk_widget_show (button);
 
-#ifdef USE_GNOME
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_BUTTON_CANCEL, "Cancel",
-                                  gtkutil_destroy, editlist_gui_window, hbox);
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_BUTTON_CANCEL, "Cancel",
+                            gtkutil_destroy, editlist_gui_window, hbox);
    gtk_widget_set_usize (button, 100, 0);
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SAVE, "Save",
+
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SAVE, "Save",
           editlist_gui_save, 0, hbox);
    gtk_widget_set_usize (button, 100, 0);
-#else
-   gtkutil_button_new ("Cancel", 100, 0, gtkutil_destroy, editlist_gui_window, hbox);
-   gtkutil_button_new ("Save", 100, 0, editlist_gui_save, 0, hbox);
-#endif
 
    hbox = gtk_hbox_new (0, 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
    gtk_widget_show (hbox);
 
-#ifdef USE_GNOME
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_PIXMAP_NEW, "Add New",
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_NEW, "Add New",
         editlist_gui_addnew, 0, hbox);
    gtk_widget_set_usize (button, 100, 0);
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_PIXMAP_CUT, "Delete",
+
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_CUT, "Delete",
         editlist_gui_delete, 0, hbox);
    gtk_widget_set_usize (button, 100, 0);
-#else
-   gtkutil_button_new ("Add New", 100, 0, editlist_gui_addnew, 0, hbox);
-   gtkutil_button_new ("Delete", 100, 0, editlist_gui_delete, 0, hbox);
-#endif
 
    button = gtk_vseparator_new ();
    gtk_container_add (GTK_CONTAINER (hbox), button);
    gtk_widget_show (button);
 
-#ifdef USE_GNOME
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SPELLCHECK, "Sort",
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SPELLCHECK, "Sort",
           editlist_gui_sort, 0, hbox);
    gtk_widget_set_usize (button, 100, 0);
-   button = gtkutil_stock_button (editlist_gui_window, GNOME_STOCK_PIXMAP_HELP, "Help",
+
+   button = gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_HELP, "Help",
           editlist_gui_help, 0, hbox);
    gtk_widget_set_usize (button, 100, 0);
-#else
-   gtkutil_button_new ("Sort", 100, 0, editlist_gui_sort, 0, hbox);
-   button = gtkutil_button_new ("Help", 100, 0, editlist_gui_help, 0, hbox);
-#endif
+
    if (!help)
       gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
@@ -480,7 +460,7 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file, char *
    gtk_box_pack_start (GTK_BOX (hbox), editlist_gui_entry_name, 0, 0, 0);
    gtk_widget_show (editlist_gui_entry_name);
 
-   editlist_gui_entry_cmd = gtk_entry_new_with_max_length (128);
+   editlist_gui_entry_cmd = gtk_entry_new_with_max_length (255);
    gtk_signal_connect (GTK_OBJECT (editlist_gui_entry_cmd), "changed",
                        GTK_SIGNAL_FUNC (editlist_gui_handle_cmd), 0);
    gtk_container_add (GTK_CONTAINER (hbox), editlist_gui_entry_cmd);
