@@ -20,6 +20,7 @@
 #include <fcntl.h>
 #include "xchat.h"
 
+
 /* gtkutil.c, just some gtk wrappers */
 
 typedef void (*filereqcallback) (void *, void *, char *file);
@@ -130,11 +131,27 @@ gtkutil_window_new (char *title, char *wmclass, int x, int y, void *callback, gp
    gtk_window_set_wmclass (GTK_WINDOW (win), wmclass, "X-Chat");
    gtk_widget_set_usize (win, x, y);
    gtk_window_set_title (GTK_WINDOW (win), title);
+   gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_MOUSE);
    gtk_signal_connect (GTK_OBJECT (win), "destroy",
                        GTK_SIGNAL_FUNC (callback), userdata);
    return win;
 }
 
+GtkWidget *
+gtkutil_dialog_new (char *title, char *wmclass, void *callback, gpointer userdata)
+{
+   GtkWidget *dialog;
+
+   dialog = gtk_dialog_new();
+   gtk_window_set_wmclass (GTK_WINDOW (dialog), wmclass, "X-Chat");
+   gtk_window_set_title (GTK_WINDOW (dialog), title);
+   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+   gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+                       GTK_SIGNAL_FUNC (callback), userdata);
+
+   return dialog;
+}
+
 void
 gtkutil_destroy (GtkWidget * igad, GtkWidget * dgad)
 {
@@ -174,17 +191,40 @@ gtkutil_simpledialog (char *msg)
 }
 
 GtkWidget *
-gtkutil_button_new (char *label, int width, int height, void *callback, gpointer userdata, GtkWidget * box)
+gtkutil_button (GtkWidget * win, char *stock, char *labeltext,
+                void *callback, gpointer userdata, GtkWidget * box)
 {
-   GtkWidget *button;
+   GtkWidget *button, *label, *hbox;
+#ifdef USE_GNOME
+   GtkWidget *pixmap;
+#endif
 
-   button = gtk_button_new_with_label (label);
-   gtk_widget_set_usize (button, width, height);
+   hbox = gtk_hbox_new (0, 0);
+   gtk_widget_show (hbox);
+
+   button = gtk_button_new ();
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (callback), userdata);
-   gtk_container_add (GTK_CONTAINER (box), button);
    gtk_widget_show (button);
 
+#ifdef USE_GNOME
+   if(stock) {
+      pixmap = gnome_stock_pixmap_widget_at_size (win, stock, 12, 14);
+      gtk_container_add (GTK_CONTAINER (hbox), pixmap);
+      gtk_widget_show (pixmap);
+   }
+#endif
+
+   if (labeltext)
+   {
+      label = gtk_label_new (labeltext);
+      gtk_container_add (GTK_CONTAINER (hbox), label);
+      gtk_widget_show (label);
+   }
+   gtk_container_add (GTK_CONTAINER (button), hbox);
+   if (box)
+      gtk_container_add (GTK_CONTAINER (box), button);
+
    return button;
 }
 
@@ -248,42 +288,6 @@ gtkutil_clist_new (int columns, char *titles[],
    return clist;
 }
 
-#ifdef USE_GNOME
-
-GtkWidget *
-gtkutil_stock_button (GtkWidget * win, char *stock, char *labeltext,
-    void *callback, gpointer userdata,
-                      GtkWidget * box)
-{
-   GtkWidget *button, *label, *pixmap, *hbox;
-
-   hbox = gtk_hbox_new (0, 0);
-   gtk_widget_show (hbox);
-
-   button = gtk_button_new ();
-   gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                       GTK_SIGNAL_FUNC (callback), userdata);
-   gtk_widget_show (button);
-
-   pixmap = gnome_stock_pixmap_widget_at_size (win, stock, 12, 14);
-   gtk_container_add (GTK_CONTAINER (hbox), pixmap);
-   gtk_widget_show (pixmap);
-
-   if (labeltext)
-   {
-      label = gtk_label_new (labeltext);
-      gtk_container_add (GTK_CONTAINER (hbox), label);
-      gtk_widget_show (label);
-   }
-   gtk_container_add (GTK_CONTAINER (button), hbox);
-   if (box)
-      gtk_container_add (GTK_CONTAINER (box), button);
-
-   return button;
-}
-
-#endif
-
 int
 gtkutil_clist_selection (GtkWidget * clist)
 {
@@ -294,7 +298,7 @@ gtkutil_clist_selection (GtkWidget * clist)
 }
 
 void
-gtkutil_null_this_var (GtkWidget *unused, GtkWidget **dialog)
+gtkutil_null_this_var (GtkWidget * unused, GtkWidget ** dialog)
 {
    *dialog = 0;
 }
