@@ -16,16 +16,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#define SKIPGNOME
 #include <string.h>
 #include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include "userlist.h"
 #include "xchat.h"
+#include "userlist.h"
 #include "menu.h"
-#ifdef USE_GNOME
-#include <zvt/zvtterm.h>
-#endif
 
 extern GSList *sess_list;
 extern GSList *fkey_list;
@@ -37,6 +35,7 @@ extern GtkWidget *main_book;
 extern GtkWidget *main_window;
 extern struct xchatprefs prefs;
 
+int key_handle_key_press (GtkWidget * wid, GdkEventKey * evt);
 
 #define STATE_SHIFT     GDK_SHIFT_MASK
 #define	STATE_ALT	GDK_MOD1_MASK
@@ -45,8 +44,7 @@ extern struct xchatprefs prefs;
 void
 replace_handle (GtkWidget * t)
 {
-   char *text = gtk_entry_get_text (GTK_ENTRY (t)),
-       *postfix_pnt;
+   char *text, *postfix_pnt;
    struct popup *pop;
    GSList *list = replace_list;
    char word[140];
@@ -54,6 +52,8 @@ replace_handle (GtkWidget * t)
    char outbuf[4096];
    int c, len, xlen;
 
+   text = gtk_entry_get_text (GTK_ENTRY(t));
+
    len = strlen (text);
    for (c = len - 1; c > 0; c--)
    {
@@ -171,7 +171,9 @@ nick_comp_chng (GtkWidget * t, int updown)
    sess = find_session_from_inputgad (t);
    if (sess == NULL)
       return;
+
    text = gtk_entry_get_text (GTK_ENTRY (t));
+   
    if (nick_comp_get_nick (text, nick) == -1)
       return;
    user = sess->userlist;
@@ -236,8 +238,7 @@ tab_comp_find_common (char *a, char *b)
 void
 tab_comp_cmd (GtkWidget * t)
 {
-   char *text = gtk_entry_get_text (GTK_ENTRY (t)),
-       *last = NULL, *cmd, *postfix = NULL;
+   char *text, *last = NULL, *cmd, *postfix = NULL;
    GSList *list = command_list;
    struct popup *pop;
    int len, i, slen;
@@ -245,6 +246,8 @@ tab_comp_cmd (GtkWidget * t)
    char buf[2048];
    char lcmd[2048];
 
+   text = gtk_entry_get_text (GTK_ENTRY (t));
+
    sess = find_session_from_inputgad (t);
 
    text++;
@@ -353,30 +356,106 @@ tab_comp_cmd (GtkWidget * t)
    gtk_entry_set_text (GTK_ENTRY (t), buf);
 }
 
+/* In the following 'b4' is *before* the text (just say b4 and before out loud)
+   and c5 is *after* (because c5 is next after b4, get it??) --AGL */
+
 int
-tab_nick_comp (GtkWidget * t)
+tab_nick_comp_next (struct session *sess, GtkWidget * wid, char *b4,
+      char *nick, char *c5, int shift)
+{
+   struct user *user, *last = NULL;
+   char buf[4096];
+
+   user = sess->userlist;
+   while (user)
+   {
+      if (strcmp (user->user, nick) == 0)
+         break;
+      last = user;
+      user = user->next;
+   }
+   if (!user)
+      return 0;
+   if (shift)
+   {
+      if (last)
+         snprintf (buf, 4096, "%s %s%s", b4, last->user, c5);
+      else
+         snprintf (buf, 4096, "%s %s%s", b4, nick, c5);
+   } else
+   {
+      if (user->next)
+         snprintf (buf, 4096, "%s %s%s", b4, user->next->user, c5);
+      else
+      {
+         if (sess->userlist)
+            snprintf (buf, 4096, "%s %s%s", b4, sess->userlist->user, c5);
+         else
+            snprintf (buf, 4096, "%s %s%s", b4, nick, c5);
+      }
+   }
+   gtk_entry_set_text (GTK_ENTRY (wid), buf);
+
+   return 1;
+}
+
+int
+tab_nick_comp (GtkWidget * t, int shift)
 {
    struct session *sess;
    struct user *user, *best = NULL;
-   char *text = gtk_entry_get_text (GTK_ENTRY (t));
-   int len, slen, highest = -10000;
-   char buf[64];
+   char *text;
+   int len, slen, highest = -10000, first = 0, i, j;
+   char buf[16384], *b4 = NULL, *c5 = NULL;
+
+   text = gtk_entry_get_text (GTK_ENTRY (t));
 
-   if (text[0] == '/')
-   {
-      tab_comp_cmd (t);
-      return 0;
-   }
    sess = find_session_from_inputgad (t);
    if (sess->is_dialog)
       return 0;
    if (sess == NULL)
       return 0;
 
+   len = strlen (text);
+   if (!strchr (text, ' '))
+   {
+      if (text[0] == '/')
+      {
+         tab_comp_cmd (t);
+         return 0;
+      }
+   }
    if (text_is_more_than_just_nick (text))
    {
-      return -1;
-   }
+      j = GTK_EDITABLE (t)->current_pos;
+      b4 = (char *) malloc (len + 1);
+      c5 = (char *) malloc (len + 1);
+      memcpy (c5, &text[j], len - j);
+      c5[len - j] = 0;
+      memcpy (b4, text, len + 1);
+
+      for (i = j - 1; i > -1; i--)
+      {
+         if (b4[i] == ' ')
+         {
+            b4[i] = 0;
+            break;
+         }
+         b4[i] = 0;
+      }
+      memcpy (text, &text[i + 1], (j - i) + 1);
+      text[(j - i) - 1] = 0;
+
+      if (tab_nick_comp_next (sess, t, b4, text, c5, shift))
+      {
+         free (b4);
+         free (c5);
+         return 1;
+      }
+      first = 0;
+   } else
+      first = 1;
+
    user = sess->userlist;
    len = strlen (text);
 
@@ -402,7 +481,16 @@ tab_nick_comp (GtkWidget * t)
    }
    if (best == NULL)
       return 0;
-   snprintf (buf, sizeof (buf), "%s: ", best->user);
+
+   if (first)
+      snprintf (buf, sizeof (buf), "%s: ", best->user);
+   else
+   {
+      snprintf (buf, sizeof (buf), "%s %s%s", b4, best->user, c5);
+      GTK_EDITABLE (t)->current_pos = strlen (b4) + strlen (best->user);
+      free (b4);
+      free (c5);
+   }
    gtk_entry_set_text (GTK_ENTRY (t), buf);
    return 0;
 }
@@ -426,43 +514,10 @@ history_free (struct history *his)
    for (i = 0; i < HISTORY_SIZE; i++)
    {
       if (his->lines[i])
+	{
          free (his->lines[i]);
-   }
-}
-
-void
-history_insert_text (GtkWidget * entry, char *text)
-{
-   char newtext[3000];
-   char *oldtext;
-   int pos;
-
-   oldtext = gtk_entry_get_text (GTK_ENTRY (entry));
-   pos = gtk_editable_get_position ((GtkEditable *) entry);
-   strcpy (newtext, oldtext);
-   newtext[pos] = 0;
-   strcat (newtext, text);
-   strcat (newtext, &oldtext[pos]);
-   gtk_entry_set_text (GTK_ENTRY (entry), newtext);
-   gtk_entry_set_position (GTK_ENTRY (entry), pos + strlen (text));
-}
-
-void
-history_function_key (GtkWidget * entry, int which)
-{
-   GSList *list = fkey_list;
-   int i = 0;
-
-   while (list)
-   {
-      char *macro = (char *) list->data;
-      i++;
-      if (i == which)
-      {
-         history_insert_text (entry, macro);
-         return;
-      }
-      list = list->next;
+	   his->lines[i] = 0;
+	}
    }
 }
 
@@ -502,246 +557,3 @@ history_up (GtkWidget * widget, struct history *his)
          his->pos++;
    }
 }
-
-/* handles PageUp/Down keys */
-void
-scroll_page (GtkWidget * inputgad, int up)
-{
-   int value, end;
-   GtkAdjustment *adj;
-   struct session *sess = find_session_from_inputgad (inputgad);
-
-   if (sess)
-   {
-#ifdef USE_GNOME
-      if (sess->zvt)
-         adj = ((ZvtTerm *) sess->textgad)->adjustment;
-      else
-         adj = GTK_TEXT (sess->textgad)->vadj;
-#else
-      adj = GTK_TEXT (sess->textgad)->vadj;
-#endif
-
-      if (up)                   /* PageUp */
-      {
-         value = adj->value - (adj->page_size / 2);
-         if (value < 0)
-            value = 0;
-      } else
-      {                         /* PageDown */
-         end = adj->upper - adj->lower - adj->page_size;
-         value = adj->value + (adj->page_size / 2);
-         if (value > end)
-            value = end;
-      }
-      gtk_adjustment_set_value (adj, value);
-   }
-}
-
-int
-history_keypress (GtkWidget * widget, GdkEventKey * event, struct history *his)
-{
-   GtkEditable *editable;
-   gint tmp_pos;
-
-   editable = GTK_EDITABLE (widget);
-
-   switch (event->keyval)
-   {
-   case GDK_space:
-      replace_handle (widget);
-      return 0;
-   case GDK_F1:
-      history_function_key (widget, 1);
-      break;
-   case GDK_F2:
-      history_function_key (widget, 2);
-      break;
-   case GDK_F3:
-      history_function_key (widget, 3);
-      break;
-   case GDK_F4:
-      history_function_key (widget, 4);
-      break;
-   case GDK_F5:
-      history_function_key (widget, 5);
-      break;
-   case GDK_F6:
-      history_function_key (widget, 6);
-      break;
-   case GDK_F7:
-      history_function_key (widget, 7);
-      break;
-   case GDK_F8:
-      history_function_key (widget, 8);
-      break;
-   case GDK_F9:
-      history_function_key (widget, 9);
-      break;
-   case GDK_F10:
-      history_function_key (widget, 10);
-      break;
-   case GDK_Tab:
-      if (tab_nick_comp (widget) == -1)
-         return 0;
-      else
-         break;
-   case GDK_Down:
-      history_down (widget, his);
-      break;
-
-   case GDK_Up:
-      history_up (widget, his);
-      break;
-
-   case GDK_p:                 /* History up */
-      if (prefs.emacs_key_bindings)
-         if (event->state & STATE_CTRL)
-            history_up (widget, his);
-      return 0;
-
-   case GDK_n:                 /* History down */
-      if (prefs.emacs_key_bindings)
-         if (event->state & STATE_CTRL)
-            history_down (widget, his);
-      return 0;
-
-   case GDK_Page_Down:
-      if (event->state & STATE_SHIFT)
-         nick_comp_chng (widget, 0);
-      else
-         scroll_page (widget, 0);
-      break;
-
-   case GDK_Page_Up:
-      if (event->state & STATE_SHIFT)
-         nick_comp_chng (widget, 1);
-      else
-         scroll_page (widget, 1);
-      break;
-
-      /*case GDK_Page_Down:
-         nick_comp_chng(widget, 0); break;
-
-         case GDK_Page_Up:
-         nick_comp_chng(widget, 1); break; */
-
-   case GDK_equal:
-      if ((event->state & STATE_ALT) && main_window)
-      {
-         if (event->state & STATE_CTRL)
-            gtk_notebook_set_page (GTK_NOTEBOOK (main_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_book)) + 3);
-         else
-            gtk_notebook_next_page (GTK_NOTEBOOK (main_book));
-      }
-      return 0;
-   case GDK_minus:
-      if ((event->state & STATE_ALT) && main_window)
-      {
-         if (event->state & STATE_CTRL)
-            gtk_notebook_set_page (GTK_NOTEBOOK (main_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_book)) - 3);
-         else
-            gtk_notebook_prev_page (GTK_NOTEBOOK (main_book));
-      }
-      return 0;
-   case GDK_1:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 0);
-      return 0;
-   case GDK_2:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 1);
-      return 0;
-   case GDK_3:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 2);
-      return 0;
-   case GDK_4:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 3);
-      return 0;
-   case GDK_5:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 4);
-      return 0;
-   case GDK_6:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 5);
-      return 0;
-   case GDK_7:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 6);
-      return 0;
-   case GDK_8:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 7);
-      return 0;
-   case GDK_9:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 8);
-      return 0;
-   case GDK_0:
-      if ((event->state & STATE_ALT) && main_window)
-         gtk_notebook_set_page (GTK_NOTEBOOK (main_book), 9);
-      return 0;
-   case GDK_k:                 /* Color */
-      if (prefs.emacs_key_bindings)
-         return 0;
-      if (event->state & STATE_CTRL)
-      {
-         tmp_pos = editable->current_pos;
-         gtk_editable_insert_text (editable, "\3", 1, &tmp_pos);
-         editable->current_pos = tmp_pos;
-         gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-      }
-      return 0;
-   case GDK_b:                 /* bold */
-      if (prefs.emacs_key_bindings)
-         return 0;
-      if (event->state & STATE_CTRL)
-      {
-         tmp_pos = editable->current_pos;
-         gtk_editable_insert_text (editable, "\2", 1, &tmp_pos);
-         editable->current_pos = tmp_pos;
-         gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-      }
-      return 0;
-      /* removed underline beceause ctrl-u is used to clear the textgadget */
-/*      case GDK_u: Underline
-   if(event->state & STATE_CTRL)
-   {
-   tmp_pos = editable->current_pos;
-   gtk_editable_insert_text (editable, "\037", 1, &tmp_pos);
-   editable->current_pos = tmp_pos;
-   gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
-   }
-   return 0; */
-   case GDK_r:                 /* reverse */
-      if (prefs.emacs_key_bindings)
-         return 0;
-      if (event->state & STATE_CTRL)
-      {
-         tmp_pos = editable->current_pos;
-         gtk_editable_insert_text (editable, "\026", 1, &tmp_pos);
-         editable->current_pos = tmp_pos;
-         gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-      }
-      return 0;
-   case GDK_c:                 /* Clear */
-      if (prefs.emacs_key_bindings)
-         return 0;
-      if (event->state & STATE_CTRL)
-      {
-         tmp_pos = editable->current_pos;
-         gtk_editable_insert_text (editable, "\017", 1, &tmp_pos);
-         editable->current_pos = tmp_pos;
-         gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-      }
-      return 0;
-
-   default:
-      return 0;
-   }
-   gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-   return 1;
-}
