@@ -18,10 +18,11 @@
 
 /* LASTLOG by Erik Scrafford <eriks@chilisoft.com>. */
 
-#include "style.h"
 #include "xchat.h"
 #include "gtkutil.h"
-#undef USE_GNOME
+#ifdef USE_GNOME
+#include <zvt/zvtterm.h>
+#endif
 
 
 extern GdkFont *dialog_font_normal;
@@ -43,6 +44,7 @@ search_lastlog (struct session *_sess, char *search)
    char line[512];
    char *start, *realstart, *end;
    struct session *sess;
+   GtkAdjustment *adj;
 
    if (_sess == NULL)
    {
@@ -56,31 +58,51 @@ search_lastlog (struct session *_sess, char *search)
    if (search == NULL)
       return;
 
+#ifdef USE_GNOME
    if (sess->zvt)
    {
-      gtkutil_simpledialog ("Lastlog doesn't work with zvt.");
-      return;
-   }
-   realstart = start = gtk_editable_get_chars ((GtkEditable *) sess->textgad, 0, -1);
-   gtk_text_freeze (GTK_TEXT (lastlog_text));
-   while ((end = strchr (start, '\n')) && ((end - start) <= GTK_TEXT (sess->textgad)->gap_position))
-   {
-      strncpy (line, start, end - start);
-      line[end - start] = 0;
-      if (nocasestrstr (line, search))
-      {
-         strcat (line, "\n");
-         gtk_text_insert (GTK_TEXT (lastlog_text), dialog_font_normal, &colors[prefs.dialog_fg_color], &colors[prefs.dialog_bg_color], line, strlen (line));
-      }
-      start = ++end;
-   }
-   g_free (realstart);
-   gtk_text_thaw (GTK_TEXT (lastlog_text));
+	int len;
 
+	realstart = start = zvt_term_get_buffer ((ZvtTerm *) sess->textgad, &len,
+                                    VT_SELTYPE_LINE, 0, -((ZvtTerm *) sess->textgad)->vx->vt.scrollbackmax,
+                                    0, ((ZvtTerm *) sess->textgad)->vx->vt.height);
+	while ((end = strchr (start, '\n')))
+	{
+   	   strncpy (line, start, end - start);
+   	   line[end - start] = 0;
+   	   if (nocasestrstr (line, search))
+   	   {
+      	strcat (line, "\n");
+      	gtk_text_insert (GTK_TEXT (lastlog_text), dialog_font_normal, &colors[0], &colors[1], line, strlen (line));
+   	   }
+   	   start = ++end;
+	}
+	g_free (realstart);
+
+   } else
    {
-      GtkAdjustment *adj = (GTK_TEXT (lastlog_text))->vadj;
+#endif
+	realstart = start = gtk_editable_get_chars ((GtkEditable *) sess->textgad, 0, -1);
+	gtk_text_freeze (GTK_TEXT (lastlog_text));
+	while ((end = strchr (start, '\n')) && ((end - start) <= GTK_TEXT (sess->textgad)->gap_position))
+	{
+   	   strncpy (line, start, end - start);
+   	   line[end - start] = 0;
+   	   if (nocasestrstr (line, search))
+   	   {
+      	strcat (line, "\n");
+      	gtk_text_insert (GTK_TEXT (lastlog_text), dialog_font_normal, &colors[0], &colors[1], line, strlen (line));
+   	   }
+   	   start = ++end;
+	}
+	g_free (realstart);
+	gtk_text_thaw (GTK_TEXT (lastlog_text));
+
+      adj = (GTK_TEXT (lastlog_text))->vadj;
       gtk_adjustment_set_value (adj, adj->upper - adj->lower - adj->page_size);
+#ifdef USE_GNOME
    }
+#endif
 
    if (search[0])
       snprintf (title, 270, "Lastlog - %s", search);
@@ -145,4 +167,5 @@ show_lastlog_window (void)
 
       gtk_widget_show (lastlog_window);
    }
+   gdk_window_show (lastlog_window->window);
 }
