@@ -553,7 +553,7 @@ zvt_clear (GtkWidget * wid)
 {
    zvt_term_set_scrollback ((ZvtTerm *) wid, 0);
    zvt_term_set_scrollback ((ZvtTerm *) wid, prefs.zvt_lines);
-   zvt_term_feed ((ZvtTerm *) wid, "\e[2J\e[1H", 8);
+   zvt_term_feed ((ZvtTerm *) wid, "\033[2J\033[1H", 8);
 }
 
 void
