@@ -22,12 +22,22 @@
 #include <sys/utsname.h>
 #include "cfgfiles.h"
 #include "gtkutil.h"
+#include "ignore.h"
+#include "userlist.h"
+#ifdef USE_GNOME
+#include <zvt/zvtterm.h>
+#endif
 
 
 extern struct session *current_tab;
 extern struct xchatprefs prefs;
 extern GdkColor colors[];
+extern GSList *sess_list;
 
+extern void connect_server (struct session *sess, char *server, int port, int quiet);
+extern void userlist_hide (GtkWidget * igad, struct session *sess);
+extern void set_channel (struct session *sess);
+extern void palette_edit (void);
 extern GdkFont *my_font_load (char *fontname);
 extern int tcp_send_len (struct server *serv, char *buf, int len);
 extern int tcp_send (struct server *serv, char *buf);
@@ -59,6 +69,7 @@ extern int cmd_unloadall (struct session *sess, char *tbuf, char *word[], char *
 extern int module_unload (char *name, struct session *sess);
 extern void module_glist (struct session *sess);
 extern void pevent_dialog_show ();
+extern void key_dialog_show ();
 
 struct session *menu_sess = 0;
 GSList *popup_list = 0;
@@ -78,17 +89,25 @@ struct submenu_list
 struct submenu_list *first;
 
 
+void 
+goto_url (void *unused, char *url)
+{
+#ifdef USE_GNOME
+   gnome_url_show (url);
+#else
+   char tbuf[256];
+   snprintf (tbuf, sizeof tbuf,
+             "netscape -remote 'openURL(%s)'", url);
+   popen (tbuf, "r");
+#endif
+}
+
 void
-popup_cmd (GtkWidget * igad, char *cmd)
+popup_eval_command (struct session *sess, char *cmd, char *nick, char *allnick)
 {
    int fin = 0, j = 0;
    char *buf;
-   char *nick;
 
-   if (menu_sess->is_dialog)
-      nick = menu_sess->channel;
-   else
-      nick = find_selected_nick (menu_sess);
    if (!nick)
       nick = "";
 
@@ -101,16 +120,42 @@ popup_cmd (GtkWidget * igad, char *cmd)
       case 0:
          fin = 1;
          buf[j] = 0;
+         break;
+ 		case '\\':
+ 			cmd++;
+ 			switch (*cmd)
+ 			{
+ 			case 'n':
+	  	    	buf[j] = 0;
+   	   	handle_command (buf, menu_sess, FALSE, FALSE);	/* handle the comand */
+      		j=0;		
+      		break;  
+         default:
+            cmd--;
+            buf[j] = *cmd;
+            j++;
+         }
          break;
       case '%':
          cmd++;
          switch (*cmd)
          {
+         case 'c':
+            buf[j] = 0;
+            if (sess)
+               strcat (buf, sess->channel);
+            j = strlen (buf);
+            break;
          case 's':
             buf[j] = 0;
             strcat (buf, nick);
             j = strlen (buf);
             break;
+         case 'a':
+            buf[j] = 0;
+            strcat (buf, allnick);
+            j = strlen (buf);
+            break;                                             
          default:
             cmd--;
             buf[j] = *cmd;
@@ -129,7 +174,118 @@ popup_cmd (GtkWidget * igad, char *cmd)
 }
 
 void
-menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags)
+popup_do_for_all (struct session *sess, GtkWidget * igad, char *cmd)
+{
+   GList *cur;
+   int row, i=0, numnick=0, have_s=0;
+   struct user *user;
+   char *allnick=NULL, *ptr, tnick[64]="";
+	
+   cur = GTK_CLIST (igad)->selection;
+   
+   if ((unsigned long) cur == 1)
+   {
+      printf ("Why the **** is it == 1!!??\n");
+      abort ();
+   }
+   if(cur == 0)
+   {
+      popup_eval_command (sess, cmd, "", "");
+      return;
+   }
+
+	while (cur)	/* count the nicks*/
+	{
+		row = (int) cur->data;
+		if (row == -1)
+			break;
+		numnick++;
+		cur = cur->next;
+	}
+	
+	allnick=malloc(numnick*64);
+	allnick[0]=0;	
+	
+   cur = GTK_CLIST (igad)->selection;
+   
+	while (cur)
+	{
+		/*printf("it 1\n"); */
+		row = (int) cur->data;
+		if (row == -1)
+		break;
+		user = gtk_clist_get_row_data (GTK_CLIST (sess->namelistgad), row);
+		if (!user)			  	
+			break;
+      strcat (allnick, user->user);
+		strcat (allnick, " ");
+		cur = cur->next;
+	}
+
+	ptr=cmd;
+	while (*ptr)
+		{
+		if ((*ptr=='%')&&(*(ptr+1)=='s'))
+			{
+			have_s=1;
+			break;
+			}
+		ptr++;
+		}
+
+   if (have_s)
+	{
+		 while (allnick[i])
+		 {
+		 	ptr=tnick;
+			while (allnick[i] != ' ')
+			{
+		 		*ptr=allnick[i];
+		 		ptr++;
+		 		i++;
+		 	}
+		 	*ptr=0;
+		 	i++;
+		 	popup_eval_command (sess, cmd, tnick, allnick);
+		 }
+	}
+   else
+   {
+		popup_eval_command (sess, cmd, "", allnick);		
+	}
+
+	free(allnick);
+}
+
+void
+popup_cmd (GtkWidget * igad, char *cmd)
+{
+   struct session *sess = 0;
+   char *nick = 0;
+
+   if (GTK_IS_BUTTON (igad))
+      sess = gtk_object_get_user_data (GTK_OBJECT (igad));
+
+   if (menu_sess->is_dialog)
+      nick = menu_sess->channel;
+   else
+   {
+      if (sess)
+      {
+         popup_do_for_all (sess, sess->namelistgad, cmd);
+         return;
+      }
+      if (igad)
+         nick = gtk_object_get_user_data (GTK_OBJECT (igad));
+      else
+         nick = 0;
+   }
+
+   popup_eval_command (sess, cmd, nick, nick);
+}
+
+void
+menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags, gpointer userdata)
 {
    GtkWidget *item;
    if (!label)
@@ -137,6 +293,7 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags)
    else
       item = gtk_menu_item_new_with_label (label);
    gtk_menu_append (GTK_MENU (menu), item);
+   gtk_object_set_user_data (GTK_OBJECT (item), userdata);
    if (cmd)
       gtk_signal_connect (GTK_OBJECT (item), "activate",
                           GTK_SIGNAL_FUNC (popup_cmd), cmd);
@@ -145,6 +302,19 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags)
    gtk_widget_show (item);
 }
 
+void
+menu_quick_item_with_callback (void *callback, char *label, GtkWidget * menu, void *arg)
+{
+   GtkWidget *item;
+
+   item = gtk_menu_item_new_with_label (label);
+   gtk_menu_append (GTK_MENU (menu), item);
+   gtk_signal_connect (GTK_OBJECT (item), "activate",
+     GTK_SIGNAL_FUNC (callback), arg);
+   gtk_widget_show (item);
+}
+
+
 GtkWidget *
 menu_quick_sub (char *name, GtkWidget * menu)
 {
@@ -185,16 +355,21 @@ menu_quick_endsub ()
    return (first->menu);
 }
 
-void
-menu_popup (struct session *sess, GdkEventButton * event, char *nick)
-{
+char *nick_copy = 0;
 
+void
+menu_popup (struct session *sess, GdkEventButton * event, char *nick, char *ip)
+{
    struct submenu_list *temp;
    GtkWidget *menu;
    GtkWidget *tempmenu;
    GSList *list = popup_list;
    struct popup *pop;
 
+   if (nick_copy)
+      free (nick_copy);
+   nick_copy = strdup (nick);
+
    /* Create the first element in out linked list, the main popup menu */
    temp = malloc (sizeof (struct submenu_list));
 
@@ -206,8 +381,13 @@ menu_popup (struct session *sess, GdkEventButton * event, char *nick)
    temp->next = NULL;
    first = temp;
 
-   menu_quick_item (0, nick, menu, 1);
-   menu_quick_item (0, 0, menu, 1);
+   tempmenu = menu_quick_sub (nick_copy, tempmenu);
+   if (ip && ip[0])
+      menu_quick_item (0, ip, tempmenu, 1, nick_copy);
+   else
+      menu_quick_item (0, "Host unknown", tempmenu, 1, nick_copy);
+   tempmenu = menu_quick_endsub ();
+   menu_quick_item (0, 0, tempmenu, 1, nick_copy);
 
    while (list)
    {
@@ -220,9 +400,9 @@ menu_popup (struct session *sess, GdkEventButton * event, char *nick)
             tempmenu = menu_quick_endsub ();
          /* If we get here and tempmenu equals menu that means we havent got any submenus to exit from */
       } else if (!strncasecmp (pop->name, "SEP", 3))
-         menu_quick_item (0, 0, tempmenu, 1);
+         menu_quick_item (0, 0, tempmenu, 1, nick_copy);
       else
-         menu_quick_item (pop->cmd, pop->name, tempmenu, 0);
+         menu_quick_item (pop->cmd, pop->name, tempmenu, 0, nick_copy);
 
       list = list->next;
 
@@ -241,6 +421,169 @@ menu_popup (struct session *sess, GdkEventButton * event, char *nick)
    temp = NULL;
 }
 
+void
+menu_url_netscape (GtkWidget * menu, char *url)
+{
+   char buf[256];
+
+   snprintf (buf, sizeof buf, "netscape -remote 'openURL(%s)'", url);
+   popen (buf, "r");
+}
+
+void
+menu_url_lynx (GtkWidget * menu, char *url)
+{
+   char buf[256];
+
+   snprintf (buf, sizeof buf, "xterm -e lynx %s", url);
+   popen (buf, "r");
+}
+
+void
+menu_url_ncftp (GtkWidget * menu, char *url)
+{
+   char buf[256];
+
+   snprintf (buf, sizeof buf, "xterm -e ncftp %s", url);
+   popen (buf, "r");
+}
+
+char *url_copy = 0;
+
+void
+menu_urlmenu (struct session *sess, GdkEventButton * event, char *url)
+{
+   GtkWidget *menu;
+
+   if (url_copy)
+      free (url_copy);
+
+   url_copy = strdup (url);
+
+   menu = gtk_menu_new ();
+
+   menu_quick_item (0, url, menu, 1, url_copy);
+   menu_quick_item (0, 0, menu, 1, url_copy);
+
+   menu_quick_item_with_callback (menu_url_netscape, "-> Netscape", menu, url_copy);
+   menu_quick_item_with_callback (menu_url_lynx, "-> Lynx", menu, url_copy);
+   menu_quick_item_with_callback (menu_url_ncftp, "-> NcFTP", menu, url_copy);
+#ifdef USE_GNOME
+   menu_quick_item_with_callback (goto_url, "-> Gnome URL Handler", menu, url_copy);
+#endif
+
+   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+          event->button, event->time);
+}
+
+void
+menu_chan_part (GtkWidget * menu, char *chan)
+{
+   char tbuf[256];
+
+   if (menu_sess)
+   {
+	snprintf (tbuf, sizeof tbuf, "PART %s\r\n", chan);
+	tcp_send (menu_sess->server, tbuf);
+   }
+}
+
+void
+menu_chan_join (GtkWidget * menu, char *chan)
+{
+   char tbuf[256];
+
+   if (menu_sess)
+   {
+	snprintf (tbuf, sizeof tbuf, "JOIN %s\r\n", chan);
+	tcp_send (menu_sess->server, tbuf);
+   }
+}
+
+void
+menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
+{
+   GtkWidget *menu;
+   GSList *list = sess_list;
+   int is_joined = FALSE;
+   struct session *s;
+
+   while (list)
+   {
+	s = (struct session *) list -> data;
+	if (s->server == sess->server)
+	{
+	   if(!strcasecmp(chan, s->channel))
+	   {
+		is_joined = TRUE;
+		break;
+	   }
+	}
+	list = list -> next;
+   }
+
+   if (url_copy)
+      free (url_copy);
+
+   url_copy = strdup (chan);
+
+   menu = gtk_menu_new ();
+
+   menu_quick_item (0, chan, menu, 1, url_copy);
+   menu_quick_item (0, 0, menu, 1, url_copy);
+
+   if (!is_joined)
+	menu_quick_item_with_callback (menu_chan_join, "Join Channel", menu, url_copy);
+   else
+	menu_quick_item_with_callback (menu_chan_part, "Part Channel", menu, url_copy);
+
+   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+                   event->button, event->time);
+}
+
+void
+menu_url_connectirc (GtkWidget * menu, char *hostname)
+{
+   struct session *sess;
+   struct server *serv;
+
+   serv = new_server ();
+   if (serv)
+   {
+	if (prefs.use_server_tab)
+	   sess = serv->front_session;
+	else
+   	   sess = new_session (serv, 0);
+      if (sess)
+      {
+         strcpy (sess->server->nick, prefs.nick1);
+         connect_server (sess, hostname, 6667, FALSE);
+      }
+   }
+}
+
+void
+menu_hostmenu (struct session *sess, GdkEventButton * event, char *url)
+{
+   GtkWidget *menu;
+
+   if (url_copy)
+      free (url_copy);
+
+   url_copy = strdup (url);
+
+   menu = gtk_menu_new ();
+
+   menu_quick_item (0, url, menu, 1, url_copy);
+   menu_quick_item (0, 0, menu, 1, url_copy);
+
+   menu_quick_item_with_callback (menu_url_connectirc, "Connect to IRC Server", menu, url_copy);
+   menu_quick_item_with_callback (menu_url_netscape, "Send to Netscape", menu, url_copy);
+
+   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+          event->button, event->time);
+}
+
 void
 menu_open_server_list (GtkWidget * wid, gpointer none)
 {
@@ -260,6 +603,72 @@ menu_settings (GtkWidget * wid, gpointer sess)
    settings_opengui (menu_sess);
 }
 
+#ifdef USE_GNOME
+
+int
+shell_exit (ZvtTerm * term, struct session *sess)
+{
+   gtk_widget_destroy (sess->window);
+   return 0;
+}
+
+void
+menu_newshell_tab (GtkWidget * wid, struct session *sess)
+{
+   int oldz = prefs.zvt;
+   int oldb = prefs.nochanmodebuttons;
+   int oldp = prefs.nopaned;
+   struct server *serv;
+   char *shell, *name;
+
+   prefs.zvt = prefs.nochanmodebuttons = prefs.nopaned = 1;
+   serv = new_server ();
+   if (!serv)
+      return;
+   if (prefs.use_server_tab)
+	sess = serv->front_session;
+   else
+	sess = new_session (serv, 0);
+   sess->is_shell = 1;
+   prefs.zvt = oldz;
+   prefs.nochanmodebuttons = oldb;
+   prefs.nopaned = oldp;
+
+   gtk_widget_destroy (sess->confbutton);
+   gtk_signal_disconnect_by_data (GTK_OBJECT (sess->textgad), sess);
+   gtk_widget_set_sensitive (sess->inputgad, FALSE);
+   GTK_WIDGET_UNSET_FLAGS (sess->topicgad, GTK_CAN_FOCUS);
+   userlist_hide (0, sess);
+   gtk_signal_connect (GTK_OBJECT (sess->textgad), "child_died",
+			     GTK_SIGNAL_FUNC (shell_exit), sess);
+   gtk_widget_destroy (sess->inputgad->parent);
+   sess->inputgad = 0;
+   zvt_term_set_scroll_on_keystroke ((ZvtTerm *)sess->textgad, TRUE);
+   gtk_widget_grab_focus (sess->textgad);
+
+   shell = gnome_util_user_shell ();
+   name = strrchr (shell, '/');
+   if (!name)
+      name = shell;
+   else
+      name++;
+
+   strcpy (sess->channel, name);
+   set_channel (sess);
+
+   switch (zvt_term_forkpty ((ZvtTerm *) sess->textgad, 0))
+   {
+   case -1:
+      PrintText (sess, "fork() failed.\n");
+      break;
+
+   case 0:
+      execl (shell, name, NULL);
+      _exit (127);
+   }
+}
+#endif
+
 void
 menu_newserver_window (GtkWidget * wid, gpointer sess)
 {
@@ -268,7 +677,7 @@ menu_newserver_window (GtkWidget * wid, gpointer sess)
 
    prefs.tabchannels = 0;
    serv = new_server ();
-   if (serv)
+   if (serv && !prefs.use_server_tab)
       new_session (serv, 0);
    prefs.tabchannels = old;
 }
@@ -291,7 +700,7 @@ menu_newserver_tab (GtkWidget * wid, gpointer sess)
 
    prefs.tabchannels = 1;
    serv = new_server ();
-   if (serv)
+   if (serv && !prefs.use_server_tab)
       new_session (serv, 0);
    prefs.tabchannels = old;
 }
@@ -396,9 +805,31 @@ savebuffer_req_done (GtkWidget * wid, struct session *sess)
    fh = open (file, O_TRUNC | O_WRONLY | O_CREAT, 0600);
    if (fh != -1)
    {
-      char *buf = gtk_editable_get_chars ((GtkEditable *) sess->textgad, 0, -1);
-      write (fh, buf, strlen (buf));
-      g_free (buf);
+      int len;
+      char *buf;
+
+#ifdef USE_GNOME
+      if (!sess->zvt)
+      {
+#endif
+         buf = gtk_editable_get_chars ((GtkEditable *) sess->textgad, 0, -1);
+         len = strlen (buf);
+#ifdef USE_GNOME
+      } else
+      {
+         buf = zvt_term_get_buffer ((ZvtTerm *) sess->textgad, &len,
+                                    VT_SELTYPE_LINE, 0, -((ZvtTerm *) sess->textgad)->vx->vt.scrollbackmax,
+                                    0, ((ZvtTerm *) sess->textgad)->vx->vt.height);
+      }
+#endif
+      if (buf)
+      {
+         write (fh, buf, len);
+         g_free (buf);
+      } else
+      {
+         gtkutil_simpledialog ("zvt_term_get_buffer() could not allocate enough memory!");
+      }
       close (fh);
    } else
       gtkutil_simpledialog ("Cannot write to that file.");
@@ -407,11 +838,6 @@ savebuffer_req_done (GtkWidget * wid, struct session *sess)
 void
 menu_savebuffer (GtkWidget * wid, gpointer sess)
 {
-   if (menu_sess->zvt)
-   {
-      gtkutil_simpledialog ("That feature doesn't work for zvt.");
-      return;
-   }
    if (freq)
       close_savebuffer ();
    freq = gtk_file_selection_new ("Select an output filename");
@@ -502,11 +928,6 @@ menu_chanlist (GtkWidget * wid, gpointer sess)
    chanlist_opengui (((struct session *) menu_sess)->server);
 }
 
-/*void goto_url()
-   {
-   popen("netscape -remote 'openURL(http://xchat.linuxpower.org)'", "r"); 
-   } */
-
 #ifndef __DATE__
 #define __DATE__ "<unknown>"
 #endif
@@ -518,7 +939,7 @@ menu_about (GtkWidget * wid, gpointer sess)
 {
    GtkWidget *about;
    const gchar *author[] =
-   {"Peter Zelezny <zed@linuxpower.org>", 0};
+   {"Peter Zelezny <zed@linux.com>", 0};
    char buf[512];
    struct utsname un;
 
@@ -531,8 +952,8 @@ menu_about (GtkWidget * wid, gpointer sess)
              "This Binary Compiled:   " __DATE__ "\n"
              "GTK+ Version: %d.%d.%d\n"
              "Currently Running on:   %s %s [%s]\n"
-             "Author EMail:   zed@linuxpower.org\n"
-             "X-Chat Web Page:   http://xchat.linuxpower.org",
+     "Author EMail:   zed@linux.com\n"
+             "X-Chat Web Page:   http://xchat.org",
              gtk_major_version, gtk_minor_version, gtk_micro_version,
    un.sysname, un.release, un.machine);
 
@@ -615,8 +1036,8 @@ menu_about (GtkWidget * wid, gpointer sess)
              "  This Binary Compiled ..... : " __DATE__ "\n"
              "  GTK+ Version ............. : %d.%d.%d\n"
              "  Currently Running on ..... : %s %s [%s]\n"
-             "  Author EMail ............. : zed@linuxpower.org\n"
-             "  X-Chat Web Page .......... : http://xchat.linuxpower.org\n",
+             "  Author EMail ............. : zed@linux.com\n"
+             "  X-Chat Web Page .......... : http://xchat.org\n",
              gtk_major_version, gtk_minor_version, gtk_micro_version,
    un.sysname, un.release, un.machine);
 
@@ -654,7 +1075,8 @@ menu_loadperl_callback (struct session *sess, void *data2, char *file)
    if (file)
    {
       char tbuf[256];
-      sprintf (tbuf, "/LOAD %s", file);
+
+      snprintf (tbuf, sizeof tbuf, "/LOAD %s", file);
       free (file);
       handle_command (tbuf, sess, FALSE, FALSE);
    }
@@ -681,7 +1103,7 @@ menu_loadplugin_callback (struct session *sess, void *data2, char *file)
    {
       char tbuf[256];
 
-      sprintf (tbuf, "/LOADDLL %s", file);
+      snprintf (tbuf, sizeof tbuf, "/LOADDLL %s", file);
       free (file);
       handle_command (tbuf, sess, FALSE, FALSE);
    }
@@ -734,6 +1156,12 @@ menu_evtpopup (void)
    pevent_dialog_show ();
 }
 
+void
+menu_keypopup (void)
+{
+   key_dialog_show ();
+}
+
 void
 menu_ulbuttons (void)
 {
@@ -748,7 +1176,7 @@ menu_ctcpguiopen (void)
 
 #ifdef USE_GNOME
 
-#include "win.xpm"
+#include "pixmaps/win.xpm"
 
 GnomeUIInfo xchatmenu[] =
 {
@@ -756,7 +1184,7 @@ GnomeUIInfo xchatmenu[] =
       GNOME_APP_UI_ITEM,
       N_ ("Server List.."), 0,
       menu_open_server_list, 0, 0,
-      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_RED,
       0, 0, 0
    },
    GNOMEUIINFO_SEPARATOR,
@@ -789,6 +1217,14 @@ GnomeUIInfo xchatmenu[] =
       0, 0, 0
    },
    GNOMEUIINFO_SEPARATOR,
+   {
+      GNOME_APP_UI_ITEM,
+      N_ ("New Shell Tab.."), 0,
+      menu_newshell_tab, 0, 0,
+      GNOME_APP_PIXMAP_DATA, win_xpm,
+      0, 0, 0
+   },
+   GNOMEUIINFO_SEPARATOR,
    {
       GNOME_APP_UI_ITEM,
       N_ ("Close"), 0,
@@ -849,6 +1285,11 @@ GnomeUIInfo windowsmenu[] =
       N_ ("Lastlog Window.."), 0,
       menu_lastlog, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
    },
+   {
+      GNOME_APP_UI_ITEM,
+      N_ ("Ignore Window.."), 0,
+      ignore_gui_open, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
+   },
    GNOMEUIINFO_SEPARATOR,
    {
       GNOME_APP_UI_ITEM,
@@ -927,6 +1368,13 @@ GnomeUIInfo settingsmenu[] =
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
       'S', GDK_MOD1_MASK, 0
    },
+   {
+      GNOME_APP_UI_ITEM,
+      N_ ("Palette.."), 0,
+      palette_edit, 0, 0,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
+      0, 0, 0
+   },
    {
       GNOME_APP_UI_ITEM,
       N_ ("User Commands.."), 0,
@@ -955,13 +1403,6 @@ GnomeUIInfo settingsmenu[] =
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
       0, 0, 0
    },
-   {
-      GNOME_APP_UI_ITEM,
-      N_ ("Function Keys.."), 0,
-      fkeys_edit, 0, 0,
-      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
-      'F', GDK_MOD1_MASK, 0
-   },
    {
       GNOME_APP_UI_ITEM,
       N_ ("Replace Popup.."), 0,
@@ -976,6 +1417,13 @@ GnomeUIInfo settingsmenu[] =
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
       0, 0, 0
    },
+   {
+      GNOME_APP_UI_ITEM,
+      N_ ("Edit Key Bindings.."), 0,
+      menu_keypopup, 0, 0,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
+      0, 0, 0
+   },
    GNOMEUIINFO_SEPARATOR,
    {
       GNOME_APP_UI_ITEM,
@@ -1004,6 +1452,21 @@ GnomeUIInfo helpmenu[] =
       0, 0, 0
    },
    GNOMEUIINFO_SEPARATOR,
+   {
+      GNOME_APP_UI_ITEM,
+      N_ ("X-Chat Homepage.."), 0,
+      goto_url, "http://xchat.org", 0,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
+      0, 0, 0
+   },
+   {
+      GNOME_APP_UI_ITEM,
+      N_ ("Online Docs.."), 0,
+      goto_url, "http://xchat.org/docs.html", 0,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
+      0, 0, 0
+   },
+   GNOMEUIINFO_SEPARATOR,
    {
       GNOME_APP_UI_ITEM,
       N_ ("About X-Chat.."), 0,
@@ -1125,7 +1588,7 @@ createmenus (void *app, struct session *sess)
    ((GtkCheckMenuItem *) (usermodesmenu[9].widget))->active = prefs.autodialog;
    ((GtkCheckMenuItem *) (usermodesmenu[10].widget))->active = prefs.autodccchat;
    ((GtkCheckMenuItem *) (usermodesmenu[11].widget))->active = prefs.autodccsend;
-   ((GtkCheckMenuItem *) (settingsmenu[10].widget))->active = prefs.autosave;
+   ((GtkCheckMenuItem *) (settingsmenu[11].widget))->active = prefs.autosave;
 
 #ifndef USE_PERL
    gtk_widget_set_sensitive (scriptsmenu[0].widget, FALSE);
@@ -1139,83 +1602,101 @@ createmenus (void *app, struct session *sess)
 }
 
 #else
+
+void
+menu_docs ()
+{
+   goto_url (0, "http://xchat.org/docs.html");
+}
+
+void
+menu_webpage ()
+{
+   goto_url (0, "http://xchat.org");
+}
+
 static struct mymenu mymenu[] =
 {
-   {M_NEWMENU, "X-Chat", 0, 0, 1},
-   {M_MENU, "Server List..", menu_open_server_list, 0, 1},
+   {M_NEWMENU, N_("X-Chat"), 0, 0, 1},
+   {M_MENU, N_("Server List.."), menu_open_server_list, 0, 1},
    {M_SEP, 0, 0, 0, 0},
-   {M_MENU, "New Server Tab..", menu_newserver_tab, 0, 1},
-   {M_MENU, "New Server Window..", menu_newserver_window, 0, 1},
-   {M_MENU, "New Channel Tab..", menu_newchannel_tab, 0, 1},
-   {M_MENU, "New Channel Window..", menu_newchannel_window, 0, 1},
+   {M_MENU, N_("New Server Tab.."), menu_newserver_tab, 0, 1},
+   {M_MENU, N_("New Server Window.."), menu_newserver_window, 0, 1},
+   {M_MENU, N_("New Channel Tab.."), menu_newchannel_tab, 0, 1},
+   {M_MENU, N_("New Channel Window.."), menu_newchannel_window, 0, 1},
    {M_SEP, 0, 0, 0, 0},
-   {M_MENU, "Close", menu_close, 0, 1},
+   {M_MENU, N_("Close"), menu_close, 0, 1},
    {M_SEP, 0, 0, 0, 0},
-{M_MENU, "Quit", xchat_cleanup, 0, 1},  /* 10 */
+   {M_MENU, N_("Quit"), xchat_cleanup, 0, 1},  /* 10 */
 
-   {M_NEWMENU, "Windows", 0, 0, 1},
-   {M_MENU, "Channel List Window..", menu_chanlist, 0, 1},
-   {M_MENU, "DCC Send Window..", open_dcc_send_window, 0, 1},
-   {M_MENU, "DCC Receive Window..", open_dcc_recv_window, 0, 1},
-   {M_MENU, "DCC Chat Window..", open_dcc_chat_window, 0, 1},
-   {M_MENU, "Raw Log Window..", menu_rawlog, 0, 1},
-   {M_MENU, "URL Grabber Window..", url_opengui, 0, 1},
-   {M_MENU, "Notify List Window..", (menucallback) notify_opengui, 0, 1},
-   {M_MENU, "Lastlog Window..", menu_lastlog, 0, 1},
+   {M_NEWMENU, N_("Windows"), 0, 0, 1},
+   {M_MENU, N_("Channel List Window.."), menu_chanlist, 0, 1},
+   {M_MENU, N_("DCC Send Window.."), open_dcc_send_window, 0, 1},
+   {M_MENU, N_("DCC Receive Window.."), open_dcc_recv_window, 0, 1},
+   {M_MENU, N_("DCC Chat Window.."), open_dcc_chat_window, 0, 1},
+   {M_MENU, N_("Raw Log Window.."), menu_rawlog, 0, 1},
+   {M_MENU, N_("URL Grabber Window.."), url_opengui, 0, 1},
+   {M_MENU, N_("Notify List Window.."), (menucallback) notify_opengui, 0, 1},
+   {M_MENU, N_("Lastlog Window.."), menu_lastlog, 0, 1},
+   {M_MENU, N_("Ignore Window.."), (menucallback) ignore_gui_open, 0, 1},
    {M_SEP, 0, 0, 0, 0},
-   {M_MENU, "Flush Buffer", menu_flushbuffer, 0, 1},
-   {M_MENU, "Save Buffer..", menu_savebuffer, 0, 1},  /* 22 */
+   {M_MENU, N_("Flush Buffer"), menu_flushbuffer, 0, 1},
+   {M_MENU, N_("Save Buffer.."), menu_savebuffer, 0, 1},  /* 23 */
 
-   {M_NEWMENU, "User Modes", 0, 0, 1},
-   {M_MENUTOG, "Invisible", menu_invisible, 1, 1},
-   {M_MENUTOG, "Receive Wallops", menu_wallops, 1, 1},
-   {M_MENUTOG, "Receive Server Notices", menu_servernotice, 1, 1},
+   {M_NEWMENU, N_("User Modes"), 0, 0, 1},
+   {M_MENUTOG, N_("Invisible"), menu_invisible, 1, 1},
+   {M_MENUTOG, N_("Receive Wallops"), menu_wallops, 1, 1},
+   {M_MENUTOG, N_("Receive Server Notices"), menu_servernotice, 1, 1},
    {M_SEP, 0, 0, 0, 0},
-   {M_MENUTOG, "Marked Away", menu_away, 0, 1},
+   {M_MENUTOG, N_("Marked Away"), menu_away, 0, 1},
    {M_SEP, 0, 0, 0},
-   {M_MENUTOG, "Auto ReJoin on Kick", menu_autorejoin, 0, 1},
-   {M_MENUTOG, "Auto ReConnect to Server", menu_autoreconnect, 0, 1},
+   {M_MENUTOG, N_("Auto ReJoin on Kick"), menu_autorejoin, 0, 1},
+   {M_MENUTOG, N_("Auto ReConnect to Server"), menu_autoreconnect, 0, 1},
    {M_SEP, 0, 0, 0},
-   {M_MENUTOG, "Auto Open Dialog Windows", menu_autodialog, 0, 1},  /* 33 */
-   {M_MENUTOG, "Auto Accept DCC Chat", menu_autodccchat, 0, 1},
-   {M_MENUTOG, "Auto Accept DCC Send", menu_autodccsend, 0, 1},
+   {M_MENUTOG, N_("Auto Open Dialog Windows"), menu_autodialog, 0, 1},  /* 34 */
+   {M_MENUTOG, N_("Auto Accept DCC Chat"), menu_autodccchat, 0, 1},
+   {M_MENUTOG, N_("Auto Accept DCC Send"), menu_autodccsend, 0, 1},
 
-   {M_NEWMENU, "Settings", 0, 0, 1},  /* 36 */
-   {M_MENU, "Setup..", menu_settings, 0, 1},
-   {M_MENU, "User Commands..", (menucallback) menu_usercommands, 0, 1},
-   {M_MENU, "CTCP Replies..", (menucallback) menu_ctcpguiopen, 0, 1},
-   {M_MENU, "Userlist Buttons..", (menucallback) menu_ulbuttons, 0, 1},
-   {M_MENU, "Userlist Popup..", (menucallback) menu_ulpopup, 0, 1},
-   {M_MENU, "Function Keys..", (menucallback) fkeys_edit, 0, 1},
-   {M_MENU, "Replace..", (menucallback) menu_rpopup, 0, 1},
-   {M_MENU, "Edit Event Texts..", (menucallback) menu_evtpopup, 0, 1},
+   {M_NEWMENU, N_("Settings"), 0, 0, 1},  /* 37 */
+   {M_MENU, N_("Setup.."), menu_settings, 0, 1},
+   {M_MENU, N_("Palette.."), (menucallback) palette_edit, 0, 1},
+   {M_MENU, N_("User Commands.."), (menucallback) menu_usercommands, 0, 1},
+   {M_MENU, N_("CTCP Replies.."), (menucallback) menu_ctcpguiopen, 0, 1},
+   {M_MENU, N_("Userlist Buttons.."), (menucallback) menu_ulbuttons, 0, 1},
+   {M_MENU, N_("Userlist Popup.."), (menucallback) menu_ulpopup, 0, 1},
+   {M_MENU, N_("Replace.."), (menucallback) menu_rpopup, 0, 1},
+   {M_MENU, N_("Edit Event Texts.."), (menucallback) menu_evtpopup, 0, 1},
+   {M_MENU, N_("Edit Key Bindings.."), (menucallback) menu_keypopup, 0, 1},
    {M_SEP, 0, 0, 0, 0},
-   {M_MENU, "Save Settings now", menu_savedefault, 0, 1},
-   {M_MENUTOG, "Save Settings on exit", menu_saveexit, 1, 1},
+   {M_MENU, N_("Save Settings now"), menu_savedefault, 0, 1},
+   {M_MENUTOG, N_("Save Settings on exit"), menu_saveexit, 1, 1},
 
-   {M_NEWMENU, "Scripts & Plugins", 0, 0, 1},
+   {M_NEWMENU, N_("Scripts & Plugins"), 0, 0, 1},
 #ifdef USE_PERL
-   {M_MENU, "Load Perl Script..", (menucallback) menu_loadperl, 0, 1},
-   {M_MENU, "Unload All Scripts", (menucallback) menu_unloadall, 0, 1},
+   {M_MENU, N_("Load Perl Script.."), (menucallback) menu_loadperl, 0, 1},
+   {M_MENU, N_("Unload All Scripts"), (menucallback) menu_unloadall, 0, 1},
 #else
-   {M_MENU, "Load Perl Script..", 0, 0, 0},
-   {M_MENU, "Unload All Scripts", 0, 0, 0},
+   {M_MENU, N_("Load Perl Script.."), 0, 0, 0},
+   {M_MENU, N_("Unload All Scripts"), 0, 0, 0},
 #endif
    {M_SEP, 0, 0, 0, 0},
 #ifdef USE_PLUGIN
-   {M_MENU, "Load Plugin (*.so)..", (menucallback) menu_loadplugin, 0, 1},
-   {M_MENU, "Unload All Plugins", (menucallback) menu_unloadallplugins, 0, 1},
-   {M_MENU, "Plugin List", (menucallback) menu_pluginlist, 0, 1},
+   {M_MENU, N_("Load Plugin (*.so).."), (menucallback) menu_loadplugin, 0, 1},
+   {M_MENU, N_("Unload All Plugins"), (menucallback) menu_unloadallplugins, 0, 1},
+   {M_MENU, N_("Plugin List"), (menucallback) menu_pluginlist, 0, 1},
 #else
-   {M_MENU, "Load Plugin (*.so)..", 0, 0, 0},
-   {M_MENU, "Unload All Plugins", 0, 0, 0},
-   {M_MENU, "Plugin List", 0, 0, 0},
+   {M_MENU, N_("Load Plugin (*.so).."), 0, 0, 0},
+   {M_MENU, N_("Unload All Plugins"), 0, 0, 0},
+   {M_MENU, N_("Plugin List"), 0, 0, 0},
 #endif
 
-   {M_NEWMENURIGHT, "Help", 0, 0, 1},
-   {M_MENU, "Help..", menu_help, 0, 1},
+   {M_NEWMENURIGHT, N_("Help"), 0, 0, 1},
+   {M_MENU, N_("Help.."), menu_help, 0, 1},
    {M_SEP, 0, 0, 0, 0},
-   {M_MENU, "About X-Chat..", menu_about, 0, 1},
+   {M_MENU, N_("X-Chat Homepage.."), menu_webpage, 0, 1},
+   {M_MENU, N_("Online Docs.."), menu_docs, 0, 1},
+   {M_SEP, 0, 0, 0, 0},
+   {M_MENU, N_("About X-Chat.."), menu_about, 0, 1},
 
    {M_END, 0, 0, 0, 0},
 };
@@ -1233,15 +1714,15 @@ createmenus (struct session *sess)
    if (!menu_sess)
       menu_sess = sess;
 
-   mymenu[24].state = prefs.invisible;
-   mymenu[25].state = prefs.wallops;
-   mymenu[26].state = prefs.servernotice;
-   mymenu[30].state = prefs.autorejoin;
-   mymenu[31].state = prefs.autoreconnect;
-   mymenu[33].state = prefs.autodialog;
-   mymenu[34].state = prefs.autodccchat;
-   mymenu[35].state = prefs.autodccsend;
-   mymenu[47].state = prefs.autosave;
+   mymenu[25].state = prefs.invisible;
+   mymenu[26].state = prefs.wallops;
+   mymenu[27].state = prefs.servernotice;
+   mymenu[31].state = prefs.autorejoin;
+   mymenu[32].state = prefs.autoreconnect;
+   mymenu[34].state = prefs.autodialog;
+   mymenu[35].state = prefs.autodccchat;
+   mymenu[36].state = prefs.autodccsend;
+   mymenu[48].state = prefs.autosave;
 
    while (1)
    {
