@@ -19,13 +19,11 @@
 #include "xchat.h"
 #include "util.h"
 #include "notify.h"
-#include "style.h"
-#include <fcntl.h>
 #include "cfgfiles.h"
 #include "gtkutil.h"
 #include "plugin.h"
-#undef USE_GNOME
-
+#include <fcntl.h>
+#include <time.h>
 
 extern int tcp_send_len (struct server *serv, char *buf, int len);
 extern int tcp_send (struct server *serv, char *buf);
@@ -33,6 +31,7 @@ extern void PrintText (struct session *sess, unsigned char *text);
 extern int waitline (int sok, char *buf, int bufsize);
 extern void update_all_of (char *name);
 extern GSList *serv_list;
+extern GdkColor colors[];
 
 GSList *notify_list = 0;
 GtkWidget *notify_window = 0;
@@ -97,9 +96,12 @@ void
 notify_gui_update (void)
 {
    struct notify *notify;
+   struct notify_per_server *servnot;
    GSList *list = notify_list;
-   gchar *new[1][3];
-   int row;
+   GSList *slist;
+   gchar *nnew[4];
+   int row, crow, online, servcount;
+   time_t lastseen;
 
    if (!notify_window)
       return;
@@ -110,16 +112,59 @@ notify_gui_update (void)
    while (list)
    {
       notify = (struct notify *) list->data;
-      new[0][0] = notify->name;
-      if (notify->ison)
-         new[0][1] = "Online";
-      else
-         new[0][1] = "Offline";
-      if (notify->servername[0])
-         new[0][2] = notify->servername;
-      else
-         new[0][2] = "-";
-      gtk_clist_append ((GtkCList *) notify_guilist, new[0]);
+      nnew[0] = notify->name;
+      
+      online = FALSE;
+      lastseen = 0;
+      /* First see if they're online on any servers */
+      slist = notify->server_list;
+      while(slist)
+      {
+	 servnot = (struct notify_per_server *)slist->data;
+	 if(servnot->ison)
+	 {
+	    online = TRUE;
+	    if( servnot->laston > lastseen )
+	      lastseen = servnot->lastseen;
+	 }
+	 slist = slist->next;
+      }
+
+      if( !online )		/* Offline on all servers */
+      {
+ 	 nnew[1] = "Offline";
+	 nnew[2] = "";
+	 if( !lastseen )
+	    nnew[3] = "Never";
+	 else
+	    nnew[3] = ctime(&lastseen);
+	 crow = gtk_clist_append ((GtkCList *) notify_guilist, nnew);
+	 gtk_clist_set_foreground(GTK_CLIST (notify_guilist),
+				  crow, &colors[4]);
+      }
+      else {			/* Online - add one line per server */
+	 servcount = 0;
+	 slist = notify->server_list;
+	 while(slist)
+	 {
+	    servnot = (struct notify_per_server *)slist->data;
+	    if( servnot->ison )
+	    {
+	       if(servcount==0)
+		  nnew[0] = notify->name;
+	       else
+		  nnew[0] = "";
+	       nnew[1] = "Online";
+	       nnew[2] = servnot->server->servername;
+	       nnew[3] = ctime(&servnot->laston);
+	       crow = gtk_clist_append ((GtkCList *) notify_guilist, nnew);
+	       gtk_clist_set_foreground(GTK_CLIST (notify_guilist),
+					crow, &colors[3]);
+	       servcount++;
+	    }
+	    slist = slist->next;
+	 }
+      }
       list = list->next;
    }
    if (row != -1)
@@ -150,12 +195,11 @@ void
 notify_opengui (void)
 {
    GtkWidget *vbox, *bbox, *Remove, *Add;
-   static gchar *titles[] =
-   {"User", "Status", "Server"};
+   static gchar *titles[] = {"User", "Status", "Server", "Last Seen"};
 
    if (notify_window)
    {
-      gtk_widget_destroy (notify_window);
+      gdk_window_show (notify_window->window);
       return;
    }
    notify_window = gtkutil_window_new ("X-Chat: Notify List", "notify", 400, 120,
@@ -166,63 +210,115 @@ notify_opengui (void)
    gtk_container_add (GTK_CONTAINER (notify_window), vbox);
    gtk_widget_show (vbox);
 
-   notify_guilist = gtkutil_clist_new (3, titles, vbox, GTK_POLICY_ALWAYS,
-                                 0, 0,
-          0, 0, GTK_SELECTION_BROWSE);
+   notify_guilist = gtkutil_clist_new (4, titles, vbox, GTK_POLICY_ALWAYS,
+                                       0, 0,
+                                       0, 0, GTK_SELECTION_BROWSE);
    gtk_clist_set_column_width (GTK_CLIST (notify_guilist), 0, 100);
    gtk_clist_set_column_width (GTK_CLIST (notify_guilist), 1, 60);
+   gtk_clist_set_column_width (GTK_CLIST (notify_guilist), 2, 100);
+   gtk_clist_set_auto_sort (GTK_CLIST(notify_guilist), FALSE);
 
    bbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
    gtk_widget_show (bbox);
 
-   Add = gtk_button_new_with_label ("Add");
-   gtk_signal_connect (GTK_OBJECT (Add), "clicked",
-                       GTK_SIGNAL_FUNC (notify_add_clicked), 0);
-   gtk_container_add (GTK_CONTAINER (bbox), Add);
-   gtk_widget_show (Add);
-
-   Remove = gtk_button_new_with_label ("Remove");
-   gtk_signal_connect (GTK_OBJECT (Remove), "clicked",
-                       GTK_SIGNAL_FUNC (notify_remove_clicked), 0);
-   gtk_container_add (GTK_CONTAINER (bbox), Remove);
-   gtk_widget_show (Remove);
-
-   gtk_widget_show (notify_window);
+   Add = gtkutil_button (notify_window, GNOME_STOCK_PIXMAP_NEW, "Add",
+				 notify_add_clicked, 0, bbox);
+   Remove = gtkutil_button (notify_window, GNOME_STOCK_PIXMAP_CLOSE, "Remove",
+				 notify_remove_clicked, 0, bbox);
 
    notify_gui_update ();
+
+   gtk_widget_show (notify_window);
 }
 
+
+
+struct notify_per_server *notify_find_server_entry( struct notify *notify,
+						    struct server *serv )
+{
+   GSList *list = notify->server_list;
+   struct notify_per_server *servnot;
+
+   while( list )
+   {
+      servnot = (struct notify_per_server *) list->data;
+      if( servnot->server == serv )
+	 return servnot;
+      list = list->next;
+   }
+   servnot = malloc(sizeof(struct notify_per_server));
+   if( servnot )
+   {
+      memset( servnot, 0, sizeof( struct notify_per_server ));
+      servnot->server = serv;
+      notify->server_list = g_slist_append( notify->server_list, servnot );
+   }
+   return servnot;
+}
+
+
+/* called when receiving a ISON 303 */
+
 void
 notify_markonline (struct session *sess, char *outbuf, char *word[])
 {
    struct notify *notify;
+   struct server *serv = sess->server;
+   struct notify_per_server *servnot;
    GSList *list = notify_list;
-   int i, j;
+   int i, seen;
+
+   if (!sess->server)
+   {
+	fprintf(stderr, "*** something fucked up, no sess->server\n");
+	return;
+   }
 
    while (list)
    {
       notify = (struct notify *) list->data;
+      servnot = notify_find_server_entry( notify, serv );
+      if( !servnot )
+      {
+	list = list->next;
+	continue;
+      }
       i = 4;
-      j = 0;
+      seen = FALSE;
       while (*word[i])
       {
          if (!strcasecmp (notify->name, word[i]))
          {
-            j++;
-            notify->checking = FALSE;
-            if (!notify->ison)
+	    seen = TRUE;
+	    servnot->lastseen = time (0);
+            if (!servnot->ison)
             {
-               notify->ison = TRUE;
-               strcpy (notify->servername, sess->server->servername);
-               EMIT_SIGNAL (XP_TE_NOTIFYONLINE, sess, notify->name, NULL, NULL,
-                            NULL, 0);
+               servnot->ison = TRUE;
+	       servnot->laston = time (0);
+	       EMIT_SIGNAL (XP_TE_NOTIFYONLINE, sess, notify->name, 
+			    serv->servername, NULL, NULL, 0);
                update_all_of (notify->name);
             }
             break;
          }
          i++;
+	   /* FIXME: word[] is only a 32 element array, limits notify list to
+	      about 27 people */
+	   if (i > 27)
+	   {
+		fprintf(stderr, "*** XCHAT WARNING: notify list too large.\n");
+		break;
+	   }
       }
+      if (!seen && servnot->ison)
+      {
+	servnot->ison = FALSE;
+	servnot->lastoff = time (0);
+	EMIT_SIGNAL (XP_TE_NOTIFYOFFLINE, sess, notify->name, 
+		     serv->servername, NULL, NULL, 0);
+	update_all_of (notify->name);
+	}
       list = list->next;
    }
    notify_gui_update ();
@@ -231,7 +327,7 @@ notify_markonline (struct session *sess, char *outbuf, char *word[])
 int
 notify_checklist (void)
 {
-   char *outbuf = malloc (4096);
+   char *outbuf = malloc (512);
    struct server *serv;
    struct notify *notify;
    GSList *list = notify_list;
@@ -242,11 +338,16 @@ notify_checklist (void)
    {
       i++;
       notify = (struct notify *) list->data;
-      if (notify->checking)
-         notify->ison = FALSE;
-      notify->checking = TRUE;
       strcat (outbuf, notify->name);
       strcat (outbuf, " ");
+	if (strlen(outbuf) > 460)
+	{
+	   /* LAME: we can't send more than 512 bytes to the server, but     *
+	    * if we split it in two packets, our offline detection wouldn't  *
+	      work                                                           */
+	   fprintf(stderr, "*** XCHAT WARNING: notify list too large.\n");
+	   break;
+	}
       list = list->next;
    }
    if (i)
@@ -256,7 +357,7 @@ notify_checklist (void)
       while (list)
       {
          serv = (struct server *) list->data;
-         if (serv->connected)
+         if (serv->connected && serv->end_of_motd)
             tcp_send (serv, outbuf);
          list = list->next;
       }
@@ -271,13 +372,16 @@ notify_showlist (struct session *sess)
    char outbuf[256];
    struct notify *notify;
    GSList *list = notify_list;
+   struct notify_per_server *servnot;
    int i = 0;
+
    PrintText (sess, "\00308,02 \002-- Notify List --------------- \003\n");
    while (list)
    {
       i++;
       notify = (struct notify *) list->data;
-      if (notify->ison)
+      servnot = notify_find_server_entry( notify, sess->server );
+      if ( servnot && servnot->ison)
          sprintf (outbuf, "  %-20s online\n", notify->name);
       else
          sprintf (outbuf, "  %-20s offline\n", notify->name);
@@ -296,15 +400,24 @@ int
 notify_deluser (char *name)
 {
    struct notify *notify;
+   struct notify_per_server *servnot;
    GSList *list = notify_list;
+
    while (list)
    {
       notify = (struct notify *) list->data;
       if (!strcasecmp (notify->name, name))
       {
-         notify->ison = FALSE;
          update_all_of (notify->name);
+	 /* Remove the records for each server */
+	 while( notify->server_list )
+	 {
+	    servnot = (struct notify_per_server *) notify->server_list->data;
+	    notify->server_list	= g_slist_remove (notify->server_list,servnot);
+	    free( servnot );
+	 }
          notify_list = g_slist_remove (notify_list, notify);
+	 free (notify->name);
          free (notify);
          notify_gui_update ();
          return 1;
@@ -320,10 +433,9 @@ notify_adduser (char *name)
    struct notify *notify = malloc (sizeof (struct notify));
    if (notify)
    {
-      notify->servername[0] = 0;
-      notify->ison = FALSE;
-      notify->checking = FALSE;
-      strcpy (notify->name, name);
+      memset(notify, 0, sizeof(struct notify));
+      notify->name = strdup (name);
+      notify->server_list = 0; 
       notify_list = g_slist_append (notify_list, notify);
       notify_checklist ();
       update_all_of (notify->name);
@@ -335,6 +447,7 @@ int
 notify_isnotify (struct session *sess, char *name)
 {
    struct notify *notify;
+   struct notify_per_server *servnot;
    GSList *list = notify_list;
 
    while (list)
@@ -342,7 +455,8 @@ notify_isnotify (struct session *sess, char *name)
       notify = (struct notify *) list->data;
       if (!strcasecmp (notify->name, name))
       {
-         if (notify->ison)
+	 servnot = notify_find_server_entry( notify, sess->server );
+	 if (servnot && servnot->ison)
             return TRUE;
       }
       list = list->next;
@@ -350,3 +464,50 @@ notify_isnotify (struct session *sess, char *name)
 
    return FALSE;
 }
+
+void notify_cleanup()
+{
+   GSList *list = notify_list;
+   GSList *nslist, *srvlist;
+   struct notify *notify;
+   struct notify_per_server *servnot;
+   struct server *serv;
+   int valid;
+
+   while (list)
+   {
+      /* Traverse the list of notify structures */
+       notify = (struct notify *) list->data;
+       nslist = notify->server_list;
+       while(nslist)
+       {
+	  /* Look at each per-server structure */
+	  servnot = (struct notify_per_server *)nslist->data;
+	  
+	  /* Check the server is valid */
+	  valid = FALSE;
+	  srvlist = serv_list;
+	  while (srvlist)
+	  {
+	     serv = (struct server *) srvlist->data;
+	     if( servnot->server == serv )
+	     {
+		valid = serv->connected; /* Only valid if server is too */
+		break;
+	     }
+	     srvlist = srvlist->next;
+	  }
+	  if( !valid )
+	  {
+	     notify->server_list = g_slist_remove(notify->server_list,servnot);
+	     nslist = notify->server_list;
+	  }
+	  else
+	  {
+	     nslist = nslist->next;
+	  }
+       }
+       list = list->next;
+   }
+   notify_gui_update();
+}
