@@ -16,13 +16,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include "userlist.h"
-#include "style.h"
 #include "xchat.h"
+#include "userlist.h"
 #include "dcc.h"
-#include "draw.h"
+/*#include "draw.h"*/
 #include "menu.h"
 #include "plugin.h"
+#include "ignore.h"
 #include <sys/time.h>
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -36,7 +36,6 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 
-
 int handle_command (char *cmd, struct session *sess, int history, int);
 void handle_multiline (struct session *sess, char *cmd, int history, int);
 void check_special_chars (char *cmd);
@@ -50,6 +49,7 @@ extern struct xchatprefs prefs;
 extern struct session *current_tab;
 extern struct session *menu_sess;
 
+extern void do_dns (struct session *sess, char *tbuf, char *nick, char *host);
 extern struct session *new_session (struct server *serv, char *from);
 extern GtkWidget *gtkutil_simpledialog (char *msg);
 extern int tcp_send_len (struct server *serv, char *buf, int len);
@@ -168,6 +168,7 @@ int cmd_ban (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_clear (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_close (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_ctcp (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
+int cmd_cycle (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_debug (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_delbutton (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
@@ -175,6 +176,7 @@ int cmd_deop (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_devoice (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_discon (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
+int cmd_echo (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_execk (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_execs (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
@@ -185,6 +187,7 @@ int cmd_execw (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 int cmd_gate (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_hidever (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
+int cmd_ignore (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_invite (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_join (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_kick (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
@@ -215,6 +218,7 @@ int cmd_scpinfo (struct session *sess, char *tbuf, char *word[], char *word_eol[
 int cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_topic (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_unban (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
+int cmd_unignore (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_unloadall (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_wallchop (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
 int cmd_wallchan (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
@@ -224,16 +228,17 @@ struct commands cmds[] =
 {
    {"ADDBUTTON", cmd_addbutton, 0, 0, "/ADDBUTTON <name> <action>, adds a button under the user-list\n"},
    {"AWAY", cmd_away, 1, 0, "/AWAY [<reason>], sets you away\n"},
-   {"BAN", cmd_ban, 1, 1, "/BAN <mask>, bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)\n"},
+   {"BAN", cmd_ban, 1, 1, "/BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)\n"},
    {"CLEAR", cmd_clear, 0, 0, "/CLEAR, Clears the current text window\n"},
    {"CLOSE", cmd_close, 0, 0, "/CLOSE, Closes the current window/tab\n"},
    {"CTCP", cmd_ctcp, 1, 0, "/CTCP <nick> <message>, send the CTCP message to nick, common messages are VERSION and USERINFO\n"},
+   {"CYCLE", cmd_cycle, 1, 1, "/CYCLE, parts current channel and immediately rejoins\n"},
    {"DCC", cmd_dcc, 0, 0, "\n"
     "/DCC GET <nick>          - receive an offered file\n"
     "/DCC SEND <nick> <file>  - send a file to someone\n"
     "/DCC LIST                - show DCC list\n"
     "/DCC CHAT <nick>         - offer DCC CHAT to someone\n"
-    "/DCC DRAW <nick>         - offer DCC DRAW to another X-Chat user\n"
+/*    "/DCC DRAW <nick>         - offer DCC DRAW to another X-Chat user\n"*/
     "/DCC CLOSE <type> <nick> <file>         example:\n"
     "         /dcc close send johnsmith file.tar.gz\n"
    },
@@ -243,6 +248,7 @@ struct commands cmds[] =
    {"DEVOICE", cmd_devoice, 1, 1, "/DEVOICE <nick>, removes voice status from the nick on the current channel (needs chanop)\n"},
    {"DISCON", cmd_discon, 0, 0, "/DISCON, Disconnects from server\n"},
    {"DNS", cmd_dns, 1, 0, "/DNS <nick>, Finds a users IP number\n"},
+   {"ECHO", cmd_echo, 0, 0, "/ECHO <text>, Prints text locally\n"},
    {"EXEC", cmd_exec, 0, 0, "/EXEC [-o] <command>, runs the command. If -o flag is used then output is sent to current channel, else is printed to current text box\n"},
    {"EXECKILL", cmd_execk, 0, 0, "/EXECKILL [-9], kills a running exec in the current session. If -9 is given the process is SIGKILL'ed\n"},
    {"EXECSTOP", cmd_execs, 0, 0, "/EXECSTOP, sends the process SIGSTOP\n"},
@@ -253,6 +259,11 @@ struct commands cmds[] =
    {"GATE", cmd_gate, 0, 0, "/GATE <host> [<port>], proxies through a host, port defaults to 23\n"},
    {"HELP", cmd_help, 0, 0, 0},
    {"HIDEVER", cmd_hidever, 0, 0, "/HIDEVER, Toggles if the auto responce to CTCP VERSION requests is enabled\n"},
+   {"IGNORE", cmd_ignore, 0, 0,
+    "/IGNORE <mask> <types..>\n"
+    "    mask - host mask to ignore, eg: *!*@*.aol.com\n"
+    "    types - types of data to ignore, one or all of:\n"
+    "            PRIV, CHAN, NOTI, CTCP, INVI, ALL\n"},
    {"INVITE", cmd_invite, 1, 0, "/INVITE <nick> [<channel>], invites someone to a channel, by default the current channel (needs chanop)\n"},
    {"JOIN", cmd_join, 1, 0, "/JOIN <channel>, joins the channel\n"},
    {"KICK", cmd_kick, 1, 1, "/KICK <nick>, kicks the nick from the current channel (needs chanop)\n"},
@@ -269,7 +280,7 @@ struct commands cmds[] =
    {"MKICK", cmd_mkick, 1, 1, "/MKICK, Mass kicks everyone except you in the current channel (needs chanop)\n"},
    {"MKICKB", cmd_mkickb, 1, 1, "/MKICKB, Sets a ban of *@* and mass kicks everyone except you in the current channel (needs chanop)\n"},
    {"ME", cmd_me, 1, 1, "/ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)\n"},
-   {"MSG", cmd_msg, 1, 0, "/MSG <nick> <message>, sends a private message\n"},
+   {"MSG", cmd_msg, 0, 0, "/MSG <nick> <message>, sends a private message\n"},
    {"MODE", cmd_mode, 1, 0, "/MODE [<channel>] <mode>, sets a channel mode (see the RFC)\n"},
    {"NAMES", cmd_names, 1, 0, "/NAMES, Lists the nicks on the current channel\n"},
    {"NCTCP", cmd_nctcp, 1, 0, "/NCTCP <nick> <message>, Sends a CTCP notice\n"},
@@ -289,9 +300,10 @@ struct commands cmds[] =
 #ifdef USE_PERL
    {"SCPINFO", cmd_scpinfo, 0, 0, "/SCPINFO, Lists some information about current Perl bindings\n"},
 #endif
-   {"SERVER", cmd_server, 0, 0, "/SERVER <host> [<port>], connects to a server, the default port is 6667\n"},
+   {"SERVER", cmd_server, 0, 0, "/SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667\n"},
    {"TOPIC", cmd_topic, 1, 1, "/TOPIC [<topic>], sets the topic is one is given, else shows the current topic\n"},
    {"UNBAN", cmd_unban, 1, 1, "/UNBAN <mask>, removes a current ban on a mask\n"},
+   {"UNIGNORE", cmd_unignore, 0, 0, "/UNIGNORE <mask>\n"},
 #ifdef USE_PERL
    {"UNLOADALL", cmd_unloadall, 0, 0, "/UNLOADALL, Unloads all perl scripts\n"},
 #endif
@@ -329,10 +341,44 @@ help (struct session *sess, char *helpcmd, int quiet)
       PrintText (sess, "No such command.\n");
 }
 
+void
+send_channel_modes (struct session *sess, char *tbuf,
+			  char *word[],
+		        int start, int end,
+		        char sign, char mode)
+{
+   int left;
+   int i = start;
+
+   while (1)
+   {
+	left = end - i;
+	switch (left)
+	{
+	   case 0:
+		return;
+	   case 1:
+		sprintf(tbuf, "MODE %s %c%c %s\r\n", sess->channel, sign, mode, word[i]);
+		break;
+	   case 2:
+		sprintf(tbuf, "MODE %s %c%c%c %s %s\r\n", sess->channel, sign, mode, mode, word[i], word[i+1]);
+		i++;
+		break;
+	   default:
+		sprintf(tbuf, "MODE %s %c%c%c%c %s %s %s\r\n", sess->channel, sign, mode, mode, mode, word[i], word[i+1], word[i+2]);
+		i += 2;
+		break;
+	}
+	tcp_send (sess->server, tbuf);
+	if (left < 3)
+	   return;
+	i++;
+   }
+}
+
 #define find_word_to_end(a, b) word_eol[b]
 #define find_word(a, b) word[b]
 
-
 int
 cmd_addbutton (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -349,23 +395,135 @@ int
 cmd_away (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    char *reason = find_word_to_end (cmd, 2);
+
    if (*reason)
    {
       sprintf (tbuf, "AWAY :%s\r\n", reason);
       tcp_send (sess->server, tbuf);
    } else
+   {
       tcp_send_len (sess->server, "AWAY\r\n", 6);
+   }
+   if (prefs.show_away_message)
+   {
+      char *away_me;
+      GSList *list = sess_list;
+
+      if (*reason)
+      {
+         /* /me is away: + reason + \0 */
+         away_me = malloc (13 + strlen (reason) + 1);
+         sprintf (away_me, "/me is away: %s", reason);
+      } else
+      {
+         away_me = malloc (13);
+	   strcpy (away_me, "/me is back");
+      }
+
+      while (list)
+      {
+		/* am I the right server and not a dialog box */
+         if (((struct session *) list->data)->server == sess->server 
+			&& ! ((struct session *) list->data)->is_dialog)
+         {
+            handle_command (away_me, (struct session *) list->data,
+                        FALSE, FALSE);
+         }
+         list = list->next;
+      }
+
+      free (away_me);
+   }
    return TRUE;
 }
 
 int
 cmd_ban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
+   int bantype;
+   struct user *user;
    char *mask = find_word (pdibuf, 2);
    if (*mask)
-      sprintf (tbuf, "MODE %s +b %s\r\n", sess->channel, mask);
-   else
-      sprintf (tbuf, "MODE %s +b\r\n", sess->channel);
+   {
+      user = find_name (sess, mask);
+      if (user && user->hostname)  /* it's a nickname, let's find a proper ban mask */
+      {
+	   char *at, *dot, *mask;
+	   char username[64], fullhost[128], domain[128];
+
+	   mask = user->hostname;
+
+	   at = strchr(mask, '@');
+	   if (!at) return TRUE; /* can't happen? */
+	   *at = 0;
+
+	   strcpy (username, mask);
+	   *at = '@';
+	   strcpy (fullhost, at + 1);
+
+	   dot = strchr(mask, '.');
+	   if (dot)
+		strcpy (domain, dot);
+	   else
+		strcpy (domain, fullhost);
+
+	   if (*word[3])
+		bantype = atoi (word[3]);
+	   else
+		bantype = prefs.bantype;
+
+	   if (inet_addr(fullhost) != -1) /* "fullhost" is really a IP number */
+	   {
+		char *lastdot = strrchr (fullhost, '.');
+		if(!lastdot) return TRUE; /* can't happen? */
+
+		*lastdot = 0;
+		strcpy(domain, fullhost);
+		*lastdot = '.';
+
+		switch (bantype)
+		{
+		   case 0:
+			sprintf (tbuf, "MODE %s +b *!*@%s.*\r\n", sess->channel, domain);
+			break;
+
+		   case 1:
+			sprintf (tbuf, "MODE %s +b *!*@%s\r\n", sess->channel, fullhost);
+			break;
+
+		   case 2:
+			sprintf (tbuf, "MODE %s +b *!%s@%s.*\r\n", sess->channel, username, domain);
+			break;
+
+		   case 3:
+			sprintf (tbuf, "MODE %s +b *!%s@%s\r\n", sess->channel, username, fullhost);
+			break;
+		}
+	   } else {
+		switch (bantype)
+		{
+		   case 0:
+			sprintf (tbuf, "MODE %s +b *!*@*%s\r\n", sess->channel, domain);
+			break;
+
+		   case 1:
+			sprintf (tbuf, "MODE %s +b *!*@%s\r\n", sess->channel, fullhost);
+			break;
+
+		   case 2:
+			sprintf (tbuf, "MODE %s +b *!%s@*%s\r\n", sess->channel, username, domain);
+			break;
+
+		   case 3:
+			sprintf (tbuf, "MODE %s +b *!%s@%s\r\n", sess->channel, username, fullhost);
+			break;
+		}
+	   }
+
+      } else
+         sprintf (tbuf, "MODE %s +b %s\r\n", sess->channel, mask);
+   } else
+      sprintf (tbuf, "MODE %s +b\r\n", sess->channel);  /* banlist */
    tcp_send (sess->server, tbuf);
    return TRUE;
 }
@@ -424,6 +582,20 @@ cmd_ctcp (struct session *sess, char *tbuf, char *word[], char *word_eol[])
    return FALSE;
 }
 
+int 
+cmd_cycle (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+   char *key = sess->channelkey;
+   char *chan = sess->channel;
+   if (*chan && !sess->is_dialog)
+   {
+      sprintf (tbuf, "PART %s\r\nJOIN %s %s\r\n", chan, chan, key);
+      tcp_send (sess->server, tbuf);
+      return TRUE;
+   }
+   return FALSE;
+}
+
 int
 cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -452,7 +624,7 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
                if (dcc)
                   dcc_close (dcc, 0, TRUE);
                goodtype = TRUE;
-            }
+            }/*
             if (!strcasecmp (word[3], "DRAW"))
             {
                dcc = find_dcc (word[4], "", TYPE_DRAWRECV);
@@ -466,7 +638,7 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
                      dcc_close (dcc, 0, TRUE);
                }
                goodtype = TRUE;
-            }
+            }*/
             if (!strcasecmp (word[3], "CHAT"))
             {
                dcc = find_dcc (word[4], "", TYPE_CHATRECV);
@@ -498,7 +670,7 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
       {
          dcc_show_list (sess, tbuf);
          return TRUE;
-      }
+      }/*
       if (!strcasecmp (type, "DRAW"))
       {
          if (*word[3])
@@ -507,7 +679,7 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
             return TRUE;
          } else
             return FALSE;
-      }
+      }*/
       if (!strcasecmp (type, "GET"))
       {
          char *nick = find_word (pdibuf, 3);
@@ -615,44 +787,55 @@ int
 cmd_deop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    int i = 2;
+
    while (1)
    {
-      char *name = find_word (pdibuf, i);
-      char *nextname = find_word (pdibuf, i + 1);
-      if (!*name)
-         break;
-      if (*nextname)
-      {
-         sprintf (tbuf, "MODE %s -oo %s %s\r\n", sess->channel, name, nextname);
-         i++;
-      } else
-      {
-         sprintf (tbuf, "MODE %s -o %s\r\n", sess->channel, name);
-      }
-      tcp_send (sess->server, tbuf);
-      i++;
+	if (!*word[i])
+	{
+	   if (i == 2)
+		return FALSE;
+	   send_channel_modes (sess, tbuf, word, 2, i, '-', 'o');
+	   return TRUE;
+	}
+	i++;
    }
-   return TRUE;
 }
 
 int
 cmd_mdeop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    struct user *user;
+   char *nicks[3];
+   int i = 0;
+
    user = sess->userlist;
    while (user)
    {
-      if (user->op)
+      if (user->op && (strcmp (user->user, sess->server->nick) != 0))
       {
-         if (strcmp (user->user, sess->server->nick) != 0)
+         nicks[i] = user->user;
+         i++;
+         if (i == 3)
          {
-            sprintf (tbuf, "MODE %s -o %s\r\n", sess->channel, user->user);
+            i = 0;
+            sprintf (tbuf, "MODE %s -ooo %s %s %s\r\n", sess->channel, nicks[0], nicks[1], nicks[2]);
             tcp_send (sess->server, tbuf);
          }
       }
       user = user->next;
    }
-
+   switch (i)
+   {
+   case 1:
+      sprintf (tbuf, "MODE %s -o %s\r\n", sess->channel, nicks[0]);
+      break;
+   case 2:
+      sprintf (tbuf, "MODE %s -oo %s %s\r\n", sess->channel, nicks[0], nicks[1]);
+      break;
+   default:
+      return TRUE;
+   }
+   tcp_send (sess->server, tbuf);
    return TRUE;
 }
 
@@ -713,16 +896,18 @@ int
 cmd_devoice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    int i = 2;
+
    while (1)
    {
-      char *name = find_word (pdibuf, i);
-      if (!*name)
-         break;
-      sprintf (tbuf, "MODE %s -v %s\r\n", sess->channel, name);
-      tcp_send (sess->server, tbuf);
-      i++;
+	if (!*word[i])
+	{
+	   if (i == 2)
+		return FALSE;
+	   send_channel_modes (sess, tbuf, word, 2, i, '-', 'v');
+	   return TRUE;
+	}
+	i++;
    }
-   return TRUE;
 }
 
 int
@@ -736,16 +921,33 @@ int
 cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    char *nick = word[2];
+   struct user *user;
+
    if (*nick)
    {
-      sprintf (tbuf, "USERHOST %s\r\n", nick);
-      tcp_send (sess->server, tbuf);
-      sess->server->doing_dns = TRUE;
+      user = find_name (sess, nick);
+      if (user && user->hostname)
+      {
+         do_dns (sess, tbuf, user->user, user->hostname);
+      } else
+      {
+         sprintf (tbuf, "WHO %s\r\n", nick);
+         tcp_send (sess->server, tbuf);
+         sess->server->doing_who = TRUE;
+      }
       return TRUE;
    }
    return FALSE;
 }
 
+int
+cmd_echo (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+   PrintText (sess, word_eol[2]);
+   PrintText (sess, "\n");
+   return TRUE;
+}
+
 void
 exec_check_process (struct session *sess)
 {
@@ -867,135 +1069,6 @@ char exec_color_data[2][8][2] =
    }
 };
 
-/*int color_data[] =
-   {
-   1,
-   4,
-   3,
-   5,
-   2,
-   6,
-   10,
-   0,
-   };
-
-   int color_data_bold[] =
-   {
-   1,
-   13,
-   9,
-   8,
-   12,
-   13,
-   11,
-   0,
-   }; */
-
-/*void insert_color(char *buf, int n)
-   {
-   static int bold = FALSE;  probably should be sess->bold
-
-   if(n == 1)
-   {
-   bold = TRUE;
-   *buf = 0;
-   return;
-   }
-
-   if(n > 39)
-   {
-   if(bold)
-   sprintf(buf, "\003,%02d", color_data_bold[n - 40]);
-   else
-   sprintf(buf, "\003,%02d", color_data[n - 40]);
-   return;
-   }
-
-   if(n > 29)
-   {
-   if(bold)
-   sprintf(buf, "\003%02d", color_data_bold[n - 30]);
-   else
-   sprintf(buf, "\003%02d", color_data[n - 30]);
-   return;
-   }
-
-   sprintf(buf, "\003%02d", n);
-   }
-
-   int esc = 0; lame, sess->esc ??
-
-   static void
-   exec_handle_colors (char *buf)
-   {
-   int i = 0, j = 0, n = 0, len;
-   char *newbuf;
-   char num[16];
-
-   len = strlen(buf);
-   newbuf = malloc(len + 1);
-
-   while(i < len)
-   {
-   if(esc && isdigit(buf[i]))
-   {
-   num[n] = buf[i];
-   n++;
-   if(n > 14) n = 14;
-   } else {
-   switch(buf[i])
-   {
-   case '\r':
-   break;
-   case 27:
-   esc = TRUE;
-   break;
-   case '[':
-   if(!esc)
-   {
-   newbuf[j] = buf[i];
-   j++;
-   }
-   break;
-   case ';':
-   if(esc)
-   {
-   num[n] = 0;
-   n = atoi(num);
-   insert_color(&newbuf[j], n);
-   j = strlen(newbuf);
-   n = 0;
-   } else {
-   newbuf[j] = buf[i];
-   j++;
-   }
-   break;
-   case 'm':
-   if(esc)
-   {
-   num[n] = 0;
-   n = atoi(num);
-   insert_color(&newbuf[j], n);
-   j = strlen(newbuf);
-   n = 0;
-   esc = FALSE;
-   } else {
-   newbuf[j] = buf[i];
-   j++;
-   }
-   break;
-   default:
-   newbuf[j] = buf[i];
-   j++;
-   }
-   }
-   i++;
-   }
-   newbuf[j] = 0;
-   memcpy(buf, newbuf, j+1);
-   free(newbuf);
-   } */
-
 /* This is pretty lame, anyone want to write a better one? --AGL */
 static void
 exec_handle_colors (char *buf)
@@ -1120,18 +1193,13 @@ int
 cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    int tochannel = FALSE;
-   char *cmd = word_eol[2];
+   char *cmd = word[2];
    int fds[2], pid = 0;
+   int argstart, i;
    struct nbexec *s;
-   struct stat st;
 
    if (*cmd)
    {
-      if (stat ("/bin/sh", &st) == -1)
-      {
-         gtkutil_simpledialog ("I need /bin/sh to run!\n");
-         return TRUE;
-      }
       exec_check_process (sess);
       if (sess->running_exec != NULL)
       {
@@ -1144,10 +1212,12 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
             return FALSE;
          else
          {
-            cmd = word_eol[3];
+            cmd = word[3];
             tochannel = TRUE;
+		argstart = 3;
          }
-      }
+      } else
+	   argstart = 2;
 #ifdef __EMX__                  /* if os/2 */
       if (pipe (fds) < 0)
       {
@@ -1169,6 +1239,19 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
       s->childfd = fds[1];
       s->tochannel = tochannel;
       s->sess = sess;
+
+	/* make the word[] array null terminated, instead of pointer to null */
+	i = argstart;
+	while(i < 32)
+	{
+	   if(*word[i] == 0)
+	   {
+		word[i] = 0;
+		break;
+	   }
+	   i++;
+	}	
+
       pid = fork ();
       if (pid == 0)
       {
@@ -1182,7 +1265,8 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
          /* Also copy it to stdin so we can write to it */
          dup2 (s->childfd, 0);
          /* Now we call /bin/sh to run our cmd ; made it more friendly -DC1 */
-         execl ("/bin/sh", "sh", "-c", cmd, 0);
+	   if (execvp (cmd, &word[argstart]) == -1)
+		_exit(0);
          /* cmd is in control of this child... if it dies child dies */
       }
       if (pid == -1)
@@ -1322,6 +1406,67 @@ cmd_hidever (struct session *sess, char *tbuf, char *word[], char *word_eol[])
    return TRUE;
 }
 
+int
+cmd_ignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+   int i;
+   int priv = 0;
+   int noti = 0;
+   int chan = 0;
+   int ctcp = 0;
+   int invi = 0;
+   int unignore = 0;
+
+   if (!*word[2])
+   {
+      ignore_showlist (sess);
+      return TRUE;
+   }
+   if (!*word[3])
+      return FALSE;
+
+   i = 3;
+   while (1)
+   {
+      if (!*word[i])
+      {
+         if (!priv && !noti && !chan && !ctcp && !invi && !unignore)
+            return FALSE;
+         i = ignore_add (word[2], priv, noti, chan, ctcp, invi, unignore);
+         if (i == 1)
+         {
+            EMIT_SIGNAL (XP_TE_IGNOREADD, sess, word[2], NULL, NULL, NULL, 0);
+         }
+         if (i == 2)            /* old ignore changed */
+         {
+            EMIT_SIGNAL (XP_TE_IGNORECHANGE, sess, word[2], NULL, NULL, NULL, 0);
+         }
+         return TRUE;
+      }
+      if (!strcasecmp (word[i], "UNIGNORE"))
+         unignore = 1;
+      else if (!strcasecmp (word[i], "ALL"))
+         priv = noti = chan = ctcp = invi = 1;
+      else if (!strcasecmp (word[i], "PRIV"))
+         priv = 1;
+      else if (!strcasecmp (word[i], "NOTI"))
+         noti = 1;
+      else if (!strcasecmp (word[i], "CHAN"))
+         chan = 1;
+      else if (!strcasecmp (word[i], "CTCP"))
+         ctcp = 1;
+      else if (!strcasecmp (word[i], "INVI"))
+         invi = 1;
+      else
+      {
+         PrintText (sess, "Unknown arg '");
+         PrintText (sess, word[i]);
+         PrintText (sess, "' ignored.\n");
+      }
+      i++;
+   }
+}
+
 int
 cmd_invite (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1382,17 +1527,18 @@ cmd_kickban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
    char *nick = find_word (pdibuf, 2);
    if (*nick)
    {
-      sprintf (tbuf, "MODE %s +b %s!*@*\r\n", sess->channel, nick);
-      tcp_send (sess->server, tbuf);
-      {
-         char *reason = find_word_to_end (cmd, 3);
-         if (*reason)
-            sprintf (tbuf, "KICK %s %s :%s\r\n", sess->channel, nick, reason);
-         else
-            sprintf (tbuf, "KICK %s %s\r\n", sess->channel, nick);
-         tcp_send (sess->server, tbuf);
-         return TRUE;
-      }
+	char *reason = find_word_to_end (cmd, 3);
+
+	/* this will use prefs.bantype, no args allowed in /kickban */
+	cmd_ban (sess, tbuf, word, word_eol);
+
+      if (*reason)
+         sprintf (tbuf, "KICK %s %s :%s\r\n", sess->channel, nick, reason);
+      else
+         sprintf (tbuf, "KICK %s %s\r\n", sess->channel, nick);
+   	tcp_send (sess->server, tbuf);
+
+      return TRUE;
    }
    return FALSE;
 }
@@ -1479,13 +1625,18 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
          } else
             strcpy (sess->lastnick, nick);  /* prime the last nick memory */
 
-         EMIT_SIGNAL (XP_TE_MSGSEND, sess, nick, msg, NULL, NULL, 0);
-
          if (!dcc_write_chat (nick, msg))
          {
+            if (!sess->server->connected)
+            {
+               notc_msg (sess);
+               return TRUE;
+            }
             sprintf (tbuf, "PRIVMSG %s :%s\r\n", nick, msg);
             tcp_send (sess->server, tbuf);
          }
+         EMIT_SIGNAL (XP_TE_MSGSEND, sess, nick, msg, NULL, NULL, 0);
+
          return TRUE;
       }
    }
@@ -1585,16 +1736,18 @@ int
 cmd_op (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    int i = 2;
+
    while (1)
    {
-      char *name = find_word (pdibuf, i);
-      if (!*name)
-         break;
-      sprintf (tbuf, "MODE %s +o %s\r\n", sess->channel, name);
-      tcp_send (sess->server, tbuf);
-      i++;
+	if (!*word[i])
+	{
+	   if (i == 2)
+		return FALSE;
+	   send_channel_modes (sess, tbuf, word, 2, i, '+', 'o');
+	   return TRUE;
+	}
+	i++;
    }
-   return TRUE;
 }
 
 int
@@ -1603,7 +1756,7 @@ cmd_part (struct session *sess, char *tbuf, char *word[], char *word_eol[])
    char *chan = find_word (pdibuf, 2);
    if (!*chan)
       chan = sess->channel;
-   if ((*chan) && ((chan[0] == '#') || (chan[0] == '&')))
+   if ((*chan) && ((chan[0] == '#') || (chan[0] == '&') || (chan[0] == '+')))
    {
       sprintf (tbuf, "PART %s\r\n", chan);
       tcp_send (sess->server, tbuf);
@@ -1646,11 +1799,18 @@ cmd_query (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 int
 cmd_quote (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
+   int len;
    char *raw = find_word_to_end (cmd, 2);
    if (*raw)
    {
-      tcp_send (sess->server, raw);
-      tcp_send_len (sess->server, "\r\n", 2);
+	len = strlen (raw);
+	if(len < 4094)
+	{
+   	   tcp_send_len (sess->server, tbuf, sprintf(tbuf, "%s\r\n", raw));
+	} else {
+   	   tcp_send_len (sess->server, raw, len);
+   	   tcp_send_len (sess->server, "\r\n", 2);
+	}
       return TRUE;
    }
    return FALSE;
@@ -1728,6 +1888,19 @@ cmd_unban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
    return FALSE;
 }
 
+int
+cmd_unignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+   char *mask = find_word (pdibuf, 2);
+   if (*mask)
+   {
+      if (ignore_del (mask, NULL))
+         EMIT_SIGNAL (XP_TE_IGNOREREMOVE, sess, mask, NULL, NULL, NULL, 0);
+      return TRUE;
+   }
+   return FALSE;
+}
+
 int
 cmd_wallchop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1737,7 +1910,7 @@ cmd_wallchop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
       char *ttbuf = malloc (4096);
       if (!ttbuf)
          return TRUE;
-      sprintf (tbuf, "NOTICE %%s :[wallchop/@%s] %s\r\n", sess->channel, word_eol[2]);
+      sprintf (tbuf, "NOTICE %%s :[@%s] %s\r\n", sess->channel, word_eol[2]);
       while (user)
       {
          if (user->op)
@@ -1764,7 +1937,7 @@ cmd_wallchan (struct session *sess, char *tbuf, char *word[], char *word_eol[])
       while (list)
       {
          s = (struct session *) list->data;
-         if (s->channel[0] == '#')
+         if (s->channel[0] == '#' || s->channel[0] == '&' || s->channel[0] == '+')
          {
             channel_msg (s->server, tbuf, s->channel, s->server->nick, word_eol[2], TRUE);
             tcp_send_len (s->server, tbuf, sprintf (tbuf, "PRIVMSG %s :%s\r\n", s->channel, word_eol[2]));
@@ -1775,20 +1948,23 @@ cmd_wallchan (struct session *sess, char *tbuf, char *word[], char *word_eol[])
    }
    return FALSE;
 }
+
 int
 cmd_voice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
    int i = 2;
+
    while (1)
    {
-      char *name = find_word (pdibuf, i);
-      if (!*name)
-         break;
-      sprintf (tbuf, "MODE %s +v %s\r\n", sess->channel, name);
-      tcp_send (sess->server, tbuf);
-      i++;
+	if (!*word[i])
+	{
+	   if (i == 2)
+		return FALSE;
+	   send_channel_modes (sess, tbuf, word, 2, i, '+', 'v');
+	   return TRUE;
+	}
+	i++;
    }
-   return TRUE;
 }
 
 void
@@ -1949,6 +2125,11 @@ user_command (struct session *sess, char *cmd, char *word[], char *word_eol[])
          {
             switch (cmd[i + 1])
             {
+            case 'v':
+               strcpy (&tbuf[j], VERSION);
+               j = strlen (tbuf);
+               i++;
+               break;
             case 'n':
                strcpy (&tbuf[j], sess->server->nick);
                j = strlen (tbuf);
@@ -2000,13 +2181,14 @@ handle_command (char *cmd, struct session *sess, int history, int nocommand)
    if (history)
       history_add (&sess->history, cmd);
 
+#ifdef USE_PERL
+   if (perl_command (cmd, sess))
+	return TRUE;
+#endif
+
    if (cmd[0] == '/' && !nocommand)
    {
       cmd++;
-#ifdef USE_PERL
-      if (perl_command (cmd, sess))
-         return TRUE;
-#endif
 
       process_data_init (pdibuf, cmd, word, word_eol);
 
@@ -2063,48 +2245,84 @@ handle_command (char *cmd, struct session *sess, int history, int nocommand)
          }
          i++;
       }
+/*#ifndef SEND_UNKNOWN
       PrintText (sess, "Unknown Command. Try /help\n");
+#else*/
+      /* SEND_UNKNOWN patch by Michael Gold <MGold@iName.com> */
+      if (!sess->server->connected)
+      {
+        PrintText (sess, "Unknown Command. Try /help\n");
+        /*notc_msg (sess);*/
+        return TRUE;
+      }
+      sprintf(tbuf, "%s\r\n", cmd);
+      tcp_send (sess->server, tbuf);
+/*#endif*/
 
    } else
    {
 
       check_special_chars (cmd);
 
-      if (sess->server->connected)
+      if (sess->channel[0] && !sess->is_server)
       {
-         if (sess->channel[0])
+         char newcmd[4096];
+         if (prefs.nickcompletion)
+            check_nick_completion (sess, cmd, newcmd);
+         else
+            newcmd[0] = 0;
+         if (!newcmd[0])
          {
-            char newcmd[4096];
-            if (prefs.nickcompletion)
-               check_nick_completion (sess, cmd, newcmd);
-            else
-               newcmd[0] = 0;
-            if (!newcmd[0])
+            if (sess->is_dialog)
             {
-               channel_msg (sess->server, tbuf, sess->channel, sess->server->nick, cmd, TRUE);
-               sprintf (tbuf, "PRIVMSG %s :%s\r\n", sess->channel, cmd);
+               struct sockaddr_in *ip;
 
-               if (sess->channel[0] != '#' && sess->channel[0] != '&')
+               ip = dcc_write_chat (sess->channel, cmd);
+               if (ip)
                {
-                  struct sockaddr_in *ip;
-                  ip = dcc_write_chat (sess->channel, cmd);
-                  if (ip)
+                  channel_msg (sess->server, tbuf, sess->channel, sess->server->nick, cmd, TRUE);
+                  gtk_entry_set_text (GTK_ENTRY (sess->topicgad),
+                                      inet_ntoa (ip->sin_addr));
+                  return TRUE;
+               } else
+               {
+                  if (sess->server->connected)
                   {
-                     gtk_entry_set_text (GTK_ENTRY (sess->topicgad),
-                                         inet_ntoa (ip->sin_addr));
+                     channel_msg (sess->server, tbuf, sess->channel, sess->server->nick, cmd, TRUE);
+                     sprintf (tbuf, "PRIVMSG %s :%s\r\n", sess->channel, cmd);
+                  } else
+                  {
+                     notc_msg (sess);
                      return TRUE;
                   }
                }
             } else
+            {
+               if (sess->server->connected)
+               {
+                  channel_msg (sess->server, tbuf, sess->channel, sess->server->nick, cmd, TRUE);
+                  sprintf (tbuf, "PRIVMSG %s :%s\r\n", sess->channel, cmd);
+               } else
+               {
+                  notc_msg (sess);
+                  return TRUE;
+               }
+            }
+         } else
+         {
+            if (sess->server->connected)
             {
                channel_msg (sess->server, tbuf, sess->channel, sess->server->nick, newcmd, TRUE);
                sprintf (tbuf, "PRIVMSG %s :%s\r\n", sess->channel, newcmd);
+            } else
+            {
+               notc_msg (sess);
+               return TRUE;
             }
-            tcp_send (sess->server, tbuf);
-         } else
-            notj_msg (sess);
+         }
+         tcp_send (sess->server, tbuf);
       } else
-         notc_msg (sess);
+         notj_msg (sess);
    }
    return TRUE;
 }
