@@ -25,15 +25,18 @@
 #include "util.h"
 #include "plugin.h"
 #include "gtkutil.h"
+#include "util.h"
 
 #ifdef USE_PLUGIN
 
 extern void notj_msg (struct session *sess);
 extern void notc_msg (struct session *sess);
 extern void PrintText (struct session *sess, unsigned char *text);
+extern char *get_xdir (void);
 
 void unhook_all_by_mod (struct module *mod);
 void module_glist_fill (void);
+int module_load (char *name, struct session *sess);
 
 struct module *modules;
 struct module_cmd_set *mod_cmds;
@@ -47,6 +50,19 @@ int (*sighandler[NUM_XP]) (void *, void *, void *, void *, void *, char);
 
 extern void printevent_setup ();
 
+
+void
+module_autoload(char *mod)
+{
+#ifdef USE_PLUGIN
+   char tbuf[256];
+
+   snprintf(tbuf, sizeof tbuf, "\002Loading\00302: \017%s\00302...\017\n", file_part(mod));
+   PrintText(0, tbuf);
+   module_load (mod, 0);
+#endif
+}
+
 void 
 signal_setup ()
 {
@@ -187,7 +203,7 @@ module_load (char *name, struct session *sess)
    if (handle == NULL)
 #ifdef HAVE_DLERROR
    {
-      PrintText (sess, dlerror ());
+      PrintText (sess, (char *)dlerror ());
       PrintText (sess, "\n");
 #endif
       return 1;
@@ -363,6 +379,7 @@ module_setup ()
 {
    modules = NULL;
    mod_cmds = NULL;
+   for_files(get_xdir(), "*.so", module_autoload);
 }
 
 /* ************** GUI STUFF ***************** */
