@@ -1,12 +1,12 @@
 
-#ifndef	PLUGIN_H
+#ifndef PLUGIN_H
 #define PLUGIN_H
 
-#define	MODULE_IFACE_VER	2
-#define TE_MAX_VAL 98
-#define	XP_CALLBACK(x)	( (int (*) (void *, void *, void *, void *, void *, char) ) x )
+#define MODULE_IFACE_VER	2
+/*#define TE_MAX_VAL  104 */
+#define XP_CALLBACK(x)	( (int (*) (void *, void *, void *, void *, void *, char) ) x )
 
-#define	XP_USERCOMMAND	0
+#define XP_USERCOMMAND	0
 #define XP_PRIVMSG	1
 #define XP_CHANACTION	2
 #define XP_CHANMSG	3
@@ -14,14 +14,14 @@
 #define XP_JOIN		5
 #define XP_CHANSETKEY	6
 #define XP_CHANSETLIMIT	7
-#define	XP_CHANOP	8
-#define	XP_CHANVOICE	9
+#define XP_CHANOP	8
+#define XP_CHANVOICE	9
 #define XP_CHANBAN	10
 #define XP_CHANRMKEY	11
-#define	XP_CHANRMLIMIT	12
-#define	XP_CHANDEOP	13
-#define	XP_CHANDEVOICE	14
-#define	XP_CHANUNBAN	15
+#define XP_CHANRMLIMIT	12
+#define XP_CHANDEOP	13
+#define XP_CHANDEVOICE	14
+#define XP_CHANUNBAN	15
 #define XP_INBOUND	16
 
 #define XP_TE_JOIN       17
@@ -124,8 +124,19 @@
 #define XP_TE_NOTICESEND   114
 #define XP_TE_WALLOPS      115
 #define XP_HIGHLIGHT       116
+#define XP_TE_IGNOREHEADER 117
+#define XP_TE_IGNORELIST   118
+#define XP_TE_IGNOREFOOTER 119
+#define XP_TE_IGNOREADD    120
+#define XP_TE_IGNOREREMOVE 121
+#define XP_TE_RESOLVINGUSER 122
+#define XP_TE_IGNOREEMPTY  123
+#define XP_TE_IGNORECHANGE 124
+#define XP_TE_NOTIFYOFFLINE 125
+#define XP_TE_MALFORMED_FROM 126
+#define XP_TE_MALFORMED_PACKET 127
 
-#define	NUM_XP	        117
+#define NUM_XP		   128
 
 
 #define	EMIT_SIGNAL(s, a, b, c, d, e, f) (fire_signal(s, a, b, c, d, e, f))
