@@ -16,9 +16,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#define SKIPCONFIG
 #include "xchat.h"
 #include "gtkutil.h"
-#undef USE_GNOME
+#include <gdk/gdkkeysyms.h>
+#include <fcntl.h>
 
 
 extern GdkColor colors[];
@@ -31,17 +33,55 @@ close_rawlog (GtkWidget * wid, struct server *serv)
    return 0;
 }
 
-void 
+void
+rawlog_save(struct server *serv, void *unused, char *file)
+{
+   char *buf;
+   int fh = -1;
+
+   if (file)
+   {
+      if (serv->rawlog_window)
+	   fh = open (file, O_TRUNC | O_WRONLY | O_CREAT, 0600);
+	if (fh != -1)
+	{
+	   buf = gtk_editable_get_chars ((GtkEditable *) serv->rawlog_textlist, 0, -1);
+	   if (buf)
+   	   {
+      	write (fh, buf, strlen (buf));
+      	g_free (buf);
+   	   }
+	   close (fh);
+	}
+	free (file);
+   }
+}
+
+int
+rawlog_keypress (GtkWidget * wid, GdkEventKey * event, struct server *serv)
+{
+   if (event->keyval == GDK_s && event->state & GDK_MOD1_MASK)
+   {
+      gtkutil_file_req ("Save rawlog", rawlog_save, serv, NULL, TRUE);
+	return FALSE;
+   }
+   return TRUE;
+}
+
+void
 open_rawlog (struct server *serv)
 {
    GtkWidget *hbox, *vscrollbar;
+   char tbuf[256];
 
    if (serv->rawlog_window)
    {
-      gtk_widget_destroy (serv->rawlog_window);
+      gdk_window_show (serv->rawlog_window->window);
       return;
    }
-   serv->rawlog_window = gtkutil_window_new ("X-Chat: Raw Log", "rawlog", 450, 100,
+
+   snprintf(tbuf, sizeof tbuf, "X-Chat: Rawlog (%s) [alt-s to save]", serv->servername);
+   serv->rawlog_window = gtkutil_window_new (tbuf, "rawlog", 450, 100,
                   close_rawlog, serv);
 
    hbox = gtk_hbox_new (FALSE, 0);
@@ -52,7 +92,10 @@ open_rawlog (struct server *serv)
    gtk_text_set_word_wrap (GTK_TEXT (serv->rawlog_textlist), TRUE);
    gtk_text_set_editable (GTK_TEXT (serv->rawlog_textlist), FALSE);
    gtk_container_add (GTK_CONTAINER (hbox), serv->rawlog_textlist);
+   gtk_signal_connect (GTK_OBJECT (serv->rawlog_textlist), "key_press_event",
+                       GTK_SIGNAL_FUNC (rawlog_keypress), serv);
    gtk_widget_show (serv->rawlog_textlist);
+   gtk_widget_grab_focus (serv->rawlog_textlist);
 
    vscrollbar = gtk_vscrollbar_new (GTK_TEXT (serv->rawlog_textlist)->vadj);
    gtk_box_pack_start (GTK_BOX (hbox), vscrollbar, FALSE, FALSE, 0);
