@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include "style.h"
+#define SKIPGNOME
 #include "xchat.h"
 #include <netdb.h>
 #include <arpa/inet.h>
@@ -27,12 +27,12 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include "plugin.h"
-#undef USE_GNOME
 
 extern GSList *sess_list;
 extern struct xchatprefs prefs;
 
-
+extern void set_server_name (struct server *serv, char *name);
+extern void flush_server_queue (struct server *serv);
 extern int tcp_send_len (struct server *serv, char *buf, int len);
 extern int tcp_send (struct server *serv, char *buf);
 extern void clear_user_list (struct session *sess);
@@ -43,7 +43,27 @@ extern int waitline (int sok, char *buf, int bufsize);
 extern void gui_set_title (struct session *sess);
 extern void clear_channel (struct session *sess);
 extern void notc_msg (struct session *sess);
+extern void notify_cleanup(void);
 
+#ifndef HAVE_INET_ATON
+
+int inet_aton(const char *cp, struct in_addr *inp)
+{
+   struct in_addr RAddr;
+
+   RAddr.s_addr = inet_addr(cp);
+   if (RAddr.s_addr != -1)
+   {
+	if (inp)
+	{
+	   *inp = RAddr;
+	   return 1;
+	}
+   }
+
+   return 0;
+}
+#endif
 
 void 
 connecting_fin (struct session *sess)
@@ -81,7 +101,11 @@ connected_signal (struct server *serv, gint sok)
    switch (tbuf[0])
    {
    case '1':                   /* unknown host */
-      gdk_input_remove (serv->iotag);
+	if(serv->iotag != -1)
+	{
+   	   gdk_input_remove (serv->iotag);
+	   serv->iotag = -1;
+	}
       close (serv->childread);
       close (serv->sok);
       waitpid (serv->childpid, NULL, WNOHANG);
@@ -90,7 +114,11 @@ connected_signal (struct server *serv, gint sok)
       break;
    case '2':                   /* connection failed */
       waitline (serv->childread, tbuf, 128);
-      gdk_input_remove (serv->iotag);
+	if(serv->iotag != -1)
+	{
+   	   gdk_input_remove (serv->iotag);
+	   serv->iotag = -1;
+	}
       close (serv->childread);
       close (serv->sok);
       waitpid (serv->childpid, NULL, WNOHANG);
@@ -102,28 +130,30 @@ connected_signal (struct server *serv, gint sok)
       {
          char host[100];
          char ip[100];
-         int port;
 
          waitline (serv->childread, host, sizeof host);
 
          waitline (serv->childread, ip, sizeof ip);
 
-         waitline (serv->childread, tbuf, sizeof tbuf);
-         port = atoi (tbuf);
+         waitline (serv->childread, outbuf, sizeof outbuf);
 
-         sprintf (outbuf, "%d", port);
          EMIT_SIGNAL (XP_TE_CONNECT, sess, host, ip, outbuf, NULL, 0);
          strcpy (serv->hostname, ip);
       }
       break;
    case '4':                   /* success */
-      gdk_input_remove (serv->iotag);
+	if(serv->iotag != -1)
+	{
+   	   gdk_input_remove (serv->iotag);
+	   serv->iotag = -1;
+	}
       close (serv->childread);
       serv->connected = TRUE;
       waitpid (serv->childpid, NULL, WNOHANG);
       connecting_fin (sess);
       serv->iotag = gdk_input_add (serv->sok, GDK_INPUT_EXCEPTION | GDK_INPUT_READ,
                                    (GdkInputFunction) read_data, serv);
+
       if (!serv->no_login)
       {
          EMIT_SIGNAL (XP_TE_CONNECTED, sess, NULL, NULL, NULL, NULL, 0);
@@ -134,12 +164,23 @@ connected_signal (struct server *serv, gint sok)
          }
          sprintf (outbuf, "NICK %s\r\n", serv->nick);
          tcp_send (serv, outbuf);
-         sprintf (outbuf, "USER %s 0 0 :%s\r\n",
-                  prefs.username, prefs.realname);
+         sprintf (outbuf, "USER %s %s %s :%s\r\n",
+                  prefs.username,
+                  (strlen(prefs.local_name) ? prefs.local_name : "localhost"),
+                  serv->hostname, prefs.realname);
          tcp_send (serv, outbuf);
       } else
          EMIT_SIGNAL (XP_TE_SERVERCONNECTED, sess, NULL, NULL, NULL, NULL, 0);
       fcntl (serv->sok, F_SETFL, O_NONBLOCK);
+	set_server_name(serv, serv->servername);
+      break;
+   case '5':                   /* prefs ip discovered */
+      waitline (serv->childread, prefs.ipaddress, sizeof(prefs.ipaddress));
+      inet_aton(prefs.ipaddress, (struct in_addr *)&(prefs.local_ip));
+      break;
+   case '6':                   /* prefs hostname discovered */
+      waitline (serv->childread, prefs.hostname, sizeof(prefs.hostname));
+      strcpy(prefs.local_name, prefs.hostname);
       break;
    }
 }
@@ -154,8 +195,11 @@ check_connecting (struct session *sess)
       waitpid (sess->server->childpid, NULL, WNOHANG);
       sprintf (tbuf, "%d", sess->server->childpid);
       EMIT_SIGNAL (XP_TE_SCONNECT, sess, tbuf, NULL, NULL, NULL, 0);
-
-      gdk_input_remove (sess->server->iotag);
+	if(sess->server->iotag != -1)
+	{
+   	   gdk_input_remove (sess->server->iotag);
+	   sess->server->iotag = -1;
+	}
       close (sess->server->childread);
       close (sess->server->sok);
       connecting_fin (sess);
@@ -178,7 +222,15 @@ disconnect_server (struct session *sess, int sendquit)
       notc_msg (sess);
       return;
    }
-   gdk_input_remove (serv->iotag);
+
+   flush_server_queue (serv);
+
+   if (serv->iotag != -1)
+   {
+	gdk_input_remove (serv->iotag);
+	serv->iotag = -1;
+   }
+
    list = sess_list;
    while (list)                 /* print "Disconnected" to each window using this server */
    {
@@ -204,6 +256,7 @@ disconnect_server (struct session *sess, int sendquit)
    serv->connected = FALSE;
    serv->motd_skipped = FALSE;
    serv->no_login = FALSE;
+   serv->servername[0] = 0;
 
    list = sess_list;
    while (list)
@@ -213,13 +266,15 @@ disconnect_server (struct session *sess, int sendquit)
       {
          if (sess->channel[0])
          {
-            if (sess->channel[0] == '#' || sess->channel[0] == '&')
+            if (sess->channel[0] == '#' || sess->channel[0] == '&' ||
+		    sess->channel[0] == '+' || sess->is_server)
                clear_channel (sess);
          } else
             clear_channel (sess);
       }
       list = list->next;
    }
+   notify_cleanup();
 }
 
 int 
@@ -253,6 +308,8 @@ connect_server (struct session *sess, char *server, int port, int quiet)
    if (!server[0])
       return;
 
+   sess = sess->server->front_session;
+
    if (sess->server->connected)
       disconnect_server (sess, TRUE);
    else
@@ -278,10 +335,16 @@ connect_server (struct session *sess, char *server, int port, int quiet)
    sw = 1;
    setsockopt (sok, SOL_SOCKET, SO_KEEPALIVE, (char *) &sw, sizeof (sw));
 
+   prefs.local_ip = 0;
+   prefs.local_name[0] = 0;
+
+   strcpy(sess->server->servername, server);
    sess->server->nickcount = 1;
    sess->server->connecting = TRUE;
    sess->server->sok = sok;
    sess->server->port = port;
+   sess->server->end_of_motd = FALSE;
+   flush_server_queue (sess->server);
 
    if (quiet)
       sess->server->no_login = TRUE;
@@ -298,7 +361,7 @@ connect_server (struct session *sess, char *server, int port, int quiet)
    socketpair (AF_UNIX, SOCK_STREAM, 0, read_des);
 #endif
 
-   signal (SIGCHLD, SIG_IGN);
+   /*signal (SIGCHLD, SIG_IGN);*/
 
    switch (pid = fork ())
    {
@@ -313,6 +376,62 @@ connect_server (struct session *sess, char *server, int port, int quiet)
          dup2 (read_des[1], 1);
          close (read_des[0]);
          setuid (getuid ());
+
+         if (!prefs.detectip)
+         {
+            struct sockaddr_in DSAddr;
+                  
+            memset (&DSAddr, 0, sizeof (DSAddr));
+            DSAddr.sin_family = AF_INET;
+      
+            if (prefs.ipaddress && strlen(prefs.ipaddress)
+                && inet_aton (prefs.ipaddress, &(DSAddr.sin_addr))) 
+            {
+               /* Don't do anything - it worked */
+            } else if (prefs.hostname && strlen(prefs.hostname))
+            {
+               struct hostent *HostAddr;
+      
+               HostAddr = gethostbyname (prefs.hostname);
+               if (HostAddr)
+               {
+                  DSAddr.sin_addr = *((struct in_addr *) HostAddr->h_addr);
+               }
+            }
+      
+            if (DSAddr.sin_addr.s_addr)
+            {
+               if (!bind(sok, (struct sockaddr *) &DSAddr, sizeof (DSAddr)))
+               {
+                  printf ("5\n%s\n", inet_ntoa(DSAddr.sin_addr));
+                  fflush (stdout);
+
+                  /* doesn't affect master prefs - but helps us here */
+                  prefs.local_ip = DSAddr.sin_addr.s_addr;
+               }
+            }
+         }
+
+         if (!prefs.detecthost)
+         {
+            if (prefs.hostname && strlen(prefs.hostname))
+            {
+               printf ("6\n%s\n", prefs.hostname);
+               fflush (stdout);
+            } else if (prefs.local_ip)
+            {
+               struct hostent *HostAddr;
+      
+               HostAddr = gethostbyaddr ((char *)&(prefs.local_ip),
+                                         sizeof(struct in_addr), AF_INET);
+               if (HostAddr)
+               {
+                  printf ("6\n%s\n", HostAddr->h_name);
+                  fflush (stdout);
+               }
+            }
+         }
+
          HostAddr = gethostbyname (server);
          if (HostAddr)
          {
@@ -327,7 +446,38 @@ connect_server (struct session *sess, char *server, int port, int quiet)
             if (connect (sok, (struct sockaddr *) &SAddr, sizeof (SAddr)) < 0)
                printf ("2\n%d\n", errno);
             else
+            {
+               if (prefs.detectip || prefs.detecthost)
+               {
+                  struct sockaddr_in DSAddr;
+                  int len;
+         
+                  len = sizeof (DSAddr);
+                  memset (&DSAddr, 0, len);
+                  DSAddr.sin_family = AF_INET;
+                  getsockname(sok, (struct sockaddr *)&DSAddr, &len);
+         
+                  if (prefs.detectip && !prefs.ip_from_server)
+                  {
+                     printf ("5\n%s\n", inet_ntoa(DSAddr.sin_addr));
+                     fflush (stdout);
+                  }
+         
+                  if (prefs.detecthost)
+                  {
+                     struct hostent *HostAddr;
+           
+                     HostAddr = gethostbyaddr ((char *)&(DSAddr.sin_addr.s_addr),
+                                               sizeof(struct in_addr), AF_INET);
+                     if (HostAddr)
+                     {
+                        printf ("6\n%s\n", HostAddr->h_name);
+                        fflush (stdout);
+                     }
+                  }
+               }
                printf ("4\n");
+            }
          } else
             printf ("1\n");
          fflush (stdout);
