@@ -67,6 +67,7 @@ static int previous_sub = 0;
 static int closing_list = 0;
 static gboolean channelbox = 0;
 static gboolean first_autoconnect = 0;
+static int treerow = -1;
 
 /**************************
  * Public functions:      *
@@ -419,8 +420,10 @@ delete_server_clicked (GtkWidget * wid, struct session *sess)
 {
    GtkCTreeNode *selected = 0;
 
-   selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
-      GTK_CLIST (sltree)->focus_row));
+   if (treerow == -1)
+      return;
+
+   selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list, treerow));
    if (selected)
    {
       gtk_ctree_unselect (GTK_CTREE (sltree), selected);
@@ -494,8 +497,11 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
    if (temp)
       editnode = temp;
    else
-      editnode = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
-      GTK_CLIST (sltree)->focus_row));
+   {
+      if (treerow == -1)
+         return;
+      editnode = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list, treerow));
+   }
    if (!editnode)
       return;
 
@@ -596,12 +602,14 @@ new_server_clicked (GtkWidget * wid, struct session *sess)
    struct slentry *temp;
    GtkCTreeNode *node;
    GtkCTreeNode *sibling;
-   GtkCTreeNode *parent = 0;
+   GtkCTreeNode *parent;
    gchar *text[] =
    {""};
 
-   parent = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
-      GTK_CLIST (sltree)->focus_row));
+   if (treerow == -1)
+      parent = 0;
+   else
+      parent = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list, treerow));
 
    new = malloc (sizeof (struct slentry));
    new->port = 6667;
@@ -657,8 +665,10 @@ new_group_clicked (GtkWidget * wid, struct session *sess)
    gchar *text[] =
    {""};
 
-   sibling = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
-      GTK_CLIST (sltree)->focus_row));
+   if (treerow == -1)
+      sibling = 0;
+   else
+      sibling = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list, treerow));
 
    new = malloc (sizeof (struct slentry));
    new->port = 0;
@@ -848,8 +858,9 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
    char password[86];
    int port;
 
-   selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
-      GTK_CLIST (sltree)->focus_row));
+   if (treerow == -1)
+      return;
+   selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list, treerow));
    if (!selected)
       return;
 
@@ -898,8 +909,9 @@ connect_clicked (GtkWidget * wid, struct session *sess)
       			    "please change that first.");
       return;
    }
-   selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
-      GTK_CLIST (sltree)->focus_row));
+   if (treerow == -1)
+      return;
+   selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list, treerow));
    if (!selected)
       return;
 
@@ -932,6 +944,8 @@ row_selected (GtkCTree * tree, gint row, gint col, GdkEventButton * even, gpoint
    GtkCTreeNode *selected = 0;
    struct slentry *conn;
 
+   treerow = row;
+
    selected = gtk_ctree_node_nth (GTK_CTREE (sltree), row);
    if (!is_group (selected))
    {
@@ -951,6 +965,7 @@ row_selected (GtkCTree * tree, gint row, gint col, GdkEventButton * even, gpoint
 static void
 row_unselected (GtkCTree * tree, GtkCTreeNode * row, gint col)
 {
+   treerow = -1;
    gtk_widget_set_sensitive (button_connnew, FALSE);
    gtk_widget_set_sensitive (button_conn, FALSE);
    gtk_entry_set_text (GTK_ENTRY (quickview), "");
