@@ -192,7 +192,7 @@ PrintTextRawZvt (GtkWidget * textwidget, unsigned char *text)
                   mirc = colconv[mirc];
                   if (mirc > 9)
                   {
-			   mirc += 50;
+			            mirc += 50;
                      sprintf ((char *) &newtext[j], "%dm", mirc + col);
                   } else
                   {
@@ -257,8 +257,8 @@ PrintTextRawZvt (GtkWidget * textwidget, unsigned char *text)
          }
          break;
       case '\017':             /* reset all */
-         strcpy (&newtext[j], "\033[0m");
-         j = strlen (newtext);
+         strcpy (&newtext[j], "\033[0m\217");
+         j += 5;
          reverse = FALSE;
          bold = FALSE;
          under = FALSE;
