@@ -18,6 +18,7 @@
 
 #include "xchat.h"
 #include "gtkutil.h"
+#include "cfgfiles.h"
 #include <signal.h>
 
 
@@ -78,6 +79,32 @@ url_button_clear (void)
    gtk_clist_clear (GTK_CLIST (urlgrabberlist));
 }
 
+void
+url_autosave(void)
+{
+  FILE *fd;
+  GSList *list;
+  char *buf;
+
+  buf = malloc(1000);
+  snprintf(buf, 1000, "%s/%s", get_xdir(), "url.save");
+
+  fd = fopen(buf, "a");
+  free (buf);
+  if (fd == NULL)
+    return;
+
+  list = url_list;
+
+  while (list)
+    {
+      fprintf(fd, "%s\n", (char *) list->data);
+      list = list->next;
+    }
+  
+  fclose(fd);
+}
+
 void 
 url_save_callback (GtkWidget * b, GtkWidget * d)
 {
@@ -130,7 +157,7 @@ url_opengui ()
 
    if (urlgrabberwindow)
    {
-      gtk_widget_destroy (urlgrabberwindow);
+	  gdk_window_show (urlgrabberwindow->window);
       return;
    }
    urlgrabberwindow = gtkutil_window_new ("X-Chat: URL Grabber", "urlgrabber", 350, 100,
@@ -151,17 +178,14 @@ url_opengui ()
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    gtk_widget_show (hbox);
 
-   gtkutil_button_new ("Clear", 0, 0, url_button_clear, 0, hbox);
-#ifdef USE_GNOME
-   gtkutil_stock_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_FORWARD,
-                         "Lynx", url_button_sendlynx, 0, hbox);
-   gtkutil_stock_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_FORWARD,
-                         "Netscape", url_button_send, 0, hbox);
-#else
-   gtkutil_button_new ("-> Lynx", 0, 0, url_button_sendlynx, 0, hbox);
-   gtkutil_button_new ("-> Netscape", 0, 0, url_button_send, 0, hbox);
-#endif
-   gtkutil_button_new ("Save", 0, 0, url_button_save, 0, hbox);
+   gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_CLEAR,
+		   "Clear", url_button_clear, 0, hbox);
+   gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_FORWARD,
+		   "Lynx", url_button_sendlynx, 0, hbox);
+   gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_FORWARD,
+		   "Netscape", url_button_send, 0, hbox);
+   gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_SAVE,
+		   "Save", url_button_save, 0, hbox);
 
    gtk_widget_show (urlgrabberwindow);
 
