@@ -16,8 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include "voice.xpm"
-#include "op.xpm"
+#include "pixmaps/voice.xpm"
+#include "pixmaps/op.xpm"
 #include "xchat.h"
 #include "util.h"
 #include "userlist.h"
@@ -36,7 +36,7 @@ GdkPixmap *op_pixmap, *voice_pixmap;
 GdkBitmap *op_mask_bmp, *voice_mask_bmp;
 
 int userlist_insertname_sorted (struct session *sess, struct user *newuser, int first);
-int add_name (struct session *sess, char *name);
+int add_name (struct session *sess, char *name, char *hostname);
 int sub_name (struct session *sess, char *name);
 
 
@@ -90,6 +90,20 @@ userlist_shownumbers (struct session *sess)
    gtk_label_set_text (GTK_LABEL (sess->namelistinfo), tbuf);
 }
 
+int
+userlist_add_hostname(struct session *sess, char *nick, char *hostname)
+{
+   struct user *user;
+
+   user = find_name(sess, nick);
+   if(user && !user->hostname)
+   {
+	user->hostname = strdup(hostname);
+	return 1;
+   }
+   return 0;
+}
+
 int 
 nick_cmp (struct user *user1, struct user *user2)
 {
@@ -115,6 +129,8 @@ clear_user_list (struct session *sess)
    while (user)
    {
       next = user->next;
+	if (user->hostname)
+	   free (user->hostname);
       free (user);
       user = next;
    }
@@ -145,6 +161,8 @@ update_entry (struct session *sess, struct user *updateuser)
    gint row;
    int sel = FALSE;
    char nick[64];
+   int voice;
+   char *ip = 0;
 
    strcpy (nick, updateuser->user);
    if (updateuser->op)
@@ -159,8 +177,16 @@ update_entry (struct session *sess, struct user *updateuser)
         (gpointer) updateuser) == row)
          sel = TRUE;
    }
+
+   voice = updateuser->voice;
+   if(updateuser->hostname)
+	ip = strdup(updateuser->hostname);
    sub_name (sess, updateuser->user);
-   row = add_name (sess, nick);
+   row = add_name (sess, nick, ip);
+   if(ip) free(ip);
+
+   updateuser = gtk_clist_get_row_data(GTK_CLIST(sess->namelistgad), row);
+   updateuser->voice = voice;
 
    if (sel)
       gtk_clist_select_row ((GtkCList *) sess->namelistgad, row, 0);
@@ -178,8 +204,8 @@ update_entry (struct session *sess, struct user *updateuser)
    userlist_shownumbers (sess);
 }
 
-void 
-op_name (struct session *sess, char *name)
+void
+ul_op_name (struct session *sess, char *name)
 {
    struct user *user = find_name (sess, name);
    if (user)
@@ -253,6 +279,8 @@ sub_name (struct session *sess, char *name)
             sess->ops--;
          sess->total--;
          userlist_shownumbers (sess);
+	   if (user->hostname)
+		free (user->hostname);
          free (user);
          gtk_clist_remove (GTK_CLIST (sess->namelistgad), row);
          return TRUE;
@@ -347,7 +375,7 @@ userlist_insertname_sorted (struct session *sess, struct user *newuser, int firs
 }
 
 int 
-add_name (struct session *sess, char *name)
+add_name (struct session *sess, char *name, char *hostname)
 {
    int first;
    struct user *user;
@@ -377,6 +405,7 @@ add_name (struct session *sess, char *name)
          name++;
          user->voice = TRUE;
       }
+	if(hostname) user->hostname = strdup(hostname);
       strcpy (user->user, name);
       row = userlist_insertname_sorted (sess, user, first);
 
