@@ -24,8 +24,11 @@
 #include <sys/time.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <dirent.h>
 #include "util.h"
-#include <config.h>
+#ifndef SKIPCONFIG
+#include "../myconfig.h"
+#endif
 #ifdef SOCKS
 #include <socks.h>
 #endif
@@ -246,9 +249,12 @@ buf_get_line (char *ibuf, char **buf, int *position, int len)
 {
    int pos = *position, spos = pos;
 
+   if (pos == len)
+     return 0;
+   
    while (ibuf[pos++] != '\n')
    {
-      if (pos > len)
+      if (pos == len)
          return 0;
    }
    pos--;
@@ -259,6 +265,136 @@ buf_get_line (char *ibuf, char **buf, int *position, int len)
    return 1;
 }
 
+
+/* borrowed from ircII */
+
+int
+match(mask, string)
+     u_char *mask, *string;
+{
+  unsigned char *m = mask, *n = string, *ma = NULL, *na = NULL;
+  int just = 0;
+  unsigned char *pm = NULL, *pn = NULL;
+  int lp_c = 0, count = 0, last_count = 0;
+
+  while(1)
+    {
+
+      switch (*m)
+	{
+	case '*':
+	  goto ultimate_lameness;
+	  break;
+	case '%':
+	  goto ultimate_lameness2;
+	  break;
+	case '?':
+	  m++;
+	  if (!*n++) return 0;
+	  break;
+	case 0:
+	  if (!*n)
+	    return 1;
+	case '\\':
+	  if (((*m=='\\') && (m[1]=='*')) || (m[1]=='?')) m++;
+	default:
+	  if (tolower(*m) != tolower(*n))
+	    return 0;
+	  else
+	    {
+	      count++;
+	      m++;
+	      n++;
+	    }
+	}
+    }
+
+  while(1)
+    {
+
+      switch (*m)
+	{
+	case '*':
+ultimate_lameness:
+	  ma = ++m;
+	  na = n;
+	  just = 1;
+	  last_count = count;
+	  break;
+	case '%':
+ultimate_lameness2:
+	  pm = ++m;
+	  pn = n;
+	  lp_c = count;
+	  if (*n == ' ') pm = NULL;
+	  break;
+	case '?':
+	  m++;
+	  if (!*n++) return 0;
+	  break;
+	case 0:
+	  if (!*n || just)
+	    return 1;
+	case '\\':
+	  if (((*m=='\\') && (m[1]=='*')) || (m[1]=='?')) m++;
+	default:
+	  just = 0;
+	  if (tolower(*m) != tolower(*n))
+	    {
+	      if (!*n) return 0;
+	      if (pm)
+		{
+		  m = pm;
+		  n = ++pn;
+		  count = lp_c;
+		  if (*n == ' ') pm = NULL;
+		}
+	      else
+		if (ma)
+		  {
+		    m = ma;
+		    n = ++na;
+		    if (!*n) return 0;
+		    count = last_count;
+		  }
+		else return 0;
+	    }
+	  else
+	    {
+	      count++;
+	      m++;
+	      n++;
+	    }
+	}
+    }
+}
+
+void
+for_files(char *dirname, char *mask, void callback(char *file))
+{
+   DIR *dir;
+   struct dirent *ent;
+   char buf[256];
+
+   dir = opendir(dirname);
+   if(dir)
+   {
+	while((ent = readdir(dir)))
+	{
+	   if(strcmp(ent->d_name, ".") && strcmp(ent->d_name, ".."))
+	   {
+		if(match(mask, ent->d_name))
+		{
+		   sprintf(buf, "%s/%s", dirname, ent->d_name);
+		   callback(buf);
+		}
+	   }
+	}
+	closedir(dir);
+   }
+}
+
+
 /************************************************************************
  *    This technique was borrowed in part from the source code to 
  *    ircd-hybrid-5.3 to implement case-insensitive string matches which
