@@ -20,6 +20,9 @@
 #define strcasecmp      rfc_casecmp
 #define strncasecmp     rfc_ncasecmp
 
+
+extern char *file_part (char *file);
+extern void for_files(char *dirname, char *mask, void callback(char *file));
 extern int rfc_casecmp (char *, char *);
 extern int rfc_ncasecmp (char *, char *, int);
 extern unsigned char tolowertab[];
