@@ -27,6 +27,8 @@
 #include "cfgfiles.h"
 #include "dcc.h"
 #include "gtkutil.h"
+#include "palette.h"
+#include "ignore.h"
 #ifndef USE_GNOME
 #ifdef USE_IMLIB
 #include <gdk_imlib.h>
@@ -50,8 +52,8 @@ GSList *sess_list = 0;
 GSList *serv_list = 0;
 GSList *dcc_list = 0;
 GSList *url_list = 0;
-GSList *fkey_list = 0;
 GSList *away_list = 0;
+GSList *ignore_list = 0;
 
 GtkStyle *channelwin_style;
 GtkStyle *dialogwin_style;
@@ -83,13 +85,13 @@ void sigchild_handler (int sig);
 
 /* fkeys.c */
 
-extern void fkeys_load (void);
+extern void key_init();
 
 /* maingui.c */
 
 extern void make_non_channel_window (struct session *sess, int state);
 extern void create_window (struct session *sess);
-extern GtkStyle *my_widget_get_style (char *bg_pic, int bg_color);
+extern GtkStyle *my_widget_get_style (char *bg_pic);
 
 
 /* rawlog.c */
@@ -129,7 +131,7 @@ extern void disconnect_server (struct session *sess, int sendquit);
 
 /* serverlist.c */
 
-extern void serverlist_autoconnect (struct session *sess, int aut);
+extern void serverlist_autoconnect (struct session *sess);
 extern void open_server_list (GtkWidget * widd, struct session *sess);
 
 /* userlist.c */
@@ -142,6 +144,7 @@ extern struct user *find_name (struct session *sess, char *name);
 extern void notify_load (void);
 extern void notify_save (void);
 extern int notify_checklist (void);
+extern void notify_cleanup(void);
 
 /* lastlog.c */
 
@@ -168,6 +171,9 @@ extern void end_logging (int fd);
 extern void load_text_events ();
 extern void pevent_dialog_save (char *fn);
 
+/* urlgrab.c */
+extern void url_autosave(void);
+
 /* util.c */
 
 extern int is_data_avail (int sok);
@@ -176,9 +182,74 @@ extern int get_mhz (void);
 extern void menu_about (GtkWidget * wid, gpointer sess);
 
 
-int 
+void auto_reconnect (struct server *serv);
+
+/* anything above SEND_MAX bytes in 1 second is
+   queued and sent QUEUE_TIMEOUT milliseconds later */
+
+#define SEND_MAX 256
+#define QUEUE_TIMEOUT 2500
+
+/* this whole throttling system is lame, can anyone suggest
+   a better system? */
+
+int
+tcp_send_queue (struct server *serv)
+{
+   char *buf;
+   int len;
+   GSList *list = serv->outbound_queue;
+   time_t now = time (0);
+
+   while (list)
+   {
+	buf = (char *) list->data;
+	len = strlen (buf);
+
+	if (serv->last_send != now)
+	{
+	   serv->last_send = now;
+	   serv->bytes_sent = 0;
+	} else {
+	   serv->bytes_sent += len;
+	}
+
+	if (serv->bytes_sent > SEND_MAX)
+	   return 1; /* don't remove the timeout handler */
+
+	add_rawlog (serv, buf, TRUE);
+	send (serv->sok, buf, len, 0);
+
+	serv->outbound_queue = g_slist_remove(serv->outbound_queue, buf);
+	free (buf);
+
+	list = serv->outbound_queue;
+   }
+   return 0; /* remove the timeout handler */
+}
+
+int
 tcp_send_len (struct server *serv, char *buf, int len)
 {
+   time_t now = time (0);
+
+   if (serv->last_send != now)
+   {
+	serv->last_send = now;
+	serv->bytes_sent = 0;
+   } else {
+	serv->bytes_sent += len;
+   }
+
+   if (serv->bytes_sent > SEND_MAX || serv->outbound_queue)
+   {
+	buf = strdup (buf);
+	if (!serv->outbound_queue)
+	   gtk_timeout_add(QUEUE_TIMEOUT, (GtkFunction)tcp_send_queue, serv);
+	serv->outbound_queue = g_slist_append(serv->outbound_queue, buf);
+	return 1;
+   }
+
    add_rawlog (serv, buf, TRUE);
    return send (serv->sok, buf, len, 0);
 }
@@ -197,7 +268,7 @@ find_session_from_channel (char *chan, struct server *serv)
    while (list)
    {
       sess = (struct session *) list->data;
-      if (!strcasecmp (chan, sess->channel))
+      if (!sess->is_shell && !strcasecmp (chan, sess->channel))
       {
          if (!serv)
             return sess;
@@ -215,8 +286,11 @@ find_session_from_nick (char *nick, struct server *serv)
    struct session *sess;
    GSList *list = sess_list;
 
-   if (find_name (serv->front_session, nick))
-      return serv->front_session;
+   if (serv->front_session)
+   {
+	if (find_name (serv->front_session, nick))
+   	   return serv->front_session;
+   }
 
    sess = find_session_from_channel (nick, serv);
    if (sess)
@@ -243,7 +317,7 @@ find_session_from_waitchannel (char *chan, struct server *serv)
    while (list)
    {
       sess = (struct session *) list->data;
-      if (sess->server == serv)
+      if (sess->server == serv && sess->channel[0] == 0 && !sess->is_shell)
       {
          if (!strcasecmp (chan, sess->waitchannel))
             return sess;
@@ -261,19 +335,27 @@ show_generic_channel (struct server *serv, char *chan, char *msg)
       PrintText (sess, msg);
 }
 
-void 
+gint
+timeout_auto_reconnect (struct server *serv)
+{
+   connect_server (serv->front_session, serv->hostname, serv->port, FALSE);
+   return 0; /* returning 0 should remove the timeout handler */
+}
+
+void
 auto_reconnect (struct server *serv)
 {
    struct session *s;
-   GSList *list = sess_list;    /* make sure auto rejoin can work */
+   GSList *list;
 
    if (serv->front_session == NULL)
       return;
 
-   while (list)
+   list = sess_list;
+   while (list) /* make sure auto rejoin can work */
    {
       s = (struct session *) list->data;
-      if (s->channel[0] == '#' || s->channel[0] == '&')
+      if (s->channel[0] == '#' || s->channel[0] == '&' || s->channel[0] == '+')
       {
          strcpy (s->waitchannel, s->channel);
          strcpy (s->willjoinchannel, s->channel);
@@ -281,7 +363,11 @@ auto_reconnect (struct server *serv)
       list = list->next;
    }
    disconnect_server (serv->front_session, FALSE);
-   connect_server (serv->front_session, serv->hostname, serv->port, FALSE);
+   
+   if (prefs.recon_delay)
+	gtk_timeout_add(prefs.recon_delay*1000, (GtkFunction)timeout_auto_reconnect, serv);
+   else
+	connect_server (serv->front_session, serv->hostname, serv->port, FALSE);
 }
 
 void 
@@ -308,14 +394,16 @@ read_data (struct server *serv, gint sok)
       } else
       {
          int i = 0;
+
          lbuf[len] = 0;
 
          while (i < len)
-         {
+	   {
             switch (lbuf[i])
             {
             case '\r':
                break;
+
             case '\n':
                serv->linebuf[serv->pos] = 0;
                if (prefs.stripcolor)
@@ -328,13 +416,16 @@ read_data (struct server *serv, gint sok)
                   process_line (serv->front_session, serv, serv->linebuf);
                }
                serv->pos = 0;
-
                break;
+
             default:
                serv->linebuf[serv->pos] = lbuf[i];
                serv->pos++;
                if (serv->pos == 2047)
+		   {
+			fprintf(stderr, "*** X-Chat: Buffer overflow - shit server!\n");
                   serv->pos = 2046;
+		   }
             }
             i++;
          }
@@ -342,6 +433,23 @@ read_data (struct server *serv, gint sok)
    }
 }
 
+void
+flush_server_queue (struct server *serv)
+{
+   GSList *list = serv->outbound_queue;
+   void *data;
+
+   while (list)
+   {
+	data = list->data;
+	serv->outbound_queue = g_slist_remove(serv->outbound_queue, data);
+	free (data);
+	list = serv->outbound_queue;
+   }
+   serv->last_send = 0;
+   serv->bytes_sent = 0;
+}
+
 gint 
 kill_session_callback (GtkWidget * win, struct session *sess)
 {
@@ -373,14 +481,14 @@ kill_session_callback (GtkWidget * win, struct session *sess)
       free (re);
    }
 #ifdef USE_PERL
-   if (sess->channel[0] == '#' || sess->channel[0] == '&')
+   /*if (sess->channel[0] == '#' || sess->channel[0] == '&' || sess->channel[0] == '+')
    {
       snprintf (tbuf, sizeof tbuf, ":%s KILLSESSION %s %s",
                 serv->nick,
                 sess->channel,
                 serv->servername);
       perl_inbound (sess, serv, tbuf);
-   }
+   }*/
    if (perl_sess == sess)       /* need to find a new perl_sess, this one's closing */
    {
       struct session *s;
@@ -433,7 +541,7 @@ kill_session_callback (GtkWidget * win, struct session *sess)
    } else
    {
       if (sess->server && sess->server->connected && sess->channel[0] &&
-          ((sess->channel[0] == '#') || (sess->channel[0] == '&')))
+          ((sess->channel[0] == '#') || (sess->channel[0] == '&') || (sess->channel[0] == '+')))
       {
          snprintf (tbuf, sizeof tbuf, "PART %s\r\n", sess->channel);
          tcp_send (sess->server, tbuf);
@@ -455,6 +563,7 @@ kill_session_callback (GtkWidget * win, struct session *sess)
       end_logging (sess->logfd);
 
    sess_list = g_slist_remove (sess_list, sess);
+   history_free (&sess->history);
    free (sess);
 
    if (sess_list)
@@ -469,23 +578,34 @@ kill_session_callback (GtkWidget * win, struct session *sess)
       }
       if (serv->connected)
       {
-         gdk_input_remove (serv->iotag);
+	   if(serv->iotag != -1)
+	   {
+      	gdk_input_remove (serv->iotag);
+		serv->iotag = -1;
+	   }
          close (serv->sok);
       }
       if (serv->connecting)
       {
          kill (serv->childpid, SIGKILL);
          waitpid (serv->childpid, NULL, WNOHANG);
-         gdk_input_remove (serv->iotag);
+	   if(serv->iotag != -1)
+      	gdk_input_remove (serv->iotag);
          close (serv->childread);
          close (serv->sok);
       }
       dialog_notify_kill (serv);
       if (serv->rawlog_window)
-         close_rawlog (0, serv);
+	{
+	   /* we don't want the "destroy" callback to be called */
+	   gtk_signal_disconnect_by_data (GTK_OBJECT (serv->rawlog_window), serv);
+	   gtk_widget_destroy (serv->rawlog_window);
+	}
       serv_list = g_slist_remove (serv_list, serv);
       dcc_notify_kill (serv);
+	flush_server_queue (serv);
       free (serv);
+	notify_cleanup ();
       return TRUE;
    }
    if (prefs.tabchannels && main_window)
@@ -503,8 +623,16 @@ new_server (void)
       return 0;
    memset (serv, 0, sizeof (struct server));
    serv->sok = -1;
+   serv->iotag = -1;
    strcpy (serv->nick, prefs.nick1);
    serv_list = g_slist_append (serv_list, serv);
+
+   if (prefs.use_server_tab)
+   {
+	serv->front_session = new_session(serv, 0);
+	serv->front_session->is_server = TRUE;
+   }
+
    return serv;
 }
 
@@ -543,13 +671,12 @@ new_session (struct server *serv, char *from)
    memset (sess, 0, sizeof (struct session));
    sess->server = serv;
    sess->logfd = -1;
-   /*sess->running_exec = NULL; */
    if (from)
    {
       sess->is_dialog = TRUE;
       strcpy (sess->channel, from);
-   }                            /* else
-                                   sess->is_dialog = FALSE; */
+   }
+
    create_window (sess);
    if (!sess->is_dialog)
       make_non_channel_window (sess, FALSE);
@@ -570,7 +697,6 @@ free_sessions (void)
       sess = (struct session *) list->data;
       if (sess->logfd != -1)
          end_logging (sess->logfd);
-      history_free (&sess->history);
       kill_session_callback (0, sess);
       list = sess_list;
    }
@@ -628,22 +754,46 @@ show_init_msg (struct session *sess)
    mhz = get_mhz ();
    if (mhz)
       snprintf (buf, sizeof buf,
-                "X-Chat \00311" VERSION "\003 started on %s"
-                "Running on %s \00311%s\003 \0032[\00311%s\0032/\00311%dMHz\0032]\003 GTK \00311%d.%d.%d\017\n",
+                "\002\00313-\017 X-Chat \00311" VERSION "\003 started on %s"
+                "\002\00313-\017 Running on %s \00311%s\003 \0032[\00311%s\0032/\00311%dMHz\0032]\003 GTK \00311%d.%d.%d\017\n",
                 ctime (&tim), un.sysname, un.release, un.machine, mhz,
                 gtk_major_version, gtk_minor_version, gtk_micro_version);
    else
       snprintf (buf, sizeof buf,
-                "X-Chat \00311" VERSION "\003 started on %s"
-                "Running on %s \00311%s\003 \0032[\00311%s\0032]\003 GTK \00311%d.%d.%d\017\n",
+                "\002\00313-\017 X-Chat \00311" VERSION "\003 started on %s"
+                "\002\00313-\017 Running on %s \00311%s\003 \0032[\00311%s\0032]\003 GTK \00311%d.%d.%d\017\n",
                 ctime (&tim), un.sysname, un.release, un.machine,
                 gtk_major_version, gtk_minor_version, gtk_micro_version);
    PrintText (sess, buf);
+
+   strcpy(buf, "\002\00313-\017 Capabilities: ");
+#ifdef USE_IMLIB
+   strcat(buf, "Imlib ");
+#endif
+#ifdef USE_GNOME
+   strcat(buf, "Gnome Zvt ");
+#endif
+#ifdef USE_PANEL
+   strcat(buf, "Panel ");
+#endif
+#ifdef USE_PERL
+   strcat(buf, "Perl ");
+#endif
+#ifdef USE_PLUGIN
+   strcat(buf, "Plugin ");
+#endif
+#ifdef ENABLE_NLS
+   strcat(buf, "NLS ");
+#endif
+#ifdef SOCKS
+   strcat(buf, "Socks5 ");
+#endif
+   strcat(buf, "\n\n");
+   PrintText (sess, buf);
 }
 
 #define defaultconf_ctcp  "NAME TIME\nCMD /nctcp %s TIME %t\n\n"\
-                          "NAME PING\nCMD /nctcp %s PING %d\n\n"\
-                          "NAME USERINFO\nCMD /notice %s What are you doing Dave?\n"
+                          "NAME PING\nCMD /nctcp %s PING %d\n\n"
 
 #define defaultconf_popup "NAME SUB\nCMD CTCP\n\n"\
                             "NAME Version\nCMD /ctcp %s VERSION\n\n"\
@@ -651,26 +801,39 @@ show_init_msg (struct session *sess)
                             "NAME Clientinfo\nCMD /ctcp %s CLIENTINFO\n\n"\
                             "NAME Ping\nCMD /ping %s\n\n"\
                             "NAME Time\nCMD /ctcp %s TIME\n\n"\
+				    "NAME Finger\nCMD /ctcp %s FINGER\n\n"\
                           "NAME ENDSUB\nCMD \n\n"\
-                          "NAME SEP\nCMD \n\n"\
                           "NAME SUB\nCMD DCC\n\n"\
                             "NAME Send File\nCMD /dcc send %s\n\n"\
                             "NAME Chat\nCMD /dcc chat %s\n\n"\
-                            "NAME Draw\nCMD /dcc draw %s\n\n"\
                           "NAME ENDSUB\nCMD \n\n"\
+                          "NAME SUB\nCMD Oper\n\n"\
+                            "NAME Kill\nCMD /quote KILL %s :die!\n\n"\
+                          "NAME ENDSUB\nCMD \n\n"\
+				  "NAME SUB\nCMD Mode\n\n"\
+                            "NAME Give Voice\nCMD /voice %s\n\n"\
+                            "NAME Take Voice\nCMD /devoice %s\n"\
+                            "NAME SEP\nCMD \n\n"\
+				    "NAME Give Ops\nCMD /op %s\n\n"\
+				    "NAME Take Ops\nCMD /deop %s\n\n"\
+                          "NAME ENDSUB\nCMD \n\n"\
+				  "NAME SEP\nCMD \n\n"\
+				  "NAME Kick\nCMD /kick %s\n\n"\
+				  "NAME Ban\nCMD /ban %s\n\n"\
+				  "NAME Kick&Ban\nCMD /kickban %s\n\n"\
                           "NAME SEP\nCMD \n\n"\
-                          "NAME Give Voice\nCMD /voice %s\n\n"\
-                          "NAME Take Voice\nCMD /devoice %s\n"\
-                          "NAME SEP\nCMD \n\n"\
+				  "NAME Query\nCMD /query %s\n\n"\
+				  "NAME Who\nCMD /quote WHO %s\n\n"\
+				  "NAME Whois\nCMD /quote WHOIS %s\n\n"\
                           "NAME DNS Lookup\nCMD /dns %s\n\n"
 
-#define defaultconf_buttons "NAME Op\nCMD /op %s\n\n"\
-                            "NAME DeOp\nCMD /deop %s\n\n"\
+#define defaultconf_buttons "NAME Op\nCMD /op %a\n\n"\
+                            "NAME DeOp\nCMD /deop %a\n\n"\
                             "NAME Ban\nCMD /ban %s\n\n"\
                             "NAME Kick\nCMD /kick %s\n\n"\
-                            "NAME Send\nCMD /dcc send %s\n\n"\
-                            "NAME Chat\nCMD /chat %s\n\n"\
-                            "NAME Dialog\nCMD /query %s\n\n"\
+			          "NAME Send\nCMD /dcc send %s\n\n"\
+			          "NAME Dialog\nCMD /query %s\n\n"\
+                            "NAME Lookup\nCMD /dns %s\n\n"\
                             "NAME Whois\nCMD /whois %s\n"
 
 #define defaultconf_replace "NAME teh\nCMD the\n\n"\
@@ -726,28 +889,37 @@ xchat_init (int autoconnect)
 #endif
 
    load_config ();
-   fkeys_load ();
    signal_setup ();
    load_text_events ();
+   key_init ();
    notify_load ();
+   palette_load ();
+   ignore_load ();
    list_loadconf ("popup.conf", &popup_list, defaultconf_popup);
    list_loadconf ("ctcpreply.conf", &ctcp_list, defaultconf_ctcp);
    list_loadconf ("buttons.conf", &button_list, defaultconf_buttons);
    list_loadconf ("commands.conf", &command_list, defaultconf_commands);
    list_loadconf ("replace.conf", &replace_list, defaultconf_replace);
 
-   channelwin_style = my_widget_get_style (prefs.background, prefs.bg_color);
-   dialogwin_style = my_widget_get_style (prefs.background_dialog, prefs.dialog_bg_color);
-
    font_normal = my_font_load (prefs.font_normal);
    font_bold = my_font_load (prefs.font_bold);
    dialog_font_normal = my_font_load (prefs.dialog_font_normal);
    dialog_font_bold = my_font_load (prefs.dialog_font_bold);
 
+#ifdef USE_GNOME
+   palette_prepare(FALSE);
+#endif
+
+   channelwin_style = my_widget_get_style (prefs.background);
+   dialogwin_style = my_widget_get_style (prefs.background_dialog);
+
    serv = new_server ();
    if (serv)
    {
-      sess = new_session (serv, 0);
+	if (prefs.use_server_tab)
+	   sess = serv->front_session;
+	else
+   	   sess = new_session (serv, 0);
       init_userlist_xpm (sess);
       show_init_msg (sess);
 
@@ -762,7 +934,7 @@ xchat_init (int autoconnect)
          open_server_list (0, sess);
 
       if (autoconnect)
-         serverlist_autoconnect (sess, autoconnect);
+         serverlist_autoconnect (sess);
 
       if (prefs.notify_timeout)
          notify_tag = gtk_timeout_add (prefs.notify_timeout, (GtkFunction) notify_checklist, 0);
@@ -780,15 +952,21 @@ xchat_cleanup (void)
       save_config ();
       pevent_dialog_save (NULL);
    }
+   
+   if (prefs.autosave_url)
+     url_autosave();
+
    notify_save ();
+   ignore_save ();
+   palette_save ();
    free_sessions ();
-   list_free (&serv_list);
+/*   list_free (&serv_list);
    list_free (&dcc_list);
    list_free (&ctcp_list);
    list_free (&popup_list);
    list_free (&button_list);
    list_free (&command_list);
-   list_free (&replace_list);
+   list_free (&replace_list);*/ /* no point in doing this */
 #ifdef USE_PERL
    perl_end ();
 #endif
@@ -865,7 +1043,7 @@ main (int argc, char *argv[])
 #ifdef USE_GNOME
    struct poptOption options[] =
    {
-      {"connect", 'c', POPT_ARG_INT, 0, 0, "Auto connect", "Entry number"},
+      {"connect", 'c', POPT_ARG_NONE, 0, 0, "Auto connect", 0},
 #ifdef USE_PANEL
       {"no-panel", 'n', POPT_ARG_NONE, 0, 0, "Don't use GNOME Panel", 0},
 #endif
@@ -895,7 +1073,7 @@ main (int argc, char *argv[])
       if (!strcasecmp (argv[1], "-h") || !strcasecmp (argv[1], "--help"))
       {
          puts ("X-Chat " VERSION " Options:\n\n"
-               "   --connect      -c <n>    : auto connect to server entry <n>\n"
+               "   --connect      -c        : auto connect\n"
                "   --version      -v        : show version information\n"
             );
          return 0;
@@ -909,10 +1087,10 @@ main (int argc, char *argv[])
 #endif
 #endif
 
-   if (argc > 2)
+   if (argc > 1)
    {
       if (!strcasecmp (argv[1], "-c") || !strcasecmp (argv[1], "--connect"))
-         autoconnect = atoi (argv[2]);
+         autoconnect = 1;
    }
 #ifdef SOCKS
    SOCKSinit (argv[0]);
