@@ -10,7 +10,10 @@
 #include <limits.h>
 #include "history.h"
 #ifndef SKIPCONFIG
-#include <config.h>
+#include "../myconfig.h"
+#endif
+#ifdef SKIPGNOME
+#undef USE_GNOME
 #endif
 
 #define snprintf g_snprintf
@@ -32,14 +35,33 @@ void free (void *buf);
 #endif
 
 #ifdef USE_GNOME
+
 #include <gnome.h>
 #undef GNOME_APP
 #define GNOME_APP(n) ((GnomeApp*)n)
+
 #else
+
+#include "fake_gnome.h"
 #ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(x) gettext(x)
+#  include <libintl.h>
+#  define _(x) gettext(x)
+#  ifdef gettext_noop
+#    define N_(String) gettext_noop (String)
+#  else
+#    define N_(String) (String)
+#  endif
+#else
+#  define N_(String) (String)
+#  define _(x) (x)
 #endif
+
+#endif /* !USE_GNOME */
+
+#ifdef __EMX__ /* for o/s 2 */
+#define OFLAGS O_BINARY
+#else
+#define OFLAGS 0
 #endif
 
 #undef GTK_WINDOW
@@ -62,6 +84,7 @@ void free (void *buf);
 #undef GTK_TOGGLE_BUTTON
 #undef GTK_NOTEBOOK
 #undef GTK_MENU_ITEM
+#undef GTK_CTREE
 
 #define GTK_WINDOW(n) ((GtkWindow *)n)
 #define GTK_BOX(n) ((GtkBox *)n)
@@ -83,6 +106,7 @@ void free (void *buf);
 #define GTK_TOGGLE_BUTTON(n) ((GtkToggleButton *)n)
 #define GTK_NOTEBOOK(n) ((GtkNotebook*)n)
 #define GTK_MENU_ITEM(n) ((GtkMenuItem*)n)
+#define GTK_CTREE(n) ((GtkCTree*)n)
 
 #define	FONTNAMELEN	127
 
@@ -115,6 +139,9 @@ struct xchatprefs
    char background_dialog[PATH_MAX + 1];
    char dccdir[PATH_MAX + 1];
    char bluestring[64];
+   char dnsprogram[72];
+   char hostname[127];
+   char ipaddress[16];
 
    int bufsize;
    int zvt_lines;
@@ -126,61 +153,68 @@ struct xchatprefs
    int mainwindow_width;
    int mainwindow_height;
    int dccpermissions;
+   int recon_delay;
+   int bantype;
    unsigned char nu_color;
-   unsigned char bg_color;
-   unsigned char fg_color;
-   unsigned char dialog_bg_color;
-   unsigned char dialog_fg_color;
    unsigned char bt_color;
    guint32 my_ip;
+   guint32 local_ip;
+   char local_name[127];
 
-   int nopanel:1;
-   int autosave:1;
-   int autodialog:1;
-   int autoreconnect:1;
-   int invisible:1;
-   int servernotice:1;
-   int wallops:1;
-   int skipmotd:1;
-   int away:1;
-   int autorejoin:1;
-   int colorednicks:1;
-   int nochanmodebuttons:1;
-   int nouserlistbuttons:1;
-   int nickcompletion:1;
-   int tabnicks:1;
-   int tabchannels:1;
-   int nopaned:1;
-   int autodccchat:1;
-   int autodccsend:1;
-   int noautoopendccsendwindow:1;
-   int noautoopendccrecvwindow:1;
-   int zvt:1;
-   int transparent:1;
-   int tint:1;
-   int dialog_zvt:1;
-   int dialog_transparent:1;
-   int dialog_tint:1;
-   int stripcolor:1;
-   int timestamp:1;
-   int fastdccsend:1;
-   int skipserverlist:1;
-   int filterbeep:1;
-   int tabsatbottom:1;
-   int beepmsg:1;
-   int privmsgtab:1;
-   int logging:1;
-   int newtabstofront:1;
-   int dccwithnick:1;
-   int hilitenotify:1;
-   int hidever:1;
-   int get_ip_from_server:1;
-   int panelize_hide:1;
-   int panel_vbox:1;
-   int raw_modes:1;
-   int no_server_logs:1;
-   int show_away_once:1;
-   int emacs_key_bindings:1;
+   unsigned int nopanel;
+   unsigned int autosave;
+   unsigned int autodialog;
+   unsigned int autosave_url;
+   unsigned int autoreconnect;
+   unsigned int invisible;
+   unsigned int servernotice;
+   unsigned int wallops;
+   unsigned int skipmotd;
+   unsigned int away;
+   unsigned int autorejoin;
+   unsigned int colorednicks;
+   unsigned int nochanmodebuttons;
+   unsigned int nouserlistbuttons;
+   unsigned int nickcompletion;
+   unsigned int tabnicks;
+   unsigned int tabchannels;
+   unsigned int nopaned;
+   unsigned int autodccchat;
+   unsigned int autodccsend;
+   unsigned int autoresume;
+   unsigned int noautoopendccsendwindow;
+   unsigned int noautoopendccrecvwindow;
+   unsigned int noautoopendccchatwindow;
+   unsigned int zvt;
+   unsigned int transparent;
+   unsigned int tint;
+   unsigned int dialog_zvt;
+   unsigned int dialog_transparent;
+   unsigned int dialog_tint;
+   unsigned int stripcolor;
+   unsigned int timestamp;
+   unsigned int fastdccsend;
+   unsigned int skipserverlist;
+   unsigned int filterbeep;
+   unsigned int tabsatbottom;
+   unsigned int beepmsg;
+   unsigned int privmsgtab;
+   unsigned int logging;
+   unsigned int newtabstofront;
+   unsigned int dccwithnick;
+   unsigned int hilitenotify;
+   unsigned int hidever;
+   unsigned int ip_from_server;
+   unsigned int detecthost;
+   unsigned int detectip;
+   unsigned int panelize_hide;
+   unsigned int panel_vbox;
+   unsigned int raw_modes;
+   unsigned int no_server_logs;
+   unsigned int show_away_once;
+   unsigned int show_away_message;
+   unsigned int nouserhost;
+   unsigned int use_server_tab;
 };
 
 struct setup
@@ -189,13 +223,9 @@ struct setup
    GtkWidget *nu_color;
    GtkWidget *bt_color;
 
-   GtkWidget *bg_color;
-   GtkWidget *fg_color;
    GtkWidget *font_normal;
    GtkWidget *font_bold;
 
-   GtkWidget *dialog_bg_color;
-   GtkWidget *dialog_fg_color;
    GtkWidget *dialog_font_normal;
    GtkWidget *dialog_font_bold;
 
@@ -210,6 +240,8 @@ struct setup
    GtkWidget *entry_dcctimeout;
    GtkWidget *entry_dccstalltimeout;
    GtkWidget *entry_permissions;
+   GtkWidget *entry_dnsprogram;
+   GtkWidget *entry_recon_delay;
 
    GtkWidget *entry_sounddir;
    GtkWidget *entry_soundcmd;
@@ -219,11 +251,17 @@ struct setup
    GtkWidget *entry_mainw_width;
    GtkWidget *entry_mainw_height;
 
+   GtkWidget *entry_hostname;
+   GtkWidget *entry_ipaddress;
+
    GtkWidget *background;
    GtkWidget *background_dialog;
 
    GtkWidget *check_transparent;
    GtkWidget *check_tint;
+   GtkWidget *check_detecthost;
+   GtkWidget *check_detectip;
+   GtkWidget *check_ip;
 
    GtkWidget *dialog_check_transparent;
    GtkWidget *dialog_check_tint;
@@ -264,6 +302,7 @@ struct session
    GtkWidget *button_box;
    GtkWidget *bar;
    GtkWidget *leftpane;
+   GtkWidget *confbutton;       /* conference mode button */
    int userlisthidden;
 
    GtkWidget *flag_wid[8];
@@ -295,8 +334,10 @@ struct session
    struct setup *setup;
    struct nbexec *running_exec;
 
+   int is_shell:1;
    int is_tab:1;                /* is this a tab or toplevel window? */
    int is_dialog:1;
+   int is_server:1;		  /* for use_server_tab feature */
    int new_data:1;              /* new data avail? (red tab) */
    int nick_said:1;             /* your nick mentioned? (green tab) */
    int zvt:1;                   /* using zvt instead of gtk_text? */
@@ -304,6 +345,7 @@ struct session
    int ignore_mode:1;
    int ignore_names:1;
    int end_of_names:1;
+   int doing_who:1;             /* /who sent on this channel */
 };
 
 struct server
@@ -314,30 +356,37 @@ struct server
    int childpid;
    gint iotag;
    gint bartag;
-   char hostname[128];
-   char servername[128];
+   char hostname[128];          /* real ip number */
+   char servername[128];        /* what the server says is its name */
    char password[86];
    char nick[64];
-   char linebuf[1024];
+   char linebuf[2048];
    long pos;
    int nickcount;
 
+   GSList *outbound_queue;
+   time_t last_send;
+   int bytes_sent;              /* this second only */
+
    struct session *front_session;
 
    GtkWidget *rawlog_window;
    GtkWidget *rawlog_textlist;
 
+   GtkWidget *chanlist_wild;
    GtkWidget *chanlist_window;
    GtkWidget *chanlist_list;
+   GtkWidget *chanlist_refresh;
    int chanlist_minusers;
 
-   int doing_dns:1;
    int motd_skipped:1;
    int connected:1;
    int connecting:1;
    int no_login:1;
-   int skip_next_who:1;
+   int skip_next_who:1;         /* used for "get my ip from server" */
    int inside_whois:1;
+   int doing_who:1;             /* /dns has been done */
+   int end_of_motd:1;		/* end of motd reached (logged in) */
 };
 
 struct DCC
@@ -347,6 +396,7 @@ struct DCC
    char stat;                   /* 0 = QUEUED  1 = ACTIVE  2 = FAILED  3 = DONE */
    long sok;
    gint iotag;
+   gint wiotag;
    unsigned long addr;
    unsigned long size;
    unsigned long resumable;
@@ -359,6 +409,7 @@ struct DCC
    unsigned long lasttime;
    long port;
    long cps;
+   gfloat perc;
    char file[128];
    char destfile[PATH_MAX + 1];
    char nick[64];
@@ -366,6 +417,7 @@ struct DCC
    struct dccdraw *dccdraw;
    struct dcc_chat *dccchat;
    struct sockaddr_in SAddr;
+   int fastsend:1;
 };
 
 typedef int (*cmd_callback) (struct session * sess, char *tbuf, char *word[], char *word_eol[]);
